@@ -34,27 +34,23 @@
 
 class Animated : public GameHook {
  public:
-  Animated();
+  explicit Animated(int role, int maxVBOpairsOrVAOs);
 
   virtual ~Animated();
   /** Setup drawing pass of object.*/
 
-  /** new[] and glGenBuffer the index and data buffers */
-  void allocateBuffers(int N, GLuint*& idxbufs, GLuint*& databufs);
   /** Generate all buffers possibly used in this tick */
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs) = 0;
+  virtual void generateBuffers(const GLuint* idxbufs, const GLuint* databufs,
+                               const GLuint* vaolist, bool mustUpdate) const = 0;
   /** First drawing pass of object. Render opaque buffers. */
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs) = 0;
+  virtual void drawBuffers1(const GLuint* vaolist) const = 0;
   /** Draws the second pass of object. Render alpha buffers if needed. */
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs) = 0;
+  virtual void drawBuffers2(const GLuint* vaolist) const = 0;
 
   /* Object drawing passes */
   void draw();
   void draw2();
-  void drawBoundingBox();
-  /** Recomputes the bounding box of the object. Needed after changes in size */
-  virtual void computeBoundingBox();
-  void onRemove();
+  void drawBoundingBox() const;
 
   virtual void tick(Real dt);
   virtual void die(int how);
@@ -62,18 +58,12 @@ class Animated : public GameHook {
   /** General purpose field. Semantics defined by children */
   int flags;
 
-  /** Tries to make the current position reasonable. Eg. set
-      z-position to height of map */
-  virtual void has_moved();
-
-  /** Computed by game::draw if the object appears roughly on the screen or not. */
-  int onScreen;
   /** Center position of object */
   Coord3d position;
   /** Lower/higher coordinate of boundingbox relative to position */
   Coord3d boundingBox[2];
 
-  GLfloat primaryColor[4], secondaryColor[4], specularColor[4];
+  Color primaryColor, secondaryColor, specularColor;
   GLuint texture;
 
   /** Number of points the player is awarded when this object dies */
@@ -81,11 +71,18 @@ class Animated : public GameHook {
   /** Time modification player is awarded when this object dies */
   double timeOnDeath;
 
+  /** Computed by game::draw if the object appears roughly on the screen or not. */
+  bool onScreen;
+  /** Set by guile interface if object parameters have been altered; used to
+   *  update otherwise static objects */
+  bool drawChanged;
+
  private:
+  void setupVBOs();
+
   int lastFrameNumber;
-  GLuint* idxVBOs;
-  GLuint* dataVBOs;
-  GLuint nVBOs;
+  GLuint* glidxs;
+  const GLuint nVOs;
 };
 
 #endif
