@@ -23,6 +23,8 @@
 /* For std::sort, std::nth_element */
 #include <algorithm>
 
+#include <set>
+
 static int AC_DEBUG = 0;
 static int SANITY_CHECK = 0;
 
@@ -59,6 +61,10 @@ class SingleCycleAllocator {
     delete[] data;
     delete[] istack;
   }
+  void reset() {
+    index = 0;
+    stackdex = 0;
+  }
   void* allocate(size_t bytes) {
     char* ret = &data[index];
     index += bytes;
@@ -69,7 +75,7 @@ class SingleCycleAllocator {
   T* allocateFor() {
     return (T*)allocate(sizeof(T));
   }
-  size_t used() { return index; }
+  size_t used() const { return index; }
 
   void* stackPush(size_t bytes) {
     size_t nints = bytes / sizeof(int) + 1;
@@ -82,6 +88,8 @@ class SingleCycleAllocator {
     return r;
   }
   void stackPop() { stackdex -= istack[stackdex - 1] + 1; }
+  size_t maxStackSize() const { return stacklen; }
+  size_t maxSize() const { return maxlen; }
 
  private:
   char* data;
@@ -775,99 +783,158 @@ int OverlapTree<D>::rightEndsRightOf(Real lbound, void** acc, Real low[D],
   return nfound;
 }
 
-AnimatedCollection::AnimatedCollection()
-    : store(), map(NULL), alloc(NULL), retlist(NULL), ntot(0) {}
-
-AnimatedCollection::~AnimatedCollection() {
-  if (alloc) delete (SingleCycleAllocator*)alloc;
-  if (retlist) delete[] retlist;
-}
-
-void AnimatedCollection::insert(Animated* a) { store.insert(a); }
-
-void AnimatedCollection::remove(Animated* a) { store.erase(a); }
-
-static struct Rectangle<3>* staticinput = NULL;
-
 static struct Rectangle<3> rectFromAnim(const Animated* a) {
   struct Rectangle<3> r;
   r.tag = (void*)a;
-  /* Reverse dimension order to avoid Z-clustering */
-  r.lower[2] = a->position[0] + a->boundingBox[0][0];
-  r.upper[2] = a->position[0] + a->boundingBox[1][0];
-  r.lower[1] = a->position[1] + a->boundingBox[0][1];
-  r.upper[1] = a->position[1] + a->boundingBox[1][1];
-  r.lower[0] = a->position[2] + a->boundingBox[0][2];
-  r.upper[0] = a->position[2] + a->boundingBox[1][2];
-
+  for (int k = 0; k < 3; k++) {
+    r.lower[k] = a->position[k] + a->boundingBox[0][k];
+    r.upper[k] = a->position[k] + a->boundingBox[1][k];
+  }
   for (int i = 0; i < 3; i++) {
     if (r.lower[i] > r.upper[i] || !std::isfinite(r.lower[i]) || !std::isfinite(r.upper[i])) {
       warning("Invalid bounding box");
-      r.lower[0] = 0.;
-      r.lower[1] = 0.;
-      r.lower[2] = 0.;
-      r.upper[0] = 0.;
-      r.upper[1] = 0.;
-      r.upper[2] = 0.;
+      for (int k = 0; k < 3; k++) {
+        r.lower[k] = 0.;
+        r.upper[k] = 0.;
+      }
       return r;
     }
   }
   return r;
 }
 
-void AnimatedCollection::recalculateBboxMap() {
-  if (alloc) {
-    delete (SingleCycleAllocator*)alloc;
-    alloc = NULL;
-    map = NULL;
+static struct Rectangle<3>
+rectFromBounds(const double lower[3], const double upper[3]) {
+  struct Rectangle<3> r;
+  r.tag = NULL;
+  for (int i = 0; i < 3; i++) {
+    r.lower[i] = lower[i];
+    r.upper[i] = upper[i];
   }
+  for (int i = 0; i < 3; i++) {
+    if (r.lower[i] > r.upper[i] || !std::isfinite(r.lower[i]) || !std::isfinite(r.upper[i])) {
+      warning("Invalid bounding box");
+      for (int k = 0; k < 3; k++) {
+        r.lower[k] = 0.;
+        r.upper[k] = 0.;
+      }
+      return r;
+    }
+  }
+  return r;
+}
 
-  ntot = store.size();
-  struct Rectangle<3>* input = new struct Rectangle<3>[ntot];
-  int* rect_indices = new int[ntot];
-  std::set<Animated*>::iterator iter = store.begin();
-  std::set<Animated*>::iterator end = store.end();
-  for (int i = 0; iter != end; iter++, i++) {
-    struct Rectangle<3> r = rectFromAnim(*iter);
-    input[i] = r;
-    rect_indices[i] = i;
+static struct Rectangle<3>
+reverseAxisOrder(const struct Rectangle<3>& s) {
+  struct Rectangle<3> r;
+  r.tag = s.tag;
+  for (int i = 0; i < 3; i++) {
+    r.lower[i] = s.lower[2 - i];
+    r.upper[i] = s.upper[2 - i];
   }
-  if (SANITY_CHECK) {
-    if (staticinput) delete[] staticinput;
-    staticinput = new struct Rectangle<3>[ntot];
-    memcpy(staticinput, input, sizeof(struct Rectangle<3>) * ntot);
+  return r;
+}
+
+AnimatedCollection::AnimatedCollection()
+    : map(NULL),
+      alloc(NULL),
+      retlist(NULL),
+      ntot(0),
+      nreserved(0),
+      rect_indices(NULL),
+      input(NULL) {}
+
+AnimatedCollection::~AnimatedCollection() {
+  if (alloc) delete (SingleCycleAllocator*)alloc;
+  if (retlist) delete[] retlist;
+  if (rect_indices) delete[] rect_indices;
+  if (input) delete[](struct Rectangle<3>*) input;
+}
+
+void AnimatedCollection::add(Animated* a) {
+  struct Rectangle<3> r = rectFromAnim(a);
+  r = reverseAxisOrder(r);
+  if (ntot >= nreserved) { reserve(std::max(2 * ntot, 16)); }
+  ((struct Rectangle<3>*)input)[ntot] = r;
+  ntot++;
+}
+
+void AnimatedCollection::reserve(int N) {
+  if (N <= nreserved) return;
+  nreserved = N;
+
+  struct Rectangle<3>* ninput = new struct Rectangle<3>[nreserved];
+  int* nrect_indices = new int[nreserved];
+  for (int i = 0; i < nreserved; i++) { nrect_indices[i] = i; }
+  for (int i = 0; i < ntot; i++) { ninput[i] = ((struct Rectangle<3>*)input)[i]; }
+
+  if (input) delete[](struct Rectangle<3>*) input;
+  if (rect_indices) delete[] rect_indices;
+  if (retlist) delete[] retlist;
+  input = (void*)ninput;
+  rect_indices = nrect_indices;
+  retlist = new void*[nreserved];
+}
+
+void AnimatedCollection::clear() {
+  ntot = 0;
+  nreserved = 0;
+  if (input) delete[](struct Rectangle<3>*) input;
+  if (rect_indices) delete[] rect_indices;
+  if (retlist) delete[] retlist;
+  retlist = NULL;
+  input = NULL;
+  rect_indices = NULL;
+}
+
+void AnimatedCollection::recalculateBboxMap() {
+  if (ntot == 0) {
+    map = NULL;
+    return;
   }
 
   /* For < 200 entities, maxmem < 5MB, and in practice ~200Kb is used */
   size_t maxmem = DFoldRectangleTree<3>::maxMemoryUse(ntot);
   size_t nstack = (ntot + 1) * (sizeof(int) * 3 + sizeof(Real) * 2);
-  alloc = new SingleCycleAllocator(maxmem, nstack);
+
+  /* Reset memory */
+  if (alloc) {
+    size_t am = ((SingleCycleAllocator*)alloc)->maxSize();
+    size_t ns = ((SingleCycleAllocator*)alloc)->maxStackSize();
+    if (maxmem > am || am > 2 * maxmem || nstack > ns || ns > 2 * nstack) {
+      delete (SingleCycleAllocator*)alloc;
+      alloc = new SingleCycleAllocator(3 * maxmem / 2, 3 * nstack / 2);
+    } else {
+      ((SingleCycleAllocator*)alloc)->reset();
+    }
+  } else {
+    alloc = new SingleCycleAllocator(3 * maxmem / 2, 3 * nstack / 2);
+  }
+
+  /* Build map */
   map = ((SingleCycleAllocator*)alloc)->allocate(sizeof(DFoldRectangleTree<3>));
-  new (map) DFoldRectangleTree<3>(input, rect_indices, ntot, (SingleCycleAllocator*)alloc);
+  new (map) DFoldRectangleTree<3>((struct Rectangle<3>*)input, rect_indices, ntot,
+                                  (SingleCycleAllocator*)alloc);
   size_t usage = ((SingleCycleAllocator*)alloc)->used();
   (void)usage;
-  delete[] input;
-  delete[] rect_indices;
-
-  if (retlist) {
-    delete[] retlist;
-    retlist = NULL;
-  }
-  retlist = new void*[ntot];
 }
 
-std::set<Animated*> AnimatedCollection::bboxOverlapsWith(const Animated* a) const {
+size_t AnimatedCollection::bboxOverlapsWith(const Animated* a, Animated*** ret) const {
+  struct Rectangle<3> r = rectFromAnim(a);
+  return bboxOverlapsWith(r.lower, r.upper, ret);
+}
+size_t AnimatedCollection::bboxOverlapsWith(const double lower[3], const double upper[3],
+                                            Animated*** ret) const {
   if (!map) {
-    warning("Map not yet created");
-    return std::set<Animated*>();
+    /* Either map has not been created or there are no rectangles */
+    return 0;
   }
   DFoldRectangleTree<3>* dmap = (DFoldRectangleTree<3>*)map;
-  struct Rectangle<3> r = rectFromAnim(a);
+  struct Rectangle<3> r = rectFromBounds(lower, upper);
+  r = reverseAxisOrder(r);
 
   int nfound = dmap->intersect(retlist, r.lower, r.upper);
 
-  std::set<Animated*> ret;
-  for (int i = 0; i < nfound; i++) { ret.insert((Animated*)retlist[i]); }
   if (SANITY_CHECK) {
     /* Sanity check via O(n^2) algorithm */
 
@@ -875,7 +942,7 @@ std::set<Animated*> AnimatedCollection::bboxOverlapsWith(const Animated* a) cons
     std::set<Animated*> coll;
     int nco = 0;
     for (int l = 0; l < ntot; l++) {
-      struct Rectangle<3> s = staticinput[l];
+      struct Rectangle<3> s = ((struct Rectangle<3>*)input)[l];
 
       int nover = 0;
       for (int i = 0; i < 3; i++) {
@@ -883,22 +950,25 @@ std::set<Animated*> AnimatedCollection::bboxOverlapsWith(const Animated* a) cons
       }
       if (nover == 3) {
         coll.insert((Animated*)s.tag);
-        if (ret.count((Animated*)s.tag)) { nco++; }
+        for (int k = 0; k < nfound; k++) {
+          if (retlist[k] == (Animated*)s.tag) {
+            nco++;
+            break;
+          }
+        }
       }
     }
 
     static int psu = 0;
-    if (coll.size() != nco || coll.size() != ret.size() | ret.size() != nfound) {
-      warning("Interval tree %d error %d -> %d (ideal %d (%d present)), %d prior passed", ntot,
-              nfound, ret.size(), coll.size(), nco, psu);
+    if (coll.size() != nco || coll.size() != nfound) {
+      warning("Interval tree %d error %d (ideal %d (%d present)), %d prior passed", ntot,
+              nfound, coll.size(), nco, psu);
       psu = 0;
     } else {
       psu++;
     }
-    return coll;
   }
 
-  return ret;
+  *ret = (Animated**)retlist;
+  return nfound;
 }
-
-const std::set<Animated*>& AnimatedCollection::asSet() const { return store; }
