@@ -42,24 +42,24 @@
 
 #define PHYSICS_RESOLUTION 0.002
 
-#include <set>
 #include "animated.h"
 
+class Cell;
+
 /** This class is an ABSTRACT class from which specific balls
     (player/opponents/babies) are inherited */
 class Ball : public Animated {
  public:
-  Ball();
+  explicit Ball(int role);
   virtual ~Ball();
   bool physics(Real time);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(Real);
   void doExpensiveComputations();
-  void onRemove();
   void setReflectivity(double reflectivity, int metallic);
   /* When we crash into something. Returns if we are still alive */
   virtual bool crash(Real speed);
@@ -67,10 +67,8 @@ class Ball : public Animated {
   void drive(Real xacc, Real yacc);
   void queueJump(Real strength);
 
-  static void init();
-
   Coord3d velocity;
-  int inTheAir, inPipe;
+  bool inTheAir, inPipe;
   float modTimeLeft[NUM_MODS], modTimePhaseIn[NUM_MODS];
   float rotation[2];
   double nextJumpStrength;
@@ -81,7 +79,7 @@ class Ball : public Animated {
   Real radius, realRadius;
   Real friction, gravity, bounceFactor;
   Real crashTolerance;
-  int no_physics;
+  bool no_physics;
   /** 0 for low resolution balls (eg. debris), 1 for normal balls, 2
        for high res balls (eg. player) */
   int ballResolution;
@@ -100,18 +98,14 @@ class Ball : public Animated {
   double reflectivity;
   int metallic, dontReflectSelf;
 
-  static GLfloat dizzyTexCoords[4];
-
-  int locateContactPoints(class Map*, class Cell**, Coord3d*, Coord3d*, ICoord2d*, double*,
-                          double*);
-  int locateWallBounces(class Map*, Coord3d*);
+  int locateContactPoints(Map*, Cell**, Coord3d*, Coord3d*, ICoord2d*, double*, double*);
+  int locateWallBounces(Map*, Coord3d*);
   void generateSandDebris();
-  void generateDebris(GLfloat color[4]);
+  void generateDebris(const Color& color);
   void handleBallCollisions();
   void handleForcefieldCollisions();
   void handlePipes(Real time);
-  bool handleGround(class Map*, class Cell**, Coord3d*, Coord3d*, ICoord2d*, double*, int,
-                    Real);
+  bool handleGround(Map*, Cell**, Coord3d*, Coord3d*, ICoord2d*, double*, int, Real);
   bool handleWalls(Coord3d*, int);
 };
 
