@@ -28,7 +28,8 @@
 #include "map.h"
 #include "sound.h"
 
-Bird::Bird(Real x, Real y, Real dx, Real dy, Real size, Real speed) {
+Bird::Bird(Real x, Real y, Real dx, Real dy, Real size, Real speed)
+    : Animated(Role_OtherAnimated, 1) {
   this->x = x;
   this->y = y;
   this->dx = dx - x;
@@ -39,12 +40,8 @@ Bird::Bird(Real x, Real y, Real dx, Real dy, Real size, Real speed) {
   position[0] = x;
   position[1] = y;
   position[2] = Game::current->map->getHeight(position[0], position[1]) + .5;
-  primaryColor[0] = 1.;
-  primaryColor[1] = 1.;
-  primaryColor[2] = 1.;
-  secondaryColor[0] = .6;
-  secondaryColor[1] = 0.8;
-  secondaryColor[2] = 0.9;
+  primaryColor = Color(1., 1., 1., 1.);
+  secondaryColor = Color(0.6, 0.8, 0.9, 1.);
   animation = 0.0;
   rotation = M_PI - atan2(this->dx / lng, this->dy / lng);
   scoreOnDeath = Game::defaultScores[SCORE_BIRD][0];
@@ -57,14 +54,11 @@ Bird::Bird(Real x, Real y, Real dx, Real dy, Real size, Real speed) {
   boundingBox[1][2] = size;
 }
 
-int Bird::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  if (hide > 0.) return 0;
+void Bird::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                           const GLuint *vaolist, bool /*mustUpdate*/) const {
+  if (hide > 0.) return;
 
-  allocateBuffers(1, idxbufs, databufs);
-
-  GLfloat color[4];
-  for (int i = 0; i < 3; i++) color[i] = primaryColor[i];
-  color[3] = 1.0;
+  Color color = primaryColor.toOpaque();
 
   GLfloat data[4 * 8];
   char *pos = (char *)data;
@@ -92,33 +86,30 @@ int Bird::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
   pos += packObjectVertex(pos, position[0] + loc[3][0], position[1] + loc[3][1],
                           position[2] + dz, 0., 1., color, flat);
 
+  ushort idxs[2][3] = {{0, 1, 2}, {0, 3, 1}};
+
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 4 * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
-
-  ushort idxs[2][3] = {{0, 1, 2}, {0, 3, 1}};
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, 6 * sizeof(ushort), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void Bird::drawBuffers1(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void Bird::drawBuffers1(const GLuint * /*vaolist*/) const {}
 
-void Bird::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
+void Bird::drawBuffers2(const GLuint *vaolist) const {
   if (hide > 0.) return;
 
   glDisable(GL_CULL_FACE);
   glEnable(GL_BLEND);
 
-  setActiveProgramAndUniforms(shaderObject);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("wings.png")]);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0],
-              specularColor[1], specularColor[2], 1.);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 10.f / 128.f);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 10.f / 128.f, Lighting_Regular);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindTexture(GL_TEXTURE_2D, textureWings);
+
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, (void *)0);
 }
 
@@ -140,25 +131,18 @@ void Bird::tick(Real t) {
     position[2] = Game::current->map->getHeight(position[0], position[1]) + .5;
 
   // check for collisions with balls
-  Ball *ball;
-  Coord3d diff;
-  const std::set<Animated *> &balls = Game::current->balls->bboxOverlapsWith(this);
-  std::set<Animated *>::iterator iter = balls.begin();
-  std::set<Animated *>::iterator end = balls.end();
-
-  for (; iter != end; iter++) {
-    ball = (Ball *)*iter;
-    if (!ball->alive) continue;
+  Animated **balls;
+  int nballs = Game::current->balls->bboxOverlapsWith(this, &balls);
+  for (int i = 0; i < nballs; i++) {
+    Ball *ball = (Ball *)balls[i];
     if (ball->no_physics) continue;
 
-    diff = position - ball->position;
+    Coord3d diff = position - ball->position;
     // TODO: improve the collision detection
     if (length(diff) < ball->radius + size * .75) {
       if (ball->modTimeLeft[MOD_SPIKE]) {
         // the ball kills the bird !!!
         Animated::die(DIE_OTHER);
-        /*position[2] += 1.2;
-          new ScoreSign(25,position);*/
 
         playEffect(SFX_BIRD_DIE);
         // restart the bird
@@ -176,10 +160,8 @@ void Bird::tick(Real t) {
   }
 
   // check if the bird is outside its path
-  Real cdx, cdy;
-
-  cdx = position[0] - x;
-  cdy = position[1] - y;
+  Real cdx = position[0] - x;
+  Real cdy = position[1] - y;
   if (((dx < 0.) && (cdx <= dx)) || ((dx > 0.) && (cdx >= dx)) || ((dy < 0.) && (cdy <= dy)) ||
       ((dy > 0.) && (cdy >= dy))) {
     // restart
