@@ -26,7 +26,7 @@
 #include "player.h"
 #include "sound.h"
 
-Black::Black(Real x, Real y) : Ball() {
+Black::Black(Real x, Real y) : Ball(Role_Ball) {
   realRadius = 0.4;
   radius = realRadius;
   ballResolution = BALL_HIRES;
@@ -36,12 +36,8 @@ Black::Black(Real x, Real y) : Ball() {
   crashTolerance = 7;
 
   /* Set color to black */
-  primaryColor[0] = 0.0;
-  primaryColor[1] = 0.0;
-  primaryColor[2] = 0.2;
-  specularColor[0] = 1.0;
-  specularColor[1] = 1.0;
-  specularColor[2] = 1.0;
+  primaryColor = Color(0.05f, 0.05f, 0.2f, 1.f);
+  specularColor = Color(0.5f, 0.5f, 0.5f, 1.f);
 
   bounceFactor = .8;
   horizon = 5.0;
@@ -55,39 +51,30 @@ Black::Black(Real x, Real y) : Ball() {
 void Black::die(int how) {
   Ball::die(how);
 
-  if (alive) {
-    Coord3d pos, vel;
-    for (int i = 0; i < 4; i++)
-      for (int j = 0; j < 4; j++) {
-        Real a = i / 4.0 * M_PI2;
-        Real b = (j + 0.5) / 4.0 * M_PI;
-        pos[0] = position[0] + cos(a) * 0.25 * sin(b);
-        pos[1] = position[1] + sin(a) * 0.25 * sin(b);
-        pos[2] = position[2] + 0.25 * cos(b) + 0.5;
-        vel[0] = velocity[0] + 0.5 * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        vel[1] = velocity[1] + 0.5 * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        vel[2] = velocity[2] + 0.5 * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        new Debris(this, pos, vel, 2.0 + 8.0 * frandom());
-      }
+  Coord3d pos, vel;
+  for (int i = 0; i < 4; i++)
+    for (int j = 0; j < 4; j++) {
+      Real a = i / 4.0 * M_PI2;
+      Real b = (j + 0.5) / 4.0 * M_PI;
+      pos[0] = position[0] + cos(a) * 0.25 * sin(b);
+      pos[1] = position[1] + sin(a) * 0.25 * sin(b);
+      pos[2] = position[2] + 0.25 * cos(b) + 0.5;
+      vel[0] = velocity[0] + 0.5 * 1 / 2048.0 * ((rand() % 2048) - 1024);
+      vel[1] = velocity[1] + 0.5 * 1 / 2048.0 * ((rand() % 2048) - 1024);
+      vel[2] = velocity[2] + 0.5 * 1 / 2048.0 * ((rand() % 2048) - 1024);
+      Game::current->add(new Debris(this, pos, vel, 2.0 + 8.0 * frandom()));
+    }
 
-    /*    position[2] += 1.0;
-          new ScoreSign(100,position);*/
-    remove();
-    if (how == DIE_CRASH)
-      playEffect(SFX_BLACK_DIE);
-    else if (how == DIE_FF)
-      playEffect(SFX_FF_DEATH);
-  } else {
-    /* This should not happen */
-  }
+  remove();
+  if (how == DIE_CRASH)
+    playEffect(SFX_BLACK_DIE);
+  else if (how == DIE_FF)
+    playEffect(SFX_FF_DEATH);
 }
 void Black::tick(Real t) {
-  Coord3d v;
-  double dist;
-
   if (Game::current->player1->playing && is_on) {
     Coord3d v = Game::current->player1->position - position;
-    dist = length(v);
+    double dist = length(v);
 
     double d = Game::current->map->getHeight(position[0] + velocity[0] * 1.0,
                                              position[1] + velocity[1] * 1.0);
@@ -111,6 +98,8 @@ void Black::tick(Real t) {
       double vsc = length(v) > 0 ? 1. / length(v) : 0.;
       v = v * vsc;
       Ball::drive(v[0] * likesPlayer, v[1] * likesPlayer);
+    } else {
+      Ball::drive(0., 0.);
     }
   } else {
     Ball::drive(0., 0.);
@@ -118,5 +107,4 @@ void Black::tick(Real t) {
 
   Ball::tick(t);
 }
-void Black::draw() { Ball::draw(); }
 Black::~Black() {}
