@@ -28,17 +28,16 @@ class Cactus : public Animated {
  public:
   Cactus(Real x, Real y, Real radius);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(Real t);
 
  protected:
-  Real radius;
-  int killed;
   Real base_radius;
   Real killed_time;
+  bool killed;
 };
 
 #endif
