@@ -24,14 +24,10 @@
 
 #include "animated.h"
 
-class ColorModifier : public Animated {
+class ColorModifier : public GameHook {
  public:
   ColorModifier(int col, int x, int y, Real min, Real max, Real freq, Real phase);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
-
   void tick(Real t);
 
  protected:
