@@ -25,10 +25,9 @@
 
 class Debris : public Ball {
  public:
-  Debris(Ball*, Coord3d pos, Coord3d vel, double duration);
+  Debris(Ball*, const Coord3d& pos, const Coord3d& vel, double duration);
   void calcRadius();  // Use this after modifying the parameters below
   void tick(Real);
-  void draw();
   void die(int);
 
   int floatingUp;
