@@ -25,25 +25,21 @@
 #include "sign.h"
 #include "sound.h"
 
-Diamond::Diamond(Coord3d pos) {
+Diamond::Diamond(const Coord3d &pos) : Animated(Role_OtherAnimated, 1) {
   position = pos;
 
-  specularColor[0] = specularColor[1] = specularColor[2] = 1.0;
-  primaryColor[0] = 0.7;
-  primaryColor[1] = 0.7;
-  primaryColor[2] = 0.9;
-  primaryColor[3] = 0.7;
+  specularColor = Color(1., 1., 1., 1.);
+  primaryColor = Color(0.7, 0.7, 0.9, 0.7);
   fade = 1.0;
   taken = 0;
 }
 
-int Diamond::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  if (fade <= 0.) { return 0; }
-  allocateBuffers(1, idxbufs, databufs);
+void Diamond::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                              const GLuint *vaolist, bool /*mustUpdate*/) const {
+  if (fade <= 0.) return;
 
-  GLfloat color[4];
-  for (int i = 0; i < 4; i++) color[i] = primaryColor[i];
-  color[3] *= fade;
+  Color color = primaryColor;
+  color.v[3] *= fade;
 
   GLfloat flat[3] = {0.f, 0.f, 0.f};
 
@@ -60,36 +56,33 @@ int Diamond::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
   ushort idxs[12][3] = {{0, 1, 2}, {0, 2, 3}, {0, 3, 4}, {0, 4, 5}, {0, 5, 6}, {0, 6, 1},
                         {7, 2, 1}, {7, 3, 2}, {7, 4, 3}, {7, 5, 4}, {7, 6, 5}, {7, 1, 6}};
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 8 * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
-
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, 36 * sizeof(ushort), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void Diamond::drawBuffers1(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void Diamond::drawBuffers1(const GLuint * /*vaolist*/) const {}
 
-void Diamond::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
+void Diamond::drawBuffers2(const GLuint *vaolist) const {
   if (fade <= 0.) { return; }
 
   glEnable(GL_BLEND);
   glEnable(GL_CULL_FACE);
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0],
-              specularColor[1], specularColor[2], specularColor[3]);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 100.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 100.f, Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 36, GL_UNSIGNED_SHORT, (void *)0);
 }
 
 void Diamond::tick(Real t) {
+  Animated::tick(t);
+
   Coord3d v0;
   if (fade <= 0.0) return;
   v0 = Game::current->player1->position - position;
@@ -109,5 +102,5 @@ void Diamond::onGet() {
   Coord3d signPos;
   signPos = position;
   signPos[2] += 1.0;
-  new Sign(_("Save point"), 6.0, 1.0, 60.0, signPos);
+  Game::current->add(new Sign(_("Save point"), 6.0, 1.0, 60.0, signPos));
 }
