@@ -25,7 +25,7 @@
 #include "editMode_codes.h"
 #include "editWindows.h"
 #include "game.h"
-#include "guile.h"
+#include "mainMode.h"
 #include "map.h"
 #include "menusystem.h"
 #include "settings.h"
@@ -161,7 +161,7 @@ int selectedMenu = -1;
 
 const int viewPoints[4][2] = {{1, 1}, {1, -1}, {-1, -1}, {-1, 1}};
 
-EditMode* EditMode::editMode;
+EditMode* EditMode::editMode = NULL;
 
 void EditMode::loadStrings() {
   char* cMenuNames_i18n[N_SUBMENUS] = {
@@ -208,11 +208,15 @@ void EditMode::loadStrings() {
   memcpy(hillNames, hillNames_i18n, sizeof(hillNames));
 }
 
-void EditMode::init() {
-  loadStrings();
-
-  /* Create the editmode object */
-  new EditMode();
+EditMode* EditMode::init() {
+  if (!editMode) {
+    loadStrings();
+    editMode = new EditMode;
+  }
+  return editMode;
+}
+void EditMode::cleanup() {
+  if (editMode) delete editMode;
 }
 
 EditMode::EditMode() {
@@ -228,8 +232,7 @@ EditMode::EditMode() {
   scale = 0.5;
   rotation = 0.0;
   raise = 0.5;
-  color[0] = color[1] = color[2] = 0.9;
-  color[3] = 1.0;
+  color = Color(0.9, 0.9, 0.9, 1.0);
   doSave = 0;
   cellClipboard = NULL;
 
@@ -367,17 +370,7 @@ void EditMode::display() {
 
   /* configure lighting */
   activeView.fog_enabled = 0;
-  activeView.quadratic_attenuation = 0.;
-
-  GLfloat lightPosition[3] = {(GLfloat)x - 60.f, (GLfloat)y - 40.f, (GLfloat)h + 100.f};
-  GLfloat lightDiffuse[3] = {0.9, 0.9, 0.9};
-  GLfloat ambient[3] = {0.2, 0.2, 0.2};
-  GLfloat black[3] = {0., 0., 0.};
-  assign(black, activeView.global_ambient);
-  assign(ambient, activeView.light_ambient);
-  assign(lightDiffuse, activeView.light_diffuse);
-  assign(lightDiffuse, activeView.light_specular);
-  assign(lightPosition, activeView.light_position);
+  MainMode::setupLighting(false);
 
   /* Setup matrixes for the camera perspective */
   perspectiveMatrix(40, (GLdouble)screenWidth / (GLdouble)std::max(screenHeight, 1), 0.1, 200,
@@ -387,8 +380,8 @@ void EditMode::display() {
                y - viewPoints[switchViewpoint][1] * 7.0, (birdsEye ? 30.0 : 10.0) + h * 0.5, x,
                y, h, 0.0, 0.0, 1.0, activeView.modelview);
 
-  activeView.day_mode = 1;
-  updateUniforms();
+  activeView.day_mode = true;
+  markViewChanged();
   /* Shadow map rendering returns active modelview/projection to orig state */
   if (map) {
     if (Settings::settings->doShadows) {
@@ -415,21 +408,20 @@ void EditMode::display() {
     map->drawLoop(0, 0, map->width - 1, map->height - 1, 0);
     if (game) {
       /* Indicate start position and trigger object locations */
-      if (game->hooks) {
-        std::set<GameHook*>::iterator start = game->hooks->begin();
-        std::set<GameHook*>::iterator end = game->hooks->end();
-        for (std::set<GameHook*>::iterator i = start; i != end; i++) {
-          GameHook* hook = *i;
-          SmartTrigger* smart_trigger = dynamic_cast<SmartTrigger*>(hook);
-          Trigger* dumb_trigger = dynamic_cast<Trigger*>(hook);
-          if (smart_trigger) {
-            map->drawSpotRing(smart_trigger->x, smart_trigger->y, smart_trigger->radius, 2);
-          }
-          if (dumb_trigger) {
-            map->drawSpotRing(dumb_trigger->x, dumb_trigger->y, dumb_trigger->radius, 1);
-          }
+      for (int i = 0; i < game->hooks[Role_GameHook].size(); i++) {
+        GameHook* hook = game->hooks[Role_GameHook][i];
+        if (hook->invalid) continue;
+
+        SmartTrigger* smart_trigger = dynamic_cast<SmartTrigger*>(hook);
+        Trigger* dumb_trigger = dynamic_cast<Trigger*>(hook);
+        if (smart_trigger) {
+          map->drawSpotRing(smart_trigger->x, smart_trigger->y, smart_trigger->radius, 2);
+        }
+        if (dumb_trigger) {
+          map->drawSpotRing(dumb_trigger->x, dumb_trigger->y, dumb_trigger->radius, 1);
         }
       }
+
       map->drawSpotRing(map->startPosition[0], map->startPosition[1], 0.5, 0);
       game->draw();
     }
@@ -587,20 +579,28 @@ void EditMode::doCommand(int command) {
     break;
 
   case COLOR_RED:
-    color[0] = color[0] + 0.05;
-    if (color[0] > 1.01) color[0] = 0.0;
+    if (color.v[0] >= 65535)
+      color.v[0] = 0;
+    else
+      color.v[0] = std::min(65535, color.v[0] + 3277);
     break;
   case COLOR_GREEN:
-    color[1] = color[1] + 0.05;
-    if (color[1] > 1.01) color[1] = 0.0;
+    if (color.v[1] >= 65535)
+      color.v[1] = 0;
+    else
+      color.v[1] = std::min(65535, color.v[1] + 3277);
     break;
   case COLOR_BLUE:
-    color[2] = color[2] + 0.05;
-    if (color[2] > 1.01) color[2] = 0.0;
+    if (color.v[2] >= 65535)
+      color.v[2] = 0;
+    else
+      color.v[2] = std::min(65535, color.v[2] + 3277);
     break;
   case COLOR_ALPHA:
-    color[3] = color[3] + 0.05;
-    if (color[3] > 1.01) color[3] = 0.0;
+    if (color.v[3] >= 65535)
+      color.v[3] = 0;
+    else
+      color.v[3] = std::min(65535, color.v[3] + 3277);
     break;
 
   case FLAG_0:
@@ -618,13 +618,13 @@ void EditMode::doCommand(int command) {
         Cell& c2 = map->cell(x1, y1);
         c2.flags = (c2.flags & ~flag) | onoff;
       }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 0);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, false);
   } break;
   case FLAG_CH_TEXTURE: {
     int newTexture = mymod(cell.texture + 1 + (shift ? -1 : +1), numTextures + 1) - 1;
     for (x1 = xLow; x1 <= xHigh; x1++)
       for (y1 = yLow; y1 <= yHigh; y1++) { map->cell(x1, y1).texture = newTexture; }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 0);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, false);
   } break;
 
   case FEATURE_SPIKE:
@@ -668,7 +668,7 @@ void EditMode::doCommand(int command) {
                 val = map->cell(altx, alty).waterHeights[m];
                 break;
               case REPAIR_COLOR_CONT:
-                val = map->cell(altx, alty).colors[m][p];
+                val = map->cell(altx, alty).colors[m].v[p];
               }
               if (xLow <= altx && altx <= xHigh && yLow <= alty && alty <= yHigh) {
                 internal += val;
@@ -704,8 +704,8 @@ void EditMode::doCommand(int command) {
               c.waterHeights[k] = target;
               break;
             case REPAIR_COLOR_CONT:
-              dcent += target - c.colors[k][p];
-              c.colors[k][p] = target;
+              dcent += target - c.colors[k].v[p];
+              c.colors[k].v[p] = target;
             }
           }
           switch (command) {
@@ -717,7 +717,7 @@ void EditMode::doCommand(int command) {
             c.waterHeights[4] += dcent / 4;
             break;
           case REPAIR_COLOR_CONT:
-            c.colors[4][p] += dcent / 4;
+            c.colors[4].v[p] += dcent / 4;
           }
         }
     }
@@ -746,8 +746,9 @@ void EditMode::doCommand(int command) {
           break;
         case REPAIR_COLOR_CENTERS:
           for (int k = 0; k < 4; k++) {
-            c.colors[Cell::CENTER][k] =
-                0.25 * (c.colors[0][k] + c.colors[1][k] + c.colors[2][k] + c.colors[3][k]);
+            c.colors[Cell::CENTER].v[k] =
+                (c.colors[0].v[k] + c.colors[1].v[k] + c.colors[2].v[k] + c.colors[3].v[k]) /
+                4;
           }
           break;
 
@@ -768,11 +769,12 @@ void EditMode::doCommand(int command) {
           break;
         case REPAIR_COLOR_ROUND:
           for (int m = 0; m < 4; m++) {
+            float mscale = 0.05 * 65535.f;
             for (int k = 0; k < 4; k++) {
-              c.colors[k][m] = scale * roundint(c.colors[k][m] / scale);
+              c.colors[k].v[m] = mscale * roundint(c.colors[k].v[m] / mscale);
             }
-            c.colors[Cell::CENTER][m] =
-                0.25 * scale * roundint(4. * c.colors[Cell::CENTER][m] / scale);
+            c.colors[Cell::CENTER].v[m] =
+                0.25 * mscale * roundint(4. * c.colors[Cell::CENTER].v[m] / mscale);
           }
           break;
         case REPAIR_VEL_ROUND:
@@ -837,20 +839,7 @@ void EditMode::doCommand(int command) {
   case VIEW_LOAD_ENTITIES: {
     /* reset entities and state */
     if (game) delete game;
-    game = new Game(map);
-
-    /* load scripts */
-    char scmname[512];
-    snprintf(scmname, sizeof(scmname), "%s/levels/boot.scm", effectiveShareDir);
-    scmname[511] = '\0';
-    loadScript(scmname);
-    snprintf(scmname, 511, "%s/.trackballs/levels/%s.scm", getenv("HOME"), levelname);
-    scmname[511] = '\0';
-    if (!fileExists(scmname)) {
-      snprintf(scmname, 511, "%s/levels/%s.scm", effectiveShareDir, levelname);
-    }
-    scmname[511] = '\0';
-    loadScript(scmname);
+    game = new Game(map, levelname);
   } break;
 
   default:
@@ -913,7 +902,7 @@ void EditMode::doCellAction(int code, int direction) {
           c2.heights[Cell::CENTER] += (direction ? -scale : scale) / 4;
         }
       }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 1);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, true);
     break;
   case editModeColor:
     for (x1 = xLow; x1 <= xHigh; x1++)
@@ -923,26 +912,24 @@ void EditMode::doCellAction(int code, int direction) {
         if (direction) {
           /* Pick up color */
           if (ctrl)
-            for (i = 0; i < 4; i++) color[i] = c.wallColors[corner][i];
+            color = c.wallColors[corner];
           else
-            for (i = 0; i < 4; i++) color[i] = c.colors[corner][i];
+            color = c.colors[corner];
         } else if (code == CODE_CELL_ALL) {
           /* Paint color */
           if (ctrl)
-            for (j = 0; j < 4; j++)
-              for (i = 0; i < 4; i++) c2.wallColors[j][i] = color[i];
+            for (j = 0; j < 4; j++) c2.wallColors[j] = color;
           else
-            for (j = 0; j < 5; j++)
-              for (i = 0; i < 4; i++) c2.colors[j][i] = color[i];
+            for (j = 0; j < 5; j++) c2.colors[j] = color;
         } else {
           /* Paint color */
           if (ctrl)
-            for (i = 0; i < 4; i++) c2.wallColors[corner][i] = color[i];
+            c2.wallColors[corner] = color;
           else
-            for (i = 0; i < 4; i++) c2.colors[corner][i] = color[i];
+            c2.colors[corner] = color;
         }
       }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 0);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, false);
     break;
   case editModeWater:
     for (x1 = xLow; x1 <= xHigh; x1++)
@@ -962,7 +949,7 @@ void EditMode::doCellAction(int code, int direction) {
           c2.waterHeights[Cell::CENTER] += (direction ? -scale : scale) / 4;
         }
       }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 0);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, false);
     break;
 
   case editModeVelocity:
@@ -974,7 +961,7 @@ void EditMode::doCellAction(int code, int direction) {
         if (code == CODE_CELL_W) c2.velocity[0] -= direction ? 0.5 : 0.1;
         if (code == CODE_CELL_E) c2.velocity[0] += direction ? 0.5 : 0.1;
       }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 0);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, false);
     break;
 
   case editModeNoLines: {
@@ -989,37 +976,33 @@ void EditMode::doCellAction(int code, int direction) {
         Cell& c2 = map->cell(x1, y1);
         c2.flags = (c2.flags & ~flag) | onoff;
       }
-    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, 0);
+    map->markCellsUpdated(xLow, yLow, xHigh, yHigh, false);
   } break;
 
   case editModeFeatures: {
     switch (currentFeature) {
     case FEATURE_SPIKE:
-      map->cell(x, y).heights[Cell::SOUTH + Cell::WEST] += shift ? -raise : raise;
-      map->cell(x - 1, y).heights[Cell::SOUTH + Cell::EAST] += shift ? -raise : raise;
-      map->cell(x, y - 1).heights[Cell::NORTH + Cell::WEST] += shift ? -raise : raise;
-      map->cell(x - 1, y - 1).heights[Cell::NORTH + Cell::EAST] += shift ? -raise : raise;
-      map->markCellsUpdated(x - 1, y - 1, x, y, 0);
+      map->cell(x, y).heights[Cell::SW] += shift ? -raise : raise;
+      map->cell(x - 1, y).heights[Cell::SE] += shift ? -raise : raise;
+      map->cell(x, y - 1).heights[Cell::NW] += shift ? -raise : raise;
+      map->cell(x - 1, y - 1).heights[Cell::NE] += shift ? -raise : raise;
+      map->markCellsUpdated(x - 1, y - 1, x, y, false);
       break;
     case FEATURE_SMALL_HILL:
       map->cell(x, y).heights[Cell::CENTER] += (shift ? -raise : raise) * 1.2;
       for (i = 0; i < 4; i++) map->cell(x, y).heights[i] += (shift ? -raise : raise) * 1.0;
-      map->cell(x, y + 1).heights[Cell::SOUTH + Cell::EAST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x, y + 1).heights[Cell::SOUTH + Cell::WEST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x, y - 1).heights[Cell::NORTH + Cell::EAST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x, y - 1).heights[Cell::NORTH + Cell::WEST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x + 1, y).heights[Cell::SOUTH + Cell::WEST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x + 1, y).heights[Cell::NORTH + Cell::WEST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x - 1, y).heights[Cell::SOUTH + Cell::EAST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x - 1, y).heights[Cell::NORTH + Cell::EAST] += (shift ? -raise : raise) * 1.0;
-      map->cell(x + 1, y + 1).heights[Cell::SOUTH + Cell::WEST] +=
-          (shift ? -raise : raise) * 1.0;
-      map->cell(x - 1, y + 1).heights[Cell::SOUTH + Cell::EAST] +=
-          (shift ? -raise : raise) * 1.0;
-      map->cell(x + 1, y - 1).heights[Cell::NORTH + Cell::WEST] +=
-          (shift ? -raise : raise) * 1.0;
-      map->cell(x - 1, y - 1).heights[Cell::NORTH + Cell::EAST] +=
-          (shift ? -raise : raise) * 1.0;
+      map->cell(x, y + 1).heights[Cell::SE] += (shift ? -raise : raise) * 1.0;
+      map->cell(x, y + 1).heights[Cell::SW] += (shift ? -raise : raise) * 1.0;
+      map->cell(x, y - 1).heights[Cell::NE] += (shift ? -raise : raise) * 1.0;
+      map->cell(x, y - 1).heights[Cell::NW] += (shift ? -raise : raise) * 1.0;
+      map->cell(x + 1, y).heights[Cell::SW] += (shift ? -raise : raise) * 1.0;
+      map->cell(x + 1, y).heights[Cell::NW] += (shift ? -raise : raise) * 1.0;
+      map->cell(x - 1, y).heights[Cell::SE] += (shift ? -raise : raise) * 1.0;
+      map->cell(x - 1, y).heights[Cell::NE] += (shift ? -raise : raise) * 1.0;
+      map->cell(x + 1, y + 1).heights[Cell::SW] += (shift ? -raise : raise) * 1.0;
+      map->cell(x - 1, y + 1).heights[Cell::SE] += (shift ? -raise : raise) * 1.0;
+      map->cell(x + 1, y - 1).heights[Cell::NW] += (shift ? -raise : raise) * 1.0;
+      map->cell(x - 1, y - 1).heights[Cell::NE] += (shift ? -raise : raise) * 1.0;
       map->cell(x + 1, y + 1).heights[Cell::CENTER] += (shift ? -raise : raise) * 0.25;
       map->cell(x - 1, y + 1).heights[Cell::CENTER] += (shift ? -raise : raise) * 0.25;
       map->cell(x + 1, y - 1).heights[Cell::CENTER] += (shift ? -raise : raise) * 0.25;
@@ -1028,7 +1011,7 @@ void EditMode::doCellAction(int code, int direction) {
       map->cell(x - 1, y).heights[Cell::CENTER] += (shift ? -raise : raise) * 0.50;
       map->cell(x, y + 1).heights[Cell::CENTER] += (shift ? -raise : raise) * 0.50;
       map->cell(x, y - 1).heights[Cell::CENTER] += (shift ? -raise : raise) * 0.50;
-      map->markCellsUpdated(x - 1, y - 1, x + 1, y + 1, 0);
+      map->markCellsUpdated(x - 1, y - 1, x + 1, y + 1, false);
       break;
     case FEATURE_MEDIUM_HILL:
       makeHill(2);
@@ -1116,7 +1099,6 @@ void EditMode::key(int key) {
 
   return;
 }
-void EditMode::special(int key, int /*mx*/, int /*my*/) { printf("editmode: %d\n", key); }
 void EditMode::idle(Real td) {
   int x, y;
   tickMouse(td);
@@ -1144,26 +1126,26 @@ void EditMode::makeHill(int radius) {
                  sin(1. * M_PI * (my + radius + 0.5) / diameter) * (shift ? -raise : raise) *
                  10.)) *
           .1;
-      map->cell(x + mx, y + my).heights[Cell::NORTH + Cell::EAST] +=
+      map->cell(x + mx, y + my).heights[Cell::NE] +=
           ((int)(sin(1. * M_PI * (mx + radius + 1.0) / diameter) *
                  sin(1. * M_PI * (my + radius + 1.0) / diameter) * (shift ? -raise : raise) *
                  10.)) *
           .1;
-      map->cell(x + mx, y + my).heights[Cell::NORTH + Cell::WEST] +=
+      map->cell(x + mx, y + my).heights[Cell::NW] +=
           ((int)(sin(1. * M_PI * (mx + radius) / diameter) *
                  sin(1. * M_PI * (my + radius + 1.0) / diameter) * (shift ? -raise : raise) *
                  10.)) *
           .1;
-      map->cell(x + mx, y + my).heights[Cell::SOUTH + Cell::EAST] +=
+      map->cell(x + mx, y + my).heights[Cell::SE] +=
           ((int)(sin(1. * M_PI * (mx + radius + 1.0) / diameter) *
                  sin(1. * M_PI * (my + radius) / diameter) * (shift ? -raise : raise) * 10.)) *
           .1;
-      map->cell(x + mx, y + my).heights[Cell::SOUTH + Cell::WEST] +=
+      map->cell(x + mx, y + my).heights[Cell::SW] +=
           ((int)(sin(1. * M_PI * (mx + radius) / diameter) *
                  sin(1. * M_PI * (my + radius) / diameter) * (shift ? -raise : raise) * 10.)) *
           .1;
     }
-  map->markCellsUpdated(x - radius, y - radius, x + radius, y + radius, 0);
+  map->markCellsUpdated(x - radius, y - radius, x + radius, y + radius, false);
 }
 void EditMode::resizeWindows() {
   menuWindow->refreshChildPositions();
@@ -1188,7 +1170,7 @@ void EditMode::doSmooth(int radius) {
       Cell& c1 = map->cell(x + mx, y + my);
       for (i = 0; i < 5; i++) avgHeight += c1.heights[i];
     }
-  map->markCellsUpdated(x - radius, y - radius, x + radius, x + radius, 1);
+  map->markCellsUpdated(x - radius, y - radius, x + radius, x + radius, true);
   avgHeight = avgHeight / (5. * diameter * diameter);
   for (mx = -radius; mx <= radius; mx++)
     for (my = -radius; my <= radius; my++) {
@@ -1205,7 +1187,7 @@ void EditMode::doSmooth(int radius) {
               c.heights[Cell::NORTH * north + Cell::EAST * east] * (1. - s) + avgHeight * s);
         }
     }
-  map->markCellsUpdated(x - radius, y - radius, x + radius, x + radius, 1);
+  map->markCellsUpdated(x - radius, y - radius, x + radius, x + radius, true);
 }
 void EditMode::copyRegion() {
   if (markX < 0) return;
@@ -1241,7 +1223,7 @@ void EditMode::pasteRegion() {
       toCell.flags = fromCell.flags;
       toCell.texture = fromCell.texture;
     }
-  map->markCellsUpdated(x, y, x + cellClipboardWidth - 1, y + cellClipboardHeight - 1, 1);
+  map->markCellsUpdated(x, y, x + cellClipboardWidth - 1, y + cellClipboardHeight - 1, true);
 }
 
 void EditMode::closeAllDialogWindows() {
@@ -1274,11 +1256,9 @@ void EditMode::askNew() {
 }
 
 void EditMode::testLevel() {
-  if (SetupMode::setupMode) {
-    snprintf(Settings::settings->specialLevel, sizeof(Settings::settings->specialLevel), "%s",
-             levelname);
-    Settings::settings->doSpecialLevel = 1;
-    saveMap();
-    GameMode::activate(SetupMode::setupMode);
-  }
+  snprintf(Settings::settings->specialLevel, sizeof(Settings::settings->specialLevel), "%s",
+           levelname);
+  Settings::settings->doSpecialLevel = 1;
+  saveMap();
+  GameMode::activate(SetupMode::init());
 }
