@@ -24,29 +24,23 @@
 #include "player.h"
 #include "sound.h"
 
-Flag::Flag(Real x, Real y, int points, int visible, Real radius) {
-  scoreOnDeath = Game::defaultScores[SCORE_FLAG][0];
-  timeOnDeath = Game::defaultScores[SCORE_FLAG][1];
-
+Flag::Flag(Real x, Real y, int points, int visible, Real radius)
+    : Animated(Role_OtherAnimated, 1) {
   scoreOnDeath = points;
+  timeOnDeath = Game::defaultScores[SCORE_FLAG][1];
 
   this->visible = visible;
   this->radius = radius;
   position[0] = x;
   position[1] = y;
   position[2] = Game::current->map->getHeight(position[0], position[1]);
-  primaryColor[0] = 0.5;
-  primaryColor[1] = 0.5;
-  primaryColor[2] = 1.0;
-  secondaryColor[0] = 0.8;
-  secondaryColor[1] = 0.8;
-  secondaryColor[2] = 0.8;
+  primaryColor = Color(0.5, 0.5, 1.0, 1.0);
+  secondaryColor = Color(0.8, 0.8, 0.8, 1.0);
 }
 
-int Flag::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  if (!visible) return 0;
-
-  allocateBuffers(1, idxbufs, databufs);
+void Flag::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                           const GLuint *vaolist, bool /*mustUpdate*/) const {
+  if (!visible) return;
 
   GLfloat data[14 * 8];
   memset(data, 0, sizeof(data));
@@ -73,7 +67,7 @@ int Flag::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
                    0.4f * d3 * std::sin(d1 + d2 * 0.4f)};
   GLfloat dbx[5] = {dx[1] - dx[0], dx[2] - dx[0], dx[3] - dx[1], dx[4] - dx[2], dx[4] - dx[3]};
   GLfloat dby[5] = {0.1f, 0.2f, 0.2f, 0.2f, 0.1f};
-  GLfloat color[4] = {primaryColor[0], primaryColor[1], primaryColor[2], 1.0};
+  Color color = primaryColor.toOpaque();
   for (int i = 0; i < 5; i++) {
     Coord3d b(dbx[i], dby[i], 0.0);
     Coord3d up(0.0, 0.0, 1.0);
@@ -86,36 +80,33 @@ int Flag::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
                             0., 0., color, fnorm);
   }
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 14 * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
-
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, 12 * 3 * sizeof(ushort), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void Flag::drawBuffers1(GLuint *idxbufs, GLuint *databufs) {
+void Flag::drawBuffers1(const GLuint *vaolist) const {
   if (!visible) return;
 
   glDisable(GL_CULL_FACE);
   glDisable(GL_BLEND);
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0],
-              specularColor[1], specularColor[2], specularColor[3]);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 10.f / 128.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 10.f / 128.f, Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 12 * 3, GL_UNSIGNED_SHORT, (void *)0);
 }
 
-void Flag::drawBuffers2(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void Flag::drawBuffers2(const GLuint * /*vaolist*/) const {}
+
+void Flag::tick(Real t) {
+  Animated::tick(t);
 
-void Flag::tick(Real /*t*/) {
   position[2] = Game::current->map->getHeight(position[0], position[1]);
   Player *p = Game::current->player1;
   Coord3d diff;
@@ -128,9 +119,7 @@ void Flag::tick(Real /*t*/) {
 void Flag::onGet() {
   /* Triggers any events and creates scores/time bonuses */
   die(DIE_OTHER);
-  /* position[2] += 1.0;*/
   /* Play the sound effect */
   playEffect(SFX_GOT_FLAG);
-  /*new ScoreSign(points,position);*/
   remove();
 }
