@@ -20,26 +20,16 @@
 
 #include "forcefield.h"
 
-std::set<ForceField *> *ForceField::forcefields;
+#include "game.h"
 
-void ForceField::init() { forcefields = new std::set<ForceField *>(); }
-void ForceField::reset() {
-  delete forcefields;
-  forcefields = new std::set<ForceField *>();
-}
-
-ForceField::ForceField(Coord3d pos, Coord3d dir, Real h, int a) : Animated() {
+ForceField::ForceField(const Coord3d &pos, const Coord3d &dir, Real h, int a)
+    : Animated(Role_Forcefield, 1) {
   position = pos;
   direction = dir;
   height = h;
   allow = a;
-  primaryColor[0] = 0.3;
-  primaryColor[1] = 1.0;
-  primaryColor[2] = 0.3;
-  secondaryColor[0] = 1.0;
-  secondaryColor[1] = 1.0;
-  secondaryColor[2] = 1.0;
-  forcefields->insert(this);
+  primaryColor = Color(0.3, 1.0, 0.3, 1.0);
+  secondaryColor = Color(1.0, 1.0, 1.0, 1.0);
   bounceFactor = 2.5;
   boundingBox[0][0] = -abs(dir[0]);
   boundingBox[0][1] = -abs(dir[1]);
@@ -48,14 +38,9 @@ ForceField::ForceField(Coord3d pos, Coord3d dir, Real h, int a) : Animated() {
   boundingBox[1][1] = +abs(dir[1]);
   boundingBox[1][2] = +abs(dir[2]) + h;
 }
-void ForceField::onRemove() {
-  Animated::onRemove();
-  forcefields->erase(this);
-}
-
-int ForceField::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  if (!is_on) return 0;
-  allocateBuffers(1, idxbufs, databufs);
+void ForceField::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                                 const GLuint *vaolist, bool /*mustUpdate*/) const {
+  if (!is_on) return;
 
   Coord3d ndir = direction;
   GLfloat len = length(ndir);
@@ -64,9 +49,9 @@ int ForceField::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
   GLfloat edge = std::min(std::abs(len), std::abs((GLfloat)height));
   edge = std::min(edge / 5, 0.03f);
 
-  GLfloat color[4] = {primaryColor[0], primaryColor[1], primaryColor[2],
-                      0.4f + 0.2f * (GLfloat)frandom()};
-  GLfloat rimco[4] = {secondaryColor[0], secondaryColor[1], secondaryColor[2], 0.6};
+  Color color(primaryColor.f0(), primaryColor.f1(), primaryColor.f2(),
+              0.4f + 0.2f * (GLfloat)frandom());
+  Color rimco(secondaryColor.f0(), secondaryColor.f1(), secondaryColor.f2(), 0.6);
 
   GLfloat xycoords[12][2] = {{edge, edge},       {edge, (GLfloat)height - edge},
                              {len - edge, edge}, {len - edge, (GLfloat)height - edge},
@@ -74,18 +59,18 @@ int ForceField::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
                              {len - edge, edge}, {len - edge, (GLfloat)height - edge},
                              {0., 0.},           {0., (GLfloat)height},
                              {len, 0.},          {len, (GLfloat)height}};
-  GLfloat *colors[12] = {color, color, color, color, rimco, rimco,
-                         rimco, rimco, rimco, rimco, rimco, rimco};
 
   GLfloat data[12 * 8];
   char *pos = (char *)data;
   for (int i = 0; i < 12; i++) {
     GLfloat flat[3] = {0., 0., 0.};
-    pos += packObjectVertex(
-        pos, position[0] + xycoords[i][0] * ndir[0], position[1] + xycoords[i][0] * ndir[1],
-        position[2] + xycoords[i][0] * ndir[2] + xycoords[i][1], 0., 0., colors[i], flat);
+    pos += packObjectVertex(pos, position[0] + xycoords[i][0] * ndir[0],
+                            position[1] + xycoords[i][0] * ndir[1],
+                            position[2] + xycoords[i][0] * ndir[2] + xycoords[i][1], 0., 0.,
+                            i < 4 ? color : rimco, flat);
   }
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 12 * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
 
@@ -93,29 +78,24 @@ int ForceField::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
                         {5, 7, 11}, {7, 10, 11}, {7, 6, 10}, {6, 8, 10}, {6, 4, 8}};
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, 10 * 3 * sizeof(ushort), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void ForceField::drawBuffers1(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void ForceField::drawBuffers1(const GLuint * /*vaolist*/) const {}
 
-void ForceField::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
+void ForceField::drawBuffers2(const GLuint *vaolist) const {
   if (!is_on) return;
   if (activeView.calculating_shadows) return;
 
   glDisable(GL_CULL_FACE);
   glEnable(GL_BLEND);
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), 0., 0., 0., 1.);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 0.);
-  glUniform1f(glGetUniformLocation(shaderObject, "use_lighting"), -1.);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(Color(0., 0., 0., 1.), 0., Lighting_None);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 10 * 3, GL_UNSIGNED_SHORT, (void *)0);
 }
 
-void ForceField::tick(Real /*t*/) {}
+void ForceField::tick(Real t) { Animated::tick(t); }
