@@ -26,19 +26,17 @@
 #define FF_KILL2 4
 #define FF_BOUNCE2 8
 
-#include <set>
 #include "animated.h"
 
 class ForceField : public Animated {
  public:
-  ForceField(Coord3d position, Coord3d direction, Real height, int allow);
+  ForceField(const Coord3d& position, const Coord3d& direction, Real height, int allow);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   virtual void tick(Real);
-  virtual void onRemove();
 
   static void init();
   static void reset();
@@ -46,8 +44,6 @@ class ForceField : public Animated {
   Coord3d direction;
   Real height, bounceFactor;
   int allow;
-
-  static std::set<ForceField*>* forcefields;
 };
 
 #endif
