@@ -23,14 +23,15 @@
 #include "player.h"
 #include "settings.h"
 
-Fountain::Fountain(double randomSpeed, double radius, double strength)
-    : Animated(), randomSpeed(randomSpeed), radius(radius), strength(strength) {
-  primaryColor[0] = 0.4;
-  primaryColor[1] = 0.4;
-  primaryColor[2] = 0.8;
-  primaryColor[3] = 0.5;
+Fountain::Fountain(const Coord3d &pos, double randomSpeed, double radius, double strength)
+    : Animated(Role_OtherAnimated, 1),
+      randomSpeed(randomSpeed),
+      radius(radius),
+      strength(strength) {
+  primaryColor = Color(0.4, 0.4, 0.8, 0.5);
   velocity[0] = velocity[1] = 0.0;
   velocity[2] = 0.3;
+  position = pos;
   createPoint = 0.0;
   nextPoint = 0;
   drawFrom = 0;
@@ -39,16 +40,10 @@ Fountain::Fountain(double randomSpeed, double radius, double strength)
   memset(creationTime, 0, sizeof(creationTime));
 }
 
-int Fountain::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  if (Settings::settings->gfx_details <= GFX_DETAILS_MINIMALISTIC) return 0;
-  allocateBuffers(1, idxbufs, databufs);
+void Fountain::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                               const GLuint *vaolist, bool /*mustUpdate*/) const {
+  if (Settings::settings->gfx_details <= GFX_DETAILS_MINIMALISTIC) return;
 
-  Real timeNow = Game::current->gameTime;
-
-  while (creationTime[drawFrom] < timeNow - 1.5) {
-    if (drawFrom == nextPoint) break;
-    drawFrom = (drawFrom + 1) % 800;
-  }
   int skip = Settings::settings->gfx_details <= GFX_DETAILS_SIMPLE ? 2 : 1;
   if (fps < 5) skip *= 2;
   int npoints = 0;
@@ -85,19 +80,20 @@ int Fountain::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
 
   for (int i = 0; i < npoints; i++) { idxs[i] = i; }
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 3 * npoints * sizeof(GLfloat), data, GL_STATIC_DRAW);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, npoints * sizeof(ushort), idxs, GL_STATIC_DRAW);
+  glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, (void *)0);
+  glEnableVertexAttribArray(0);
   delete[] data;
   delete[] idxs;
-
-  return 1;
 }
 
-void Fountain::drawBuffers1(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void Fountain::drawBuffers1(const GLuint * /*vaolist*/) const {}
 
-void Fountain::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
+void Fountain::drawBuffers2(const GLuint *vaolist) const {
   if (Settings::settings->gfx_details <= GFX_DETAILS_MINIMALISTIC) return;
   if (activeView.calculating_shadows) return;
 
@@ -112,32 +108,30 @@ void Fountain::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
 
   glEnable(GL_BLEND);
   glPointSize(1.5 * screenWidth / 600.);
-  setActiveProgramAndUniforms(shaderLine);
-  glUniform4f(glGetUniformLocation(shaderLine, "line_color"), primaryColor[0], primaryColor[1],
-              primaryColor[2], primaryColor[3]);
+  setActiveProgramAndUniforms(Shader_Line);
+  glUniformC(uniformLocations.line_color, primaryColor);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, (void *)0);
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_POINTS, npoints, GL_UNSIGNED_SHORT, (void *)0);
 }
 
 void Fountain::tick(Real t) {
-  int i, j;
+  Animated::tick(t);
+
   int skip = Settings::settings->gfx_details <= GFX_DETAILS_SIMPLE ? 2 : 1;
 
   if (nextPoint < drawFrom) {
-    for (i = drawFrom; i < 800; i += skip) {
-      for (j = 0; j < 3; j++) positions[i][j] += velocities[i][j];
+    for (int i = drawFrom; i < 800; i += skip) {
+      for (int j = 0; j < 3; j++) positions[i][j] += velocities[i][j];
       velocities[i][2] -= t * 1.0;
     }
-    for (i = 0; i < nextPoint; i += skip) {
-      for (j = 0; j < 3; j++) positions[i][j] += velocities[i][j];
+    for (int i = 0; i < nextPoint; i += skip) {
+      for (int j = 0; j < 3; j++) positions[i][j] += velocities[i][j];
       velocities[i][2] -= t * 1.0;
     }
   } else
-    for (i = drawFrom; i < nextPoint; i += skip) {
-      for (j = 0; j < 3; j++) positions[i][j] += velocities[i][j];
+    for (int i = drawFrom; i < nextPoint; i += skip) {
+      for (int j = 0; j < 3; j++) positions[i][j] += velocities[i][j];
       velocities[i][2] -= t * 1.0;
     }
 
@@ -165,9 +159,14 @@ void Fountain::tick(Real t) {
 
     diff = position - p->position;
     if (length(diff) < p->radius + 0.2) {
-      for (i = 0; i < 3; i++)
+      for (int i = 0; i < 3; i++)
         p->velocity[i] +=
             velocity[i] / 0.3 * strength * 0.5 * (p->radius + 0.2 - length(diff)) * t;
     }
   }
+
+  while (creationTime[drawFrom] < Game::current->gameTime - 1.5) {
+    if (drawFrom == nextPoint) break;
+    drawFrom = (drawFrom + 1) % 800;
+  }
 }
