@@ -25,11 +25,11 @@
 
 class Fountain : public Animated {
  public:
-  Fountain(double randomSpeed, double radius, double strength);
+  Fountain(const Coord3d& position, double randomSpeed, double radius, double strength);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(double);
 
