@@ -29,7 +29,8 @@
 #define SCORE_FLAG 5
 #define SCORE_MAX 6
 
-#include <set>
+#include <vector>
+#include "gameHook.h"
 #include "general.h"
 #include "glHelp.h"
 
@@ -38,21 +39,23 @@ class Map;
 class Player;
 class Weather;
 class EditMode;
+class Gamer;
 
 class Game {
  public:
-  Game(const char *, class Gamer *gamer);
-  explicit Game(Map *editmap);
+  Game(const char *, Gamer *gamer);
+  Game(Map *editmap, const char *levelname);
   virtual ~Game();
 
   void tick(Real);
-  void draw();                        /* Used for normal drawing mode */
-  void drawReflection(Coord3d focus); /* Used when drawing environmentmaps */
+  void draw();                               /* Used for normal drawing mode */
+  void drawReflection(const Coord3d &focus); /* Used when drawing environmentmaps */
   void doExpensiveComputations();
-  void add(class Animated *);
-  void add(class GameHook *);
-  void remove(class Animated *);
-  void remove(class GameHook *);
+
+  void add(GameHook *);
+  void queueCall(SCM fun);
+  void queueCall(SCM fun, double val);
+  void queueCall(SCM fun, GameHook *subject, SCM object);
   void loadLevel(const char *level);
   void clearLevel();
 
@@ -80,13 +83,26 @@ class Game {
   static double defaultScores[SCORE_MAX][2];
 
  protected:
+  friend class Ball;
   friend class EditMode;
-  std::set<GameHook *> *hooks;
+  std::vector<GameHook *> hooks[Role_MaxTypes];
 
  private:
-  void setDefaults();
+  typedef struct {
+    SCM fun;
+    union {
+      double val;
+      struct {
+        GameHook *subject;
+        SCM object;
+      };
+    };
+    int type;
+  } QueuedCall;
+  std::vector<QueuedCall> queuedCalls;
+  std::vector<GameHook *> newHooks;
 
-  std::set<Animated *> *objects;
+  void setDefaults();
 };
 
 #endif
