@@ -21,59 +21,22 @@
 */
 
 #include "gameHook.h"
-#include <set>
 #include "animated.h"
 #include "game.h"
 #include "guile.h"
 
-int GameHook::nextId = 0;
+GameHook::GameHook(int role) {
+  invalid = false;
+  is_on = true;
+  entity_role = role;
 
-static std::set<GameHook *> *deadObjects;
-
-void GameHook::init() { deadObjects = new std::set<GameHook *>(); }
-GameHook::GameHook() {
-  int i;
-
-  alive = 1;
-  id = nextId++;
-  is_on = 1;
-
-  if (Game::current)
-    Game::current->add(this);
-  else
-    error("GameHook::GameHook() - no game loaded");
-
-  for (i = 0; i < GameHookEvent_MaxHooks; i++) hooks[i] = NULL;
+  for (int i = 0; i < GameHookEvent_MaxHooks; i++) hooks[i] = NULL;
 }
 
-GameHook::~GameHook() {}
+GameHook::~GameHook() { GameHook::releaseCallbacks(); }
 
-void GameHook::remove() {
-  if (alive) {
-    deadObjects->insert(this);
-    alive = 0;
-    this->onRemove();
-  }
-}
-void GameHook::onRemove() {
-  int event;
+void GameHook::remove() { invalid = true; }
 
-  /* Remove all registered guile hooks */
-  for (event = 0; event < GameHookEvent_MaxHooks; event++)
-    if (hooks[event]) scm_gc_unprotect_object(hooks[event]);
-
-  Game::current->remove(this);
-}
-void GameHook::deleteDeadObjects() {
-  std::set<GameHook *>::iterator iter = deadObjects->begin();
-  std::set<GameHook *>::iterator end = deadObjects->end();
-  for (; iter != end; iter++) {
-    GameHook *a = *iter;
-    delete a;
-  }
-  deadObjects->clear();
-}
-void GameHook::playerRestarted() {}
 void GameHook::doExpensiveComputations() {}
 
 void GameHook::registerHook(GameHookEvent event, SCM hook) {
@@ -88,15 +51,7 @@ void GameHook::registerHook(GameHookEvent event, SCM hook) {
 void GameHook::triggerHook(GameHookEvent event, SCM object) {
   if (event < 0 || event >= GameHookEvent_MaxHooks) return;
 
-  if (hooks[event]) {
-    SCM subject;
-    Animated *a = dynamic_cast<Animated *>(this);
-    if (a)
-      subject = smobAnimated_make(a);
-    else
-      subject = smobGameHook_make(this);
-    scm_catch_apply_2(hooks[event], subject, object ? object : SCM_BOOL_F);
-  }
+  if (hooks[event]) { Game::current->queueCall(hooks[event], this, object); }
 }
 
 SCM GameHook::getHook(GameHookEvent event) {
@@ -104,4 +59,11 @@ SCM GameHook::getHook(GameHookEvent event) {
   return hooks[event];
 }
 
+void GameHook::releaseCallbacks() {
+  for (int event = 0; event < GameHookEvent_MaxHooks; event++) {
+    if (hooks[event]) scm_gc_unprotect_object(hooks[event]);
+    hooks[event] = NULL;
+  }
+}
+
 void GameHook::tick(Real /*dt*/) { triggerHook(GameHookEvent_Tick, NULL); }
