@@ -33,18 +33,24 @@ typedef enum {
   GameHookEvent_MaxHooks
 } GameHookEvent;
 
+enum {
+  Role_Dead = 0,          /* collects entities for cleanup */
+  Role_GameHook = 1,      /* pure GameHooks (will not be rendered) */
+  Role_OtherAnimated = 2, /* animated: misc types */
+  Role_Ball = 3,          /* animated: all types inheriting from Ball, except */
+  Role_Player = 4,        /* animated: ... player balls */
+  Role_Pipe = 5,          /* animated: pipes */
+  Role_PipeConnector = 6, /* animated: pipe connectors */
+  Role_Forcefield = 7,    /* animated: forcefields */
+  Role_MaxTypes = 8
+};
+
 class GameHook {
  public:
-  static void init();
-
-  GameHook();
+  explicit GameHook(int role);
   virtual ~GameHook();
   virtual void tick(Real);
   virtual void doExpensiveComputations();
-  virtual void onRemove();
-  virtual void playerRestarted();
-
-  static void deleteDeadObjects();
 
   /** Trigger any guile hooks which have been registered for this
       event */
@@ -56,16 +62,17 @@ class GameHook {
   /** Returns the hook currently registered to event, or NULL */
   SCM getHook(GameHookEvent event);
 
-  /* Removes objects from the world. Is not deleted until end of level
-     (to avoid stale references) */
+  /** Marks the object for removal from the world. */
   void remove();
 
-  int alive;
-  int id;
-  int is_on;
+  /** Unregisters all hooks */
+  virtual void releaseCallbacks();
+
+  int entity_role;
+  bool is_on;   /* for switches, forcefields, objects with toggle state */
+  bool invalid; /* if true, object is slated for cleanup */
 
  private:
-  static int nextId;
   SCM hooks[GameHookEvent_MaxHooks];
 };
 
