@@ -23,6 +23,7 @@
 #define GENERAL_H
 
 #include <libintl.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <cmath>
 #define _(String) gettext(String)
@@ -40,8 +41,8 @@ typedef unsigned char uchar;
 typedef unsigned short ushort;
 typedef unsigned long ulong;
 typedef double Real;
-typedef double Matrix4d[4][4];
-typedef double Matrix3d[3][3];
+typedef Real Matrix4d[4][4];
+typedef Real Matrix3d[3][3];
 typedef int ICoord2d[2];
 
 // Simple vector type to make geometry code readable
@@ -52,7 +53,7 @@ class Coord3d {
     data[1] = 0.;
     data[2] = 0.;
   }
-  inline Coord3d(double x[3]) {
+  explicit inline Coord3d(double x[3]) {
     data[0] = x[0];
     data[1] = x[1];
     data[2] = x[2];
@@ -83,10 +84,41 @@ class Coord3d {
   }
 
  private:
-  double data[3];
+  Real data[3];
 };
 inline Coord3d operator*(double scale, const Coord3d &vec) { return vec * scale; }
 
+class Color {
+ public:
+  Color() {
+    v[0] = 0;
+    v[1] = 0;
+    v[2] = 0;
+    v[3] = 65535;
+  }
+  Color(float fr, float fg, float fb, float fa) {
+    v[0] = 65535.f * fr;
+    v[1] = 65535.f * fg;
+    v[2] = 65535.f * fb;
+    v[3] = 65535.f * fa;
+  };
+  Color toOpaque() const {
+    Color c(*this);
+    c.v[3] = 65535;
+    return c;
+  }
+  static Color mix(float blend, const Color &col0, const Color &col1) {
+    Color c;
+    for (int i = 0; i < 4; i++) { c.v[i] = col0.v[i] * (1. - blend) + col1.v[i] * blend; }
+    return c;
+  }
+  float f0() const { return v[0] / 65535.f; }
+  float f1() const { return v[1] / 65535.f; }
+  float f2() const { return v[2] / 65535.f; }
+  float f3() const { return v[3] / 65535.f; }
+  uint16_t v[4];
+};
+
 #ifndef M_PI
 #define M_PI 3.14159265358979323846264
 #endif
