@@ -36,14 +36,26 @@
 float fps = 50.0;
 int screenWidth = 640, screenHeight = 480;
 ViewParameters activeView;
+UniformLocations uniformLocations;
 
-GLuint shaderWater = 0;
-GLuint shaderTile = 0;
-GLuint shaderLine = 0;
-GLuint shaderUI = 0;
-GLuint shaderObject = 0;
-GLuint shaderReflection = 0;
-GLuint theVao = 0;
+static GLuint shaderWaterDay = 0;
+static GLuint shaderWaterNight = 0;
+static GLuint shaderTileDay = 0;
+static GLuint shaderTileNight = 0;
+static GLuint shaderTileShadow = 0;
+static GLuint shaderLine = 0;
+static GLuint shaderUI = 0;
+static GLuint shaderObjectDay = 0;
+static GLuint shaderObjectNight = 0;
+static GLuint shaderObjectShadow = 0;
+static GLuint shaderReflection = 0;
+static GLuint defaultVao = 0;
+GLuint textureBlank = 0;
+GLuint textureGlitter = 0;
+GLuint textureMousePointer = 0;
+GLuint textureDizzy = 0;
+GLuint textureWings = 0;
+GLuint textureTrack = 0;
 
 TTF_Font *ingameFont;
 extern struct timespec displayStartTime;
@@ -53,6 +65,48 @@ extern double timeDilationFactor;
 const GLfloat menuColorSelected[4] = {0.86f, 0.86f, 0.86f, 1.f};
 const GLfloat menuColor[4] = {0.86f, 0.86f, 0.25f, 1.f};
 
+const double sin6[6] = {0.0, 0.8660254037844386,  0.8660254037844386,
+                        0.0, -0.8660254037844386, -0.8660254037844386};
+const double cos6[6] = {1.0, 0.5, -0.5, -1.0, -0.5, 0.5};
+const double sin10[10] = {
+    0.0, 0.5877852522924731,  0.9510565162951535,  0.9510565162951535,  0.5877852522924731,
+    0.0, -0.5877852522924731, -0.9510565162951535, -0.9510565162951535, -0.5877852522924731};
+const double cos10[10] = {
+    1.0,  0.8090169943749475,  0.30901699437494745,  -0.30901699437494745, -0.8090169943749475,
+    -1.0, -0.8090169943749475, -0.30901699437494745, 0.30901699437494745,  0.8090169943749475};
+const double sin12[12] = {0.0, 0.5,  0.8660254037844386,  1.0,  0.8660254037844386,  0.5,
+                          0.0, -0.5, -0.8660254037844386, -1.0, -0.8660254037844386, -0.5};
+const double cos12[12] = {1.0,  0.8660254037844386,  0.5,  0.0, -0.5, -0.8660254037844386,
+                          -1.0, -0.8660254037844386, -0.5, 0.0, 0.5,  0.8660254037844386};
+const double sin14[14] = {0.0,
+                          0.4338837391175581,
+                          0.7818314824680298,
+                          0.9749279121818236,
+                          0.9749279121818236,
+                          0.7818314824680298,
+                          0.4338837391175581,
+                          0.0,
+                          -0.4338837391175581,
+                          -0.7818314824680298,
+                          -0.9749279121818236,
+                          -0.9749279121818236,
+                          -0.7818314824680298,
+                          -0.4338837391175581};
+const double cos14[14] = {1.0,
+                          0.9009688679024191,
+                          0.6234898018587336,
+                          0.22252093395631445,
+                          -0.22252093395631445,
+                          -0.6234898018587336,
+                          -0.9009688679024191,
+                          -1.0,
+                          -0.9009688679024191,
+                          -0.6234898018587336,
+                          -0.22252093395631445,
+                          0.22252093395631445,
+                          0.6234898018587336,
+                          0.9009688679024191};
+
 double fake_rand[4711];
 double frand(size_t i) {
   /* size_t is nonnegative so no inputs will crash */
@@ -93,8 +147,7 @@ struct StringCache {
 static long stringTick = 0;
 static std::map<StringInfo, StringCache> strcache;
 
-static SDL_Surface *drawStringToSurface(struct StringInfo &inf, int outlined) {
-  SDL_Surface *outline;
+static SDL_Surface *drawStringToSurface(struct StringInfo &inf, bool outlined) {
   if (outlined) {
     TTF_SetFontOutline(inf.font, 0);
     SDL_Surface *inner = TTF_RenderUTF8_Blended(inf.font, inf.string, inf.color);
@@ -104,7 +157,7 @@ static SDL_Surface *drawStringToSurface(struct StringInfo &inf, int outlined) {
     }
     TTF_SetFontOutline(inf.font, 2);
     SDL_Color blkColor = {0, 0, 0, 0};
-    outline = TTF_RenderUTF8_Blended(inf.font, inf.string, blkColor);
+    SDL_Surface *outline = TTF_RenderUTF8_Blended(inf.font, inf.string, blkColor);
     if (!outline) {
       warning("Failed to render string inside '%s'", inf.string);
       return NULL;
@@ -113,18 +166,19 @@ static SDL_Surface *drawStringToSurface(struct StringInfo &inf, int outlined) {
     SDL_SetSurfaceBlendMode(inner, SDL_BLENDMODE_BLEND);
     SDL_BlitSurface(inner, NULL, outline, &rect);
     SDL_FreeSurface(inner);
+    return outline;
   } else {
-    outline = TTF_RenderUTF8_Blended(inf.font, inf.string, inf.color);
+    SDL_Surface *outline = TTF_RenderUTF8_Blended(inf.font, inf.string, inf.color);
     if (!outline) {
       warning("Failed to render string '%s'", inf.string);
       return NULL;
     }
+    return outline;
   }
-  return outline;
 }
 
 int draw2DString(TTF_Font *font, const char *string, int x, int y, float red, float green,
-                 float blue, float alpha, int outlined, int align, int maxwidth) {
+                 float blue, float alpha, bool outlined, int align, int maxwidth) {
   struct StringInfo inf;
   inf.color.r = 255 * red;
   inf.color.g = 255 * green;
@@ -163,7 +217,7 @@ void update2DStringCache() {
     erased = false;
     for (std::map<StringInfo, StringCache>::iterator i = strcache.begin(); i != strcache.end();
          ++i) {
-      if (i->second.tick < stringTick - 10) {
+      if (i->second.tick < stringTick - 100) {
         glDeleteTextures(1, &i->second.texture);
         strcache.erase(i);
         erased = true;
@@ -226,7 +280,7 @@ void draw2DRectangle(GLfloat x, GLfloat y, GLfloat w, GLfloat h, GLfloat tx, GLf
 void draw2DQuad(const GLfloat ver[4][2], const GLfloat txc[4][2], const GLfloat col[4][4],
                 GLuint tex) {
   Require2DMode();
-  if (tex == 0) { tex = textures[loadTexture("blank.png")]; }
+  if (tex == 0) { tex = textureBlank; }
 
   const GLfloat data[32] = {
       ver[0][0], ver[0][1], col[0][0], col[0][1], col[0][2], col[0][3], txc[0][0], txc[0][1],
@@ -282,18 +336,18 @@ void drawMouse(int x, int y, int w, int h) {
   GLfloat texco[4][2] = {{0., 0.}, {0., 1.}, {1., 0.}, {1., 1.}};
 
   GLfloat vco[4][2] = {{x - dx, y - dy}, {x - dy, y + dx}, {x + dy, y - dx}, {x + dx, y + dy}};
-  draw2DQuad(vco, texco, colors, textures[loadTexture("mousePointer.png")]);
+  draw2DQuad(vco, texco, colors, textureMousePointer);
 }
 
 size_t packObjectVertex(void *dest, GLfloat x, GLfloat y, GLfloat z, GLfloat tx, GLfloat ty,
-                        const GLfloat color[4], const GLfloat normal[3]) {
+                        const Color &color, const GLfloat normal[3]) {
   uint32_t *aout = (uint32_t *)dest;
   GLfloat *fout = (GLfloat *)dest;
   fout[0] = x;
   fout[1] = y;
   fout[2] = z;
-  aout[3] = (((uint32_t)(65535.f * color[1])) << 16) + (uint32_t)(65535.f * color[0]);
-  aout[4] = (((uint32_t)(65535.f * color[3])) << 16) + (uint32_t)(65535.f * color[2]);
+  aout[3] = (((uint32_t)(color.v[1])) << 16) + (uint32_t)(color.v[0]);
+  aout[4] = (((uint32_t)(color.v[3])) << 16) + (uint32_t)(color.v[2]);
   aout[5] = (((uint32_t)(65535.f * 0.5f * ty)) << 16) + (uint32_t)(65535.f * 0.5f * tx);
   aout[6] = packNormal(normal);
   return 8 * sizeof(GLfloat);
@@ -306,64 +360,168 @@ void configureObjectAttributes() {
                         (void *)(5 * sizeof(GLfloat)));
   glVertexAttribPointer(3, 4, GL_UNSIGNED_INT_2_10_10_10_REV, GL_TRUE, 8 * sizeof(GLfloat),
                         (void *)(6 * sizeof(GLfloat)));
+  glEnableVertexAttribArray(0);
+  glEnableVertexAttribArray(1);
+  glEnableVertexAttribArray(2);
+  glEnableVertexAttribArray(3);
 }
 
 static GLuint lastProgram = 0;
 
-void updateUniforms() { lastProgram = 0; }
+void markViewChanged() { lastProgram = 0; }
 
-void setViewUniforms(GLuint shader);
-void setActiveProgramAndUniforms(GLuint shader) {
-  if (shader == shaderObject) {
-    /* frequently changed unique variables */
-    glUseProgram(shader);
-    glUniform1f(glGetUniformLocation(shader, "use_lighting"), 1.);
-    glUniform1f(glGetUniformLocation(shader, "ignore_shadow"), -1.);
+void setActiveProgramAndUniforms(Shader_Type type) {
+  GLuint shader = 0;
+  switch (type) {
+  case Shader_Line:
+    shader = shaderLine;
+    break;
+  case Shader_Object:
+    if (activeView.calculating_shadows) {
+      shader = shaderObjectShadow;
+    } else {
+      if (activeView.day_mode) {
+        shader = shaderObjectDay;
+      } else {
+        shader = shaderObjectNight;
+      }
+    }
+    break;
+  case Shader_Tile:
+    if (activeView.calculating_shadows) {
+      shader = shaderTileShadow;
+    } else {
+      if (activeView.day_mode) {
+        shader = shaderTileDay;
+      } else {
+        shader = shaderTileNight;
+      }
+    }
+    break;
+  case Shader_Water:
+    if (activeView.day_mode) {
+      shader = shaderWaterDay;
+    } else {
+      shader = shaderWaterNight;
+    }
+    break;
+  case Shader_Reflection:
+    shader = shaderReflection;
+    break;
   }
-  if (shader == lastProgram) { return; }
-  lastProgram = shader;
-  if (shader == 0) { return; }
-  glBindVertexArray(theVao);
+
+  if (shader == lastProgram) return;
   glUseProgram(shader);
-  if (shader == shaderTile) {
-    glEnableVertexAttribArray(0);
-    glEnableVertexAttribArray(1);
-    glEnableVertexAttribArray(2);
-    glEnableVertexAttribArray(3);
-    glEnableVertexAttribArray(4);
-  } else if (shader == shaderWater) {
-    glEnableVertexAttribArray(0);
-    glEnableVertexAttribArray(1);
-    glEnableVertexAttribArray(2);
-    glEnableVertexAttribArray(3);
-    glEnableVertexAttribArray(4);
-  } else if (shader == shaderLine) {
-    // Pos
-    glEnableVertexAttribArray(0);
+  lastProgram = shader;
 
-  } else if (shader == shaderObject) {
-    // Pos, Color, Tex, Norm, ~Vel~,
-    glEnableVertexAttribArray(0);
-    glEnableVertexAttribArray(1);
-    glEnableVertexAttribArray(2);
-    glEnableVertexAttribArray(3);
-    glUniform1f(glGetUniformLocation(shaderObject, "use_lighting"), 1.);
-    glUniform1f(glGetUniformLocation(shaderObject, "ignore_shadow"), -1.);
+  /* Set universal uniforms */
+  Matrix4d mvp;
+  matrixMult(activeView.modelview, activeView.projection, mvp);
+  GLfloat lmvp[16], lmodel[16];
+  for (int i = 0; i < 4; i++) {
+    for (int j = 0; j < 4; j++) {
+      lmvp[4 * i + j] = mvp[i][j];
+      lmodel[4 * i + j] = activeView.modelview[i][j];
+    }
+  }
+  glUniformMatrix4fv(glGetUniformLocation(shader, "mvp_matrix"), 1, GL_FALSE, (GLfloat *)lmvp);
+  glUniformMatrix4fv(glGetUniformLocation(shader, "model_matrix"), 1, GL_FALSE,
+                     (GLfloat *)lmodel);
 
-    glUniform1i(glGetUniformLocation(shaderObject, "tex"), 0);
-  } else if (shader == shaderReflection) {
-    // Pos, Color, Tex, Norm, ~Vel~
-    glEnableVertexAttribArray(0);
-    glEnableVertexAttribArray(1);
-    glEnableVertexAttribArray(2);
-    glEnableVertexAttribArray(3);
-  } else {
-    /* We don't handle the 2d programs */
-    warning("Unidentified 3d shader program");
-    return;
+  switch (type) {
+  case Shader_Tile:
+    uniformLocations.render_stage = glGetUniformLocation(shader, "render_stage");
+    uniformLocations.arrtex = glGetUniformLocation(shader, "arrtex");
+    uniformLocations.gameTime = glGetUniformLocation(shader, "gameTime");
+    break;
+  case Shader_Water:
+    uniformLocations.wtex = glGetUniformLocation(shader, "wtex");
+    uniformLocations.gameTime = glGetUniformLocation(shader, "gameTime");
+    break;
+  case Shader_Object:
+    uniformLocations.render_stage = glGetUniformLocation(shader, "render_stage");
+    uniformLocations.specular_color = glGetUniformLocation(shader, "specular");
+    uniformLocations.shininess = glGetUniformLocation(shader, "shininess");
+    uniformLocations.use_lighting = glGetUniformLocation(shader, "use_lighting");
+    uniformLocations.ignore_shadow = glGetUniformLocation(shader, "ignore_shadow");
+    break;
+  case Shader_Reflection:
+    uniformLocations.refl_color = glGetUniformLocation(shader, "refl_color");
+    uniformLocations.tex = glGetUniformLocation(shader, "tex");
+    break;
+  case Shader_Line:
+    uniformLocations.line_color = glGetUniformLocation(shader, "line_color");
+    break;
+  }
+  if (!activeView.calculating_shadows) {
+    /* Fog applies to all display shaders */
+    glUniform1i(glGetUniformLocation(shader, "fog_active"), activeView.fog_enabled);
+    glUniform3f(glGetUniformLocation(shader, "fog_color"), activeView.fog_color[0],
+                activeView.fog_color[1], activeView.fog_color[2]);
+    glUniform1f(glGetUniformLocation(shader, "fog_start"), activeView.fog_start);
+    glUniform1f(glGetUniformLocation(shader, "fog_end"), activeView.fog_end);
+
+    if (type == Shader_Object || type == Shader_Water || type == Shader_Tile) {
+      glUniform3f(glGetUniformLocation(shader, "light_ambient"), activeView.light_ambient[0],
+                  activeView.light_ambient[1], activeView.light_ambient[2]);
+      glUniform3f(glGetUniformLocation(shader, "light_diffuse"), activeView.light_diffuse[0],
+                  activeView.light_diffuse[1], activeView.light_diffuse[2]);
+      if (type == Shader_Object) {
+        glUniform3f(glGetUniformLocation(shader, "light_specular"),
+                    activeView.light_specular[0], activeView.light_specular[1],
+                    activeView.light_specular[2]);
+      }
+
+      if (activeView.day_mode) {
+        glUniform1i(glGetUniformLocation(shader, "shadow_cascade0"), 2);
+        glActiveTexture(GL_TEXTURE2);
+        glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[0]);
+        glUniform1i(glGetUniformLocation(shader, "shadow_cascade1"), 3);
+        glActiveTexture(GL_TEXTURE3);
+        glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[1]);
+        glUniform1i(glGetUniformLocation(shader, "shadow_cascade2"), 4);
+        glActiveTexture(GL_TEXTURE4);
+        glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[2]);
+
+        const int N = 3;
+        GLfloat cscmvp[N * 16], cscmodel[N * 16];
+        for (int i = 0; i < N; i++) {
+          Matrix4d cmvp;
+          matrixMult(activeView.cascade_model[i], activeView.cascade_proj[i], cmvp);
+          for (int j = 0; j < 4; j++) {
+            for (int k = 0; k < 4; k++) {
+              cscmvp[16 * i + 4 * j + k] = cmvp[j][k];
+              cscmodel[16 * i + 4 * j + k] = activeView.cascade_model[i][j][k];
+            }
+          }
+        }
+        glUniformMatrix4fv(glGetUniformLocation(shader, "cascade_mvp"), 3, GL_FALSE,
+                           (GLfloat *)cscmvp);
+        glUniformMatrix4fv(glGetUniformLocation(shader, "cascade_model"), 3, GL_FALSE,
+                           (GLfloat *)cscmodel);
+
+        glUniform3f(glGetUniformLocation(shader, "sun_direction"), activeView.sun_direction[0],
+                    activeView.sun_direction[1], activeView.sun_direction[2]);
+      } else {
+        glUniform1i(glGetUniformLocation(shader, "shadow_map"), 1);
+        glActiveTexture(GL_TEXTURE1);
+        glBindTexture(GL_TEXTURE_CUBE_MAP, activeView.shadowMapTexture);
+
+        glUniform3f(glGetUniformLocation(shader, "light_position"),
+                    activeView.light_position[0], activeView.light_position[1],
+                    activeView.light_position[2]);
+      }
+    }
+    if (type != Shader_Line) { glActiveTexture(GL_TEXTURE0 + 0); }
+  }
+}
+void setObjectUniforms(Color c, float shininess, Object_Lighting lighting) {
+  if (lastProgram == shaderObjectDay || lastProgram == shaderObjectNight) {
+    glUniformC(uniformLocations.specular_color, c);
+    glUniform1f(uniformLocations.shininess, shininess);
+    glUniform1i(uniformLocations.ignore_shadow, lighting == Lighting_Regular);
+    glUniform1i(uniformLocations.use_lighting, lighting != Lighting_None);
   }
-  setViewUniforms(shader);
-  glActiveTexture(GL_TEXTURE0 + 0);
 }
 
 void countObjectSpherePoints(int *ntriangles, int *nvertices, int detail) {
@@ -445,8 +603,8 @@ static void constructSphereData(int detail) {
   sphere_idxs[detail] = idxs;
 }
 
-void placeObjectSphere(void *data, ushort *idxs, ushort first_index, GLfloat position[3],
-                       Matrix3d rotation, GLfloat radius, int detail, GLfloat color[4]) {
+void placeObjectSphere(void *data, ushort *idxs, ushort first_index, GLfloat const position[3],
+                       Matrix3d rotation, GLfloat radius, int detail, const Color &color) {
   if (detail < 1) {
     warning("Sphere detail level must be > 1. Drawing nothing.");
     return;
@@ -459,7 +617,12 @@ void placeObjectSphere(void *data, ushort *idxs, ushort first_index, GLfloat pos
   int ntriangles = 2 * radial_count * (nrows - 2);
   for (int i = 0; i < 3 * ntriangles; i++) { idxs[i] = first_index + sphere_idxs[detail][i]; }
 
-  // Copy and transform vertices
+  /* cast to float early */
+  GLfloat rot[3][3];
+  for (int i = 0; i < 3; i++)
+    for (int j = 0; j < 3; j++) rot[i][j] = rotation[i][j];
+
+  /* Copy and transform vertices */
   char *pos = (char *)data;
   GLfloat *pts = sphere_points[detail];
   GLfloat *txs = sphere_texcos[detail];
@@ -469,7 +632,7 @@ void placeObjectSphere(void *data, ushort *idxs, ushort first_index, GLfloat pos
 
     GLfloat off[3] = {0.f, 0.f, 0.f};
     for (int j = 0; j < 3; j++)
-      for (int k = 0; k < 3; k++) off[j] += rotation[j][k] * loc[k];
+      for (int k = 0; k < 3; k++) off[j] += rot[j][k] * loc[k];
     pos += packObjectVertex(pos, position[0] + radius * off[0], position[1] + radius * off[1],
                             position[2] + radius * off[2], txc[0], txc[1], color, off);
   }
@@ -507,99 +670,6 @@ void lookAtMatrix(GLdouble eyeX, GLdouble eyeY, GLdouble eyeZ, GLdouble centerX,
   matrixMult(trans, mat, out);
 }
 
-void setViewUniforms(GLuint shader) {
-  GLfloat lproj[16], lmodel[16];
-  for (int i = 0; i < 4; i++) {
-    for (int j = 0; j < 4; j++) {
-      lproj[4 * i + j] = activeView.projection[i][j];
-      lmodel[4 * i + j] = activeView.modelview[i][j];
-    }
-  }
-
-  glUniformMatrix4fv(glGetUniformLocation(shader, "proj_matrix"), 1, GL_FALSE,
-                     (GLfloat *)lproj);
-  glUniformMatrix4fv(glGetUniformLocation(shader, "model_matrix"), 1, GL_FALSE,
-                     (GLfloat *)lmodel);
-  glUniform1i(glGetUniformLocation(shader, "fog_active"), activeView.fog_enabled);
-  glUniform3f(glGetUniformLocation(shader, "fog_color"), activeView.fog_color[0],
-              activeView.fog_color[1], activeView.fog_color[2]);
-  glUniform1f(glGetUniformLocation(shader, "fog_start"), activeView.fog_start);
-  glUniform1f(glGetUniformLocation(shader, "fog_end"), activeView.fog_end);
-
-  glUniform3f(glGetUniformLocation(shader, "light_position"), activeView.light_position[0],
-              activeView.light_position[1], activeView.light_position[2]);
-  glUniform3f(glGetUniformLocation(shader, "light_ambient"), activeView.light_ambient[0],
-              activeView.light_ambient[1], activeView.light_ambient[2]);
-  glUniform3f(glGetUniformLocation(shader, "light_diffuse"), activeView.light_diffuse[0],
-              activeView.light_diffuse[1], activeView.light_diffuse[2]);
-  glUniform3f(glGetUniformLocation(shader, "light_specular"), activeView.light_specular[0],
-              activeView.light_specular[1], activeView.light_specular[2]);
-  glUniform3f(glGetUniformLocation(shader, "global_ambient"), activeView.global_ambient[0],
-              activeView.global_ambient[1], activeView.global_ambient[2]);
-  glUniform3f(glGetUniformLocation(shader, "sun_direction"), activeView.sun_direction[0],
-              activeView.sun_direction[1], activeView.sun_direction[2]);
-  glUniform1f(glGetUniformLocation(shader, "day_mode"), activeView.day_mode ? 1. : -1.);
-  glUniform1f(glGetUniformLocation(shader, "quadratic_attenuation"),
-              activeView.quadratic_attenuation);
-
-  /* the shadow map is always the second texture sampling unit */
-  glUniform1i(glGetUniformLocation(shader, "shadow_map"), 1);
-  glActiveTexture(GL_TEXTURE1);
-  glBindTexture(GL_TEXTURE_CUBE_MAP, activeView.shadowMapTexture);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_BASE_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAX_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
-
-  glUniform1i(glGetUniformLocation(shader, "shadow_cascade0"), 2);
-  glActiveTexture(GL_TEXTURE2);
-  glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[0]);
-
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-
-  glUniform1i(glGetUniformLocation(shader, "shadow_cascade1"), 3);
-  glActiveTexture(GL_TEXTURE3);
-  glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[1]);
-
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-
-  glUniform1i(glGetUniformLocation(shader, "shadow_cascade2"), 4);
-  glActiveTexture(GL_TEXTURE4);
-  glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[2]);
-
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-
-  const int N = 3;
-  GLfloat cscproj[N * 16], cscmodel[N * 16];
-  for (int i = 0; i < N; i++) {
-    for (int j = 0; j < 4; j++) {
-      for (int k = 0; k < 4; k++) {
-        cscproj[16 * i + 4 * j + k] = activeView.cascade_proj[i][j][k];
-        cscmodel[16 * i + 4 * j + k] = activeView.cascade_model[i][j][k];
-      }
-    }
-  }
-
-  glUniformMatrix4fv(glGetUniformLocation(shader, "cascade_proj"), 3, GL_FALSE,
-                     (GLfloat *)cscproj);
-  glUniformMatrix4fv(glGetUniformLocation(shader, "cascade_model"), 3, GL_FALSE,
-                     (GLfloat *)cscmodel);
-
-  glActiveTexture(GL_TEXTURE0);
-}
-
 void renderDummyShadowMap() {
   GLenum dirs[6] = {GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
                     GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
@@ -623,12 +693,12 @@ void renderDummyShadowCascade() {
   }
 }
 
-void renderShadowMap(Coord3d focus, Map *mp, Game *gm) {
+void renderShadowMap(const Coord3d &focus, Map *mp, Game *gm) {
   Matrix4d origMV, origProj;
   assign(activeView.modelview, origMV);
   assign(activeView.projection, origProj);
 
-  activeView.calculating_shadows = 1;
+  activeView.calculating_shadows = true;
   static int ltexsize = 0;
   if (activeView.shadowMapTexsize <= 1 || ltexsize != Settings::settings->shadowTexsize) {
     ltexsize = Settings::settings->shadowTexsize;
@@ -683,7 +753,7 @@ void renderShadowMap(Coord3d focus, Map *mp, Game *gm) {
                  activeView.light_position[1] + norv[loop][1],
                  activeView.light_position[2] + norv[loop][2], upv[loop][0], upv[loop][1],
                  upv[loop][2], activeView.modelview);
-    updateUniforms();
+    markViewChanged();
     // Render (todo: 50% alpha clip)
     if (mp) mp->draw(0, focus[0], focus[1]);
     if (gm) gm->draw();
@@ -693,22 +763,13 @@ void renderShadowMap(Coord3d focus, Map *mp, Game *gm) {
   glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
   glDeleteFramebuffers(6, cubeFBOs);
 
-  activeView.calculating_shadows = 0;
+  activeView.calculating_shadows = false;
   assign(origMV, activeView.modelview);
   assign(origProj, activeView.projection);
-  updateUniforms();
+  markViewChanged();
 }
 
-void mulMatrix4(const Matrix4d mult, const double src[4], double res[4]) {
-  /* TODO: establish a proper Mat4, Mat3, Vec4, Vec3 set of types */
-  double r[4] = {0., 0., 0., 0.};
-  for (int i = 0; i < 4; i++) {
-    for (int j = 0; j < 4; j++) { r[i] += src[j] * mult[i][j]; }
-  }
-  for (int i = 0; i < 4; i++) { res[i] = r[i]; }
-}
-
-void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
+void renderShadowCascade(const Coord3d &focus, Map *mp, Game *gm) {
   Matrix4d origMV, origProj;
   assign(activeView.modelview, origMV);
   assign(activeView.projection, origProj);
@@ -752,7 +813,7 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
       for (int k = 0; k < 4; k++) {
         eyepts[k % 2] *= -1;
         /* convert to world space */
-        Coord3d resultA = useMatrix(mvm3t, eyepts) + camera;
+        Coord3d resultA = useMatrix(mvm3t, Coord3d(eyepts)) + camera;
 
         /* reorient along light space. NOTE: this breaks total offset, but we don't care about
          * it...*/
@@ -804,7 +865,7 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);
   glViewport(0, 0, activeView.cascadeTexsize, activeView.cascadeTexsize);
-  activeView.calculating_shadows = 1;
+  activeView.calculating_shadows = true;
 
   GLuint cascadeFBOs[N];
   glGenFramebuffers(N, cascadeFBOs);
@@ -825,7 +886,7 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     assign(activeView.cascade_model[loop], activeView.modelview);
     assign(activeView.cascade_proj[loop], activeView.projection);
-    updateUniforms();
+    markViewChanged();
     if (mp) mp->draw(0, focus[0], focus[1]);
     if (gm) gm->draw();
   }
@@ -836,8 +897,8 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
 
   assign(origMV, activeView.modelview);
   assign(origProj, activeView.projection);
-  activeView.calculating_shadows = 0;
-  updateUniforms();
+  activeView.calculating_shadows = false;
+  markViewChanged();
 }
 
 /* generates a snapshot of the screen */
@@ -845,11 +906,11 @@ int createSnapshot() {
   static int snap_number = 0;
   char name[1024];
   int again = 9999;
-  FILE *f;
 
   /* find the name for the image */
   do {
     snprintf(name, 1023, "./snapshot_%04d.png", snap_number++);
+    FILE *f;
     if ((f = fopen(name, "r")) == NULL) {
       break;
     } else {
@@ -961,10 +1022,6 @@ void multiMessage(int nlines, const char *left[], const char *right[]) {
     It is safe to load the same texture multiple times since the results are cached
 */
 int loadTexture(const char *name) {
-  GLfloat texCoord[4];
-  char str[256];
-  SDL_Surface *surface;
-
   /* Check in cache if texture already loaded */
   for (int i = 0; i < numTextures; i++)
     if (strcmp(textureNames[i], name) == 0) return i;
@@ -975,8 +1032,9 @@ int loadTexture(const char *name) {
     return 0;
   }
 
+  char str[256];
   snprintf(str, sizeof(str), "%s/images/%s", effectiveShareDir, name);
-  surface = IMG_Load(str);
+  SDL_Surface *surface = IMG_Load(str);
   if (!surface) {
     warning("Failed to load texture %s", str);
     // Override texture name... (to avoid carrying on outdated tex names)
@@ -986,8 +1044,8 @@ int loadTexture(const char *name) {
     return -1;
   } else {
     textureNames[numTextures] = strdup(name);
-    textures[numTextures] = LoadTexture(surface, texCoord);  // linear filter was: font != NULL
-    /*printf("loaded texture[%d]=%d\n",numTextures,textures[numTextures]);*/
+    GLfloat texCoord[4];
+    textures[numTextures] = LoadTexture(surface, texCoord);
     numTextures++;
     SDL_FreeSurface(surface);
   }
@@ -1005,29 +1063,37 @@ void glHelpInit() {
   ingameFont = TTF_OpenFont(str, 30);
   if (!ingameFont) { error("failed to load font %s", str); }
 
+  // The VAO need only be there :-)
+  glGenVertexArrays(1, &defaultVao);
+
   /* Note: all textures must be powers of 2 since we ignore texcoords */
   loadTexture("ice.png");
   loadTexture("acid.png");
   loadTexture("sand.png");
-  loadTexture("track.png");
+  textureTrack = textures[loadTexture("track.png")];
   loadTexture("texture.png");
   loadTexture("texture2.png");
   loadTexture("texture3.png");
   loadTexture("texture4.png");
-  loadTexture("wings.png");
-  loadTexture("mousePointer.png");
+  textureWings = textures[loadTexture("wings.png")];
+  textureMousePointer = textures[loadTexture("mousePointer.png")];
+  textureGlitter = textures[loadTexture("glitter.png")];
+  textureDizzy = textures[loadTexture("dizzy.png")];
+  textureBlank = textures[loadTexture("blank.png")];
 
   sparkle2D = new Sparkle2D();
 
-  // The VAO need only be there :-)
-  glGenVertexArrays(1, &theVao);
-
   // Errors handled within loadProgram
-  shaderTile = loadProgram("basic.vert", "basic.frag");
+  shaderTileDay = loadProgram("basic.vert", "basic_day.frag");
+  shaderTileNight = loadProgram("basic.vert", "basic_night.frag");
+  shaderTileShadow = loadProgram("basic.vert", "basic_shadow.frag");
   shaderLine = loadProgram("line.vert", "line.frag");
-  shaderWater = loadProgram("water.vert", "water.frag");
+  shaderWaterDay = loadProgram("water.vert", "water_day.frag");
+  shaderWaterNight = loadProgram("water.vert", "water_night.frag");
   shaderUI = loadProgram("ui.vert", "ui.frag");
-  shaderObject = loadProgram("object.vert", "object.frag");
+  shaderObjectDay = loadProgram("object.vert", "object_day.frag");
+  shaderObjectNight = loadProgram("object.vert", "object_night.frag");
+  shaderObjectShadow = loadProgram("object.vert", "object_shadow.frag");
   shaderReflection = loadProgram("reflection.vert", "reflection.frag");
 
   // Wipe ball cache
@@ -1057,19 +1123,22 @@ void glHelpInit() {
   activeView.light_specular[0] = 0.5f;
   activeView.light_specular[1] = 0.5f;
   activeView.light_specular[2] = 0.5f;
-  activeView.global_ambient[0] = 0.f;
-  activeView.global_ambient[1] = 0.f;
-  activeView.global_ambient[2] = 0.f;
-  activeView.quadratic_attenuation = 0.f;
   activeView.sun_direction[0] = 6 / 11.;
   activeView.sun_direction[1] = 34 / 121.;
   activeView.sun_direction[2] = -93 / 121.;
 
-  activeView.calculating_shadows = 0;
+  activeView.calculating_shadows = false;
 
   glEnable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
   glGenTextures(1, &activeView.shadowMapTexture);
   glBindTexture(GL_TEXTURE_CUBE_MAP, activeView.shadowMapTexture);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_BASE_LEVEL, 0);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAX_LEVEL, 0);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
 
   /* We don't use an array for the cascade texture because it reqs.
    * more shaders/etc */
@@ -1078,28 +1147,45 @@ void glHelpInit() {
   activeView.cascadeTexture[2] = 0;
 
   glGenTextures(3, activeView.cascadeTexture);
-  for (int i = 0; i < 3; i++) { glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[i]); }
+  for (int i = 0; i < 3; i++) {
+    glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[i]);
 
-  activeView.day_mode = 1;
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+  }
+
+  activeView.day_mode = true;
 
   renderDummyShadowCascade();
   renderDummyShadowMap();
   warnForGLerrors("postGLinit");
 }
 void glHelpCleanup() {
-  if (shaderTile) glDeleteProgram(shaderTile);
+  if (shaderTileDay) glDeleteProgram(shaderTileDay);
+  if (shaderTileNight) glDeleteProgram(shaderTileNight);
+  if (shaderTileShadow) glDeleteProgram(shaderTileShadow);
   if (shaderLine) glDeleteProgram(shaderLine);
-  if (shaderWater) glDeleteProgram(shaderWater);
+  if (shaderWaterDay) glDeleteProgram(shaderWaterDay);
+  if (shaderWaterNight) glDeleteProgram(shaderWaterNight);
   if (shaderUI) glDeleteProgram(shaderUI);
-  if (shaderObject) glDeleteProgram(shaderObject);
+  if (shaderObjectDay) glDeleteProgram(shaderObjectDay);
+  if (shaderObjectNight) glDeleteProgram(shaderObjectNight);
+  if (shaderObjectShadow) glDeleteProgram(shaderObjectShadow);
   if (shaderReflection) glDeleteProgram(shaderReflection);
-  if (theVao) glDeleteVertexArrays(1, &theVao);
+  if (defaultVao) glDeleteVertexArrays(1, &defaultVao);
   shaderLine = 0;
-  shaderTile = 0;
-  shaderWater = 0;
+  shaderTileDay = 0;
+  shaderTileNight = 0;
+  shaderTileShadow = 0;
+  shaderWaterDay = 0;
+  shaderWaterNight = 0;
   shaderUI = 0;
-  shaderObject = 0;
-  theVao = 0;
+  shaderObjectDay = 0;
+  shaderObjectNight = 0;
+  shaderObjectShadow = 0;
+  defaultVao = 0;
 
   if (sparkle2D) delete sparkle2D;
 
@@ -1215,53 +1301,41 @@ void warnForGLerrors(const char *where_am_i) {
 
 #define FRAME 50
 
-int resetTextures() {
-  GLfloat texCoord[4];
-  char str[256];
-  SDL_Surface *surface;
-
-  for (int i = 0; i < numTextures; i++) {
-    if (textureNames[i] == NULL) continue;
-    snprintf(str, sizeof(str), "%s/images/%s", effectiveShareDir, textureNames[i]);
-    surface = IMG_Load(str);
-    if (!surface) {
-      warning("Failed to load texture %s", str);
-      return 0;  // error (a valid texture entry)
-    } else {
-      glDeleteTextures(1, &textures[i]);
-      textures[i] = LoadTexture(surface, texCoord);
-      SDL_FreeSurface(surface);
-    }
-  }
-  return 1;
-}
-
 /* Calculates and displays current framerate */
 void displayFrameRate() {
   /* Frame rate is given for the previous period. */
   double td = getTimeDifference(lastDisplayStartTime, displayStartTime) / timeDilationFactor;
-  if (td > 1.0)
+  if (td > 1.0) {
     fps = 1.0;
-  else if (td <= 1e-4) {
+  } else if (td <= 1e-4) {
     static int hasWarned = 0;
     if (!hasWarned) {
       warning("Warning: too fast framerate (%f)", td);
       hasWarned = 1;
     }
-  } else
+  } else {
     fps = fps * 0.95 + 0.05 / td;
+  }
 
   if (Settings::settings->showFPS > 0) {
-    char str[256];
+    char slow[256];
+    char fast[256];
     if (Settings::settings->showFPS == 1) {
-      if (fps > 0)
-        snprintf(str, sizeof(str), _("Framerate: %.1f"), fps);
-      else
-        snprintf(str, sizeof(str), _("Framerate unknown"));
+      snprintf(slow, sizeof(slow), "%s ", _("Framerate:"));
+      if (fps > 0) {
+        snprintf(fast, sizeof(fast), "%.1f", fps);
+      } else {
+        snprintf(fast, sizeof(fast), _("unknown"));
+      }
     } else {
-      snprintf(str, sizeof(str), _("%.1f ms/Frame"), 1e3 * td);
+      snprintf(slow, sizeof(slow), "%s ", _("ms/Frame:"));
+      snprintf(fast, sizeof(fast), "%.1f", 1e3 * td);
     }
-    Font::drawSimpleText(str, 15, screenHeight - 15, 10., 1., 1., 0.25, 0.8);
+    TTF_Font *active = menuFontForSize(10);
+    int w1 = draw2DString(active, slow, 15, screenHeight - 15, menuColor[0], menuColor[1],
+                          menuColor[2], menuColor[3], true, 0, 0);
+    draw2DString(active, fast, 15 + w1, screenHeight - 15, menuColor[0], menuColor[1],
+                 menuColor[2], menuColor[3], true, 0, 0);
   }
 }
 
@@ -1270,10 +1344,10 @@ void displayFrameRate() {
 /*********************/
 
 void debugMatrix(Matrix4d m) {
-  printf("%f \t%f \t%f \t%f\n", m[0][0], m[0][1], m[0][2], m[0][3]);
-  printf("%f \t%f \t%f \t%f\n", m[1][0], m[1][1], m[1][2], m[1][3]);
-  printf("%f \t%f \t%f \t%f\n", m[2][0], m[2][1], m[2][2], m[2][3]);
-  printf("%f \t%f \t%f \t%f\n", m[3][0], m[3][1], m[3][2], m[3][3]);
+  warning("%8.5f \t%8.5f \t%8.5f \t%8.5f", m[0][0], m[0][1], m[0][2], m[0][3]);
+  warning("%8.5f \t%8.5f \t%8.5f \t%8.5f", m[1][0], m[1][1], m[1][2], m[1][3]);
+  warning("%8.5f \t%8.5f \t%8.5f \t%8.5f", m[2][0], m[2][1], m[2][2], m[2][3]);
+  warning("%8.5f \t%8.5f \t%8.5f \t%8.5f", m[3][0], m[3][1], m[3][2], m[3][3]);
 }
 
 /* C <- A * B */
@@ -1320,42 +1394,34 @@ void identityMatrix(Matrix4d m) {
 }
 
 void rotateX(double v, Matrix4d m) {
-  Matrix4d mr = {{1.0, 0.0, 0.0, 0.0},
-                 {0.0, cos(v), sin(v), 0.0},
-                 {0.0, -sin(v), cos(v), 0.0},
-                 {0.0, 0.0, 0.0, 1.0}};
-  Matrix4d morig;
-  assign(m, morig);
-  matrixMult(morig, mr, m);
+  double cv = std::cos(v), sv = std::sin(v);
+  for (int i = 0; i < 4; i++) {
+    double r1 = m[i][1], r2 = m[i][2];
+    m[i][1] = cv * r1 - sv * r2;
+    m[i][2] = sv * r1 + cv * r2;
+  }
 }
 void rotateY(double v, Matrix4d m) {
-  Matrix4d mr = {{cos(v), 0.0, sin(v), 0.0},
-                 {0.0, 1.0, 0.0, 0.0},
-                 {-sin(v), 0.0, cos(v), 0.0},
-                 {0.0, 0.0, 0.0, 1.0}};
-  Matrix4d morig;
-  assign(m, morig);
-  matrixMult(morig, mr, m);
+  double cv = std::cos(v), sv = std::sin(v);
+  for (int i = 0; i < 4; i++) {
+    double r0 = m[i][0], r2 = m[i][2];
+    m[i][0] = cv * r0 - sv * r2;
+    m[i][2] = sv * r0 + cv * r2;
+  }
 }
 void rotateZ(double v, Matrix4d m) {
-  Matrix4d mr = {{cos(v), sin(v), 0.0, 0.0},
-                 {-sin(v), cos(v), 0.0, 0.0},
-                 {0.0, 0.0, 1.0, 0.0},
-                 {0.0, 0.0, 0.0, 1.0}};
-  Matrix4d morig;
-  assign(m, morig);
-  matrixMult(morig, mr, m);
+  double cv = std::cos(v), sv = std::sin(v);
+  for (int i = 0; i < 4; i++) {
+    double r0 = m[i][0], r1 = m[i][1];
+    m[i][0] = cv * r0 - sv * r1;
+    m[i][1] = sv * r0 + cv * r1;
+  }
 }
-int testBboxClip(double x1, double x2, double y1, double y2, double z1, double z2,
-                 const Matrix4d model, const Matrix4d proj) {
+bool testBboxClip(double x1, double x2, double y1, double y2, double z1, double z2,
+                  const Matrix4d mvp) {
   // Construct axis-aligned bounding box in window space
   double vxl = 1e99, vyl = 1e99, vzl = 1e99;
   double vxh = -1e99, vyh = -1e99, vzh = -1e99;
-  Matrix4d mvp;
-  Matrix4d mmod, mproj;
-  memcpy(&mmod[0][0], model, sizeof(Matrix4d));
-  memcpy(&mproj[0][0], proj, sizeof(Matrix4d));
-  matrixMult(mmod, mproj, mvp);
   for (int i = 0; i < 8; i++) {
     double w[3] = {0., 0., 0.};
     double p[3] = {i & 4 ? x1 : x2, i & 2 ? y1 : y2, i & 1 ? z1 : z2};
@@ -1374,10 +1440,10 @@ int testBboxClip(double x1, double x2, double y1, double y2, double z1, double z
     vzh = std::max(vzh, w[2]);
   }
   // Window frustum is [-1,1]x[-1,1]x[0,1]
-  if (vxl > 1 || vxh < -1) return 0;
-  if (vyl > 1 || vyh < -1) return 0;
-  if (vzl > 1 || vzh < 0) return 0;
-  return 1;
+  if (vxl > 1 || vxh < -1) return false;
+  if (vyl > 1 || vyh < -1) return false;
+  if (vzl > 1 || vzh < 0) return false;
+  return true;
 }
 
 static int is_2d_mode = 0;
@@ -1398,7 +1464,7 @@ void Enter2DMode() {
   // Load program, bind VAO, and set up uniforms
   glUseProgram(shaderUI);
 
-  glBindVertexArray(theVao);
+  glBindVertexArray(defaultVao);
 
   glUniform1f(glGetUniformLocation(shaderUI, "screen_width"), (GLfloat)screenWidth);
   glUniform1f(glGetUniformLocation(shaderUI, "screen_height"), (GLfloat)screenHeight);
@@ -1422,6 +1488,8 @@ void Leave2DMode() {
   if (!is_2d_mode) { return; }
 
   is_2d_mode = 0;
+
+  glBindVertexArray(0);
 }
 
 int powerOfTwo(int input) {
