@@ -128,24 +128,17 @@ void loadScript(const char *path) {
     handleError(stack);
   }
 }
-
-static SCM sub_apply_0(void *body_data) {
-  SCM *args = (SCM *)body_data;
-  return scm_apply_0(args[0], SCM_EOL);
+static SCM sub_call_n(void *body_data) {
+  void **data = (void **)body_data;
+  SCM fun = (SCM)data[0];
+  SCM *args = (SCM *)data[1];
+  int n = *(int *)data[2];
+  return scm_call_n(fun, args, n);
 }
-static SCM sub_apply_1(void *body_data) {
-  SCM *args = (SCM *)body_data;
-  return scm_apply_1(args[0], args[1], SCM_EOL);
-}
-static SCM sub_apply_2(void *body_data) {
-  SCM *args = (SCM *)body_data;
-  return scm_apply_2(args[0], args[1], args[2], SCM_EOL);
-}
-
-SCM scm_catch_apply_0(SCM func) {
-  SCM args[1] = {func};
+SCM scm_catch_call_n(SCM func, SCM args[], int n) {
+  void *input[3] = {(void *)func, (void *)args, (void *)&n};
   SCM stack = SCM_BOOL_F;
-  SCM ret = scm_c_catch(SCM_BOOL_T, sub_apply_0, (void *)args, error_proc, NULL,
+  SCM ret = scm_c_catch(SCM_BOOL_T, sub_call_n, (void *)input, error_proc, NULL,
                         preunwind_proc, &stack);
   if (stack != SCM_BOOL_F) {
     handleError(stack);
@@ -153,29 +146,17 @@ SCM scm_catch_apply_0(SCM func) {
   }
   return ret;
 }
-
-SCM scm_catch_apply_1(SCM func, SCM arg1) {
-  SCM args[2] = {func, arg1};
-  SCM stack = SCM_BOOL_F;
-  SCM ret = scm_c_catch(SCM_BOOL_T, sub_apply_1, (void *)args, error_proc, NULL,
-                        preunwind_proc, &stack);
-  if (stack != SCM_BOOL_F) {
-    handleError(stack);
-    return SCM_UNSPECIFIED;
-  }
-  return ret;
+SCM scm_catch_call_0(SCM func) {
+  SCM args[1] = {NULL};
+  return scm_catch_call_n(func, args, 0);
 }
-
-SCM scm_catch_apply_2(SCM func, SCM arg1, SCM arg2) {
-  SCM args[3] = {func, arg1, arg2};
-  SCM stack = SCM_BOOL_F;
-  SCM ret = scm_c_catch(SCM_BOOL_T, sub_apply_2, (void *)args, error_proc, NULL,
-                        preunwind_proc, &stack);
-  if (stack != SCM_BOOL_F) {
-    handleError(stack);
-    return SCM_UNSPECIFIED;
-  }
-  return ret;
+SCM scm_catch_call_1(SCM func, SCM arg1) {
+  SCM args[1] = {arg1};
+  return scm_catch_call_n(func, args, 1);
+}
+SCM scm_catch_call_2(SCM func, SCM arg1, SCM arg2) {
+  SCM args[2] = {arg1, arg2};
+  return scm_catch_call_n(func, args, 2);
 }
 
 SCM scm_port_from_gzip(const char *path, int maxsize) {
@@ -225,7 +206,6 @@ SCM_DEFINE(translate_string, "translate", 1, 0, 0, (SCM str),
   SCM out = scm_from_utf8_string(trans);
   free(text);
   return out;
-  return out;
 }
 #undef FUNC_NAME
 
@@ -252,8 +232,11 @@ SCM_DEFINE(new_mr_black, "new-mr-black", 2, 0, 0, (SCM x, SCM y),
 {
   SCM_ASSERT(scm_is_real(x), x, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT(scm_is_real(y), y, SCM_ARG2, FUNC_NAME);
-  if (Game::current)
-    return smobAnimated_make(new Black(scm_to_double(x) + DX, scm_to_double(y) + DY));
+  if (Game::current) {
+    Black *black = new Black(scm_to_double(x) + DX, scm_to_double(y) + DY);
+    Game::current->add(black);
+    return smobAnimated_make(black);
+  }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -265,8 +248,11 @@ SCM_DEFINE(new_baby, "new-baby", 2, 0, 0, (SCM x, SCM y),
 {
   SCM_ASSERT(scm_is_real(x), x, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT(scm_is_real(y), y, SCM_ARG2, FUNC_NAME);
-  if (Game::current)
-    return smobAnimated_make(new Baby(scm_to_double(x) + DX, scm_to_double(y) + DY));
+  if (Game::current) {
+    Baby *baby = new Baby(scm_to_double(x) + DX, scm_to_double(y) + DY);
+    Game::current->add(baby);
+    return smobAnimated_make(baby);
+  }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -282,10 +268,13 @@ SCM_DEFINE(add_teleport, "add-teleport", 5, 0, 0, (SCM x, SCM y, SCM dx, SCM dy,
   SCM_ASSERT(scm_is_real(dx), dx, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_real(dy), dy, SCM_ARG4, FUNC_NAME);
   SCM_ASSERT(scm_is_real(radius), radius, SCM_ARG5, FUNC_NAME);
-  if (Game::current)
-    return smobAnimated_make(new Teleport(scm_to_double(x) + DX, scm_to_double(y) + DY,
-                                          scm_to_double(dx) + DX, scm_to_double(dy) + DY,
-                                          scm_to_double(radius)));
+  if (Game::current) {
+    Teleport *teleport =
+        new Teleport(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(dx) + DX,
+                     scm_to_double(dy) + DY, scm_to_double(radius));
+    Game::current->add(teleport);
+    return smobAnimated_make(teleport);
+  }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -302,10 +291,12 @@ SCM_DEFINE(add_bird, "add-bird", 6, 0, 0, (SCM x, SCM y, SCM dx, SCM dy, SCM siz
   SCM_ASSERT(scm_is_real(dy), dy, SCM_ARG4, FUNC_NAME);
   SCM_ASSERT(scm_is_real(size), size, SCM_ARG5, FUNC_NAME);
   SCM_ASSERT(scm_is_real(speed), speed, SCM_ARG5, FUNC_NAME);
-  if (Game::current)
-    return smobAnimated_make(new Bird(scm_to_double(x) + DX, scm_to_double(y) + DY,
-                                      scm_to_double(dx) + DX, scm_to_double(dy) + DY,
-                                      scm_to_double(size), scm_to_double(speed)));
+  if (Game::current) {
+    Bird *bird = new Bird(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(dx) + DX,
+                          scm_to_double(dy) + DY, scm_to_double(size), scm_to_double(speed));
+    Game::current->add(bird);
+    return smobAnimated_make(bird);
+  }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -320,9 +311,13 @@ SCM_DEFINE(add_flag, "add-flag", 5, 0, 0, (SCM x, SCM y, SCM points, SCM visible
   SCM_ASSERT(scm_is_integer(points), points, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_bool(visible), visible, SCM_ARG4, FUNC_NAME);
   SCM_ASSERT(scm_is_real(radius), radius, SCM_ARG5, FUNC_NAME);
-  Flag *flag = new Flag(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_int(points),
-                        scm_to_bool(visible), scm_to_double(radius));
-  return smobAnimated_make(flag);
+  if (Game::current) {
+    Flag *flag = new Flag(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_int(points),
+                          scm_to_bool(visible), scm_to_double(radius));
+    Game::current->add(flag);
+    return smobAnimated_make(flag);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -339,10 +334,14 @@ SCM_DEFINE(add_colormodifier, "add-colormodifier", 7, 0, 0,
   SCM_ASSERT(scm_is_real(max), max, SCM_ARG5, FUNC_NAME);
   SCM_ASSERT(scm_is_real(freq), freq, SCM_ARG6, FUNC_NAME);
   SCM_ASSERT(scm_is_real(phase), phase, SCM_ARG7, FUNC_NAME);
-  ColorModifier *colormodifier =
-      new ColorModifier(scm_to_int(col), scm_to_int(x), scm_to_int(y), scm_to_double(min),
-                        scm_to_double(max), scm_to_double(freq), scm_to_double(phase));
-  return smobAnimated_make(colormodifier);
+  if (Game::current) {
+    ColorModifier *colormodifier =
+        new ColorModifier(scm_to_int(col), scm_to_int(x), scm_to_int(y), scm_to_double(min),
+                          scm_to_double(max), scm_to_double(freq), scm_to_double(phase));
+    Game::current->add(colormodifier);
+    return smobGameHook_make(colormodifier);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -353,8 +352,6 @@ SCM_DEFINE(add_heightmodifier, "add-heightmodifier", 7, 3, 0,
            "Allow to modify the height of a cell's corner. Returns an 'animated' object")
 #define FUNC_NAME s_add_heightmodifier
 {
-  int not1 = -1, not2 = -1, not3 = -1;
-
   SCM_ASSERT(scm_is_integer(corner), corner, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT(scm_is_integer(x), x, SCM_ARG2, FUNC_NAME);
   SCM_ASSERT(scm_is_integer(y), y, SCM_ARG3, FUNC_NAME);
@@ -362,13 +359,19 @@ SCM_DEFINE(add_heightmodifier, "add-heightmodifier", 7, 3, 0,
   SCM_ASSERT(scm_is_real(max), max, SCM_ARG5, FUNC_NAME);
   SCM_ASSERT(scm_is_real(freq), freq, SCM_ARG6, FUNC_NAME);
   SCM_ASSERT(scm_is_real(phase), phase, SCM_ARG7, FUNC_NAME);
+
+  int not1 = -1, not2 = -1, not3 = -1;
   if (scm_is_real(n1)) not1 = scm_to_int(n1);
-  if (scm_is_real(n2)) not1 = scm_to_int(n2);
-  if (scm_is_real(n3)) not1 = scm_to_int(n3);
-  HeightModifier *heightmodifier = new HeightModifier(
-      scm_to_int(corner), scm_to_int(x), scm_to_int(y), scm_to_double(min), scm_to_double(max),
-      scm_to_double(freq), scm_to_double(phase), not1, not2, not3);
-  return smobAnimated_make(heightmodifier);
+  if (scm_is_real(n2)) not2 = scm_to_int(n2);
+  if (scm_is_real(n3)) not3 = scm_to_int(n3);
+  if (Game::current) {
+    HeightModifier *heightmodifier = new HeightModifier(
+        scm_to_int(corner), scm_to_int(x), scm_to_int(y), scm_to_double(min),
+        scm_to_double(max), scm_to_double(freq), scm_to_double(phase), not1, not2, not3);
+    Game::current->add(heightmodifier);
+    return smobGameHook_make(heightmodifier);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -380,9 +383,13 @@ SCM_DEFINE(add_cactus, "add-cactus", 3, 0, 0, (SCM x, SCM y, SCM radius),
   SCM_ASSERT(scm_is_real(x), x, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT(scm_is_real(y), y, SCM_ARG2, FUNC_NAME);
   SCM_ASSERT(scm_is_real(radius), radius, SCM_ARG5, FUNC_NAME);
-  Cactus *cactus =
-      new Cactus(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(radius));
-  return smobAnimated_make(cactus);
+  if (Game::current) {
+    Cactus *cactus =
+        new Cactus(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(radius));
+    Game::current->add(cactus);
+    return smobAnimated_make(cactus);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -398,8 +405,12 @@ SCM_DEFINE(add_spike, "add-spike", 4, 0, 0, (SCM x, SCM y, SCM speed, SCM phase)
   Coord3d pos;
   pos[0] = scm_to_double(x) + DX;
   pos[1] = scm_to_double(y) + DY;
-  Spike *spike = new Spike(pos, scm_to_double(speed), scm_to_double(phase));
-  return smobAnimated_make(spike);
+  if (Game::current) {
+    Spike *spike = new Spike(pos, scm_to_double(speed), scm_to_double(phase));
+    Game::current->add(spike);
+    return smobAnimated_make(spike);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -417,9 +428,13 @@ SCM_DEFINE(add_sidespike, "add-sidespike", 5, 0, 0,
   Coord3d pos;
   pos[0] = scm_to_double(x) + DX;
   pos[1] = scm_to_double(y) + DY;
-  SideSpike *sidespike =
-      new SideSpike(pos, scm_to_double(speed), scm_to_double(phase), scm_to_int(side));
-  return smobAnimated_make(sidespike);
+  if (Game::current) {
+    SideSpike *sidespike =
+        new SideSpike(pos, scm_to_double(speed), scm_to_double(phase), scm_to_int(side));
+    Game::current->add(sidespike);
+    return smobAnimated_make(sidespike);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -434,9 +449,14 @@ SCM_DEFINE(add_goal, "add-goal", 4, 0, 0, (SCM x, SCM y, SCM rotate, SCM nextLev
   SCM_ASSERT(scm_is_string(nextLevel), nextLevel, SCM_ARG4, FUNC_NAME);
   char *sname = scm_to_utf8_string(nextLevel);
   if (sname) {
-    Animated *a = (Animated *)new Goal(scm_to_double(x) + DX, scm_to_double(y) + DY,
-                                       scm_is_true(rotate), sname);
-    return smobAnimated_make(a);
+    if (Game::current) {
+      Goal *goal =
+          new Goal(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_is_true(rotate), sname);
+      Game::current->add(goal);
+      free(sname);
+      return smobAnimated_make(goal);
+    }
+    free(sname);
   }
   return SCM_UNSPECIFIED;
 }
@@ -460,10 +480,11 @@ SCM_DEFINE(sign, "sign", 6, 1, 0,
         pos[2] = scm_to_double(z);
       else
         pos[2] = Game::current->map->getHeight(pos[0], pos[1]) + 2.0;
-      Sign *s = new Sign(gettext(sname), scm_to_double(duration), scm_to_double(scale),
-                         scm_to_double(rotation), pos);
+      Sign *sign = new Sign(gettext(sname), scm_to_double(duration), scm_to_double(scale),
+                            scm_to_double(rotation), pos);
+      Game::current->add(sign);
       free(sname);
-      return smobAnimated_make(s);
+      return smobAnimated_make(sign);
     }
     free(sname);
   }
@@ -483,9 +504,14 @@ SCM_DEFINE(add_modpill, "add-modpill", 5, 0, 0,
   SCM_ASSERT(scm_is_real(kind), kind, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_real(length), length, SCM_ARG4, FUNC_NAME);
   SCM_ASSERT(scm_is_real(resurrecting), resurrecting, SCM_ARG5, FUNC_NAME);
-  return smobAnimated_make(new ModPill(scm_to_double(x) + DX, scm_to_double(y) + DY,
-                                       scm_to_int(kind), scm_to_int(length),
-                                       scm_to_int(resurrecting)));
+  if (Game::current) {
+    ModPill *modpill =
+        new ModPill(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_int(kind),
+                    scm_to_int(length), scm_to_int(resurrecting));
+    Game::current->add(modpill);
+    return smobAnimated_make(modpill);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -504,15 +530,20 @@ SCM_DEFINE(forcefield, "forcefield", 8, 0, 0,
   SCM_ASSERT(scm_is_real(height), height, SCM_ARG7, FUNC_NAME);
   SCM_ASSERT(scm_is_real(allow), allow, SCM_ARG7, FUNC_NAME);
 
-  Coord3d pos, dir;
-  pos[0] = scm_to_double(x) + DX;
-  pos[1] = scm_to_double(y) + DY;
-  pos[2] = Game::current->map->getHeight(pos[0], pos[1]) + scm_to_double(z);
-  dir[0] = scm_to_double(dx);
-  dir[1] = scm_to_double(dy);
-  dir[2] = scm_to_double(dz);
-  ForceField *ff = new ForceField(pos, dir, scm_to_double(height), scm_to_int(allow));
-  return smobAnimated_make(ff);
+  if (Game::current) {
+    Coord3d pos, dir;
+    pos[0] = scm_to_double(x) + DX;
+    pos[1] = scm_to_double(y) + DY;
+    pos[2] = Game::current->map->getHeight(pos[0], pos[1]) + scm_to_double(z);
+    dir[0] = scm_to_double(dx);
+    dir[1] = scm_to_double(dy);
+    dir[2] = scm_to_double(dz);
+
+    ForceField *ff = new ForceField(pos, dir, scm_to_double(height), scm_to_int(allow));
+    Game::current->add(ff);
+    return smobAnimated_make(ff);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -525,8 +556,12 @@ SCM_DEFINE(fun_switch, "switch", 4, 0, 0, (SCM x, SCM y, SCM on, SCM off),
   SCM_ASSERT(scm_is_real(y), y, SCM_ARG2, FUNC_NAME);
   SCM_ASSERT(scm_is_true(scm_procedure_p(on)), on, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_true(scm_procedure_p(off)), off, SCM_ARG3, FUNC_NAME);
-  CSwitch *sw = new CSwitch(scm_to_double(x) + DX, scm_to_double(y) + DY, on, off);
-  return smobAnimated_make(sw);
+  if (Game::current) {
+    CSwitch *sw = new CSwitch(scm_to_double(x) + DX, scm_to_double(y) + DY, on, off);
+    Game::current->add(sw);
+    return smobAnimated_make(sw);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -550,8 +585,12 @@ SCM_DEFINE(new_pipe, "pipe", 7, 0, 0,
   to[0] = scm_to_double(x1) + DX;
   to[1] = scm_to_double(y1) + DY;
   to[2] = scm_to_double(z1);
-  Pipe *pipe = new Pipe(from, to, scm_to_double(radius));
-  return smobAnimated_make(pipe);
+  if (Game::current) {
+    Pipe *pipe = new Pipe(from, to, scm_to_double(radius));
+    Game::current->add(pipe);
+    return smobAnimated_make(pipe);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -568,8 +607,12 @@ SCM_DEFINE(pipe_connector, "pipe-connector", 4, 0, 0, (SCM x0, SCM y0, SCM z0, S
   pos[0] = scm_to_double(x0) + DX;
   pos[1] = scm_to_double(y0) + DY;
   pos[2] = scm_to_double(z0);
-  PipeConnector *pipeConnector = new PipeConnector(pos, scm_to_double(radius));
-  return smobAnimated_make(pipeConnector);
+  if (Game::current) {
+    PipeConnector *pipeConnector = new PipeConnector(pos, scm_to_double(radius));
+    Game::current->add(pipeConnector);
+    return smobAnimated_make(pipeConnector);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -580,13 +623,18 @@ SCM_DEFINE(diamond, "diamond", 2, 1, 0, (SCM x, SCM y, SCM z),
 {
   SCM_ASSERT(scm_is_real(x), x, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT(scm_is_real(y), y, SCM_ARG2, FUNC_NAME);
-  Coord3d pos;
-  pos[0] = scm_to_double(x) + DX;
-  pos[1] = scm_to_double(y) + DY;
-  pos[2] = Game::current->map->getHeight(pos[0], pos[1]) + 0.4;
-  if (scm_is_real(z)) pos[2] = scm_to_double(z);
-  Diamond *d = new Diamond(pos);
-  return smobAnimated_make(d);
+  if (Game::current) {
+    Coord3d pos;
+    pos[0] = scm_to_double(x) + DX;
+    pos[1] = scm_to_double(y) + DY;
+    pos[2] = Game::current->map->getHeight(pos[0], pos[1]) + 0.4;
+    if (scm_is_real(z)) pos[2] = scm_to_double(z);
+
+    Diamond *diamond = new Diamond(pos);
+    Game::current->add(diamond);
+    return smobAnimated_make(diamond);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -602,12 +650,14 @@ SCM_DEFINE(fountain, "fountain", 6, 0, 0,
   SCM_ASSERT(scm_is_real(randomSpeed), randomSpeed, SCM_ARG4, FUNC_NAME);
   SCM_ASSERT(scm_is_real(radius), radius, SCM_ARG5, FUNC_NAME);
   SCM_ASSERT(scm_is_real(strength), strength, SCM_ARG6, FUNC_NAME);
-  Fountain *fountain =
-      new Fountain(scm_to_double(randomSpeed), scm_to_double(radius), scm_to_double(strength));
-  fountain->position[0] = scm_to_double(x) + DX;
-  fountain->position[1] = scm_to_double(y) + DY;
-  fountain->position[2] = scm_to_double(z);
-  return smobAnimated_make(fountain);
+  if (Game::current) {
+    Coord3d position(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(z));
+    Fountain *fountain = new Fountain(position, scm_to_double(randomSpeed),
+                                      scm_to_double(radius), scm_to_double(strength));
+    Game::current->add(fountain);
+    return smobAnimated_make(fountain);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -626,10 +676,8 @@ SCM_DEFINE(set_position, "set-position", 3, 1, 0, (SCM obj, SCM x, SCM y, SCM z)
   Animated *anim = (Animated *)SCM_CDR(obj);
   anim->position[0] = scm_to_double(x) + DX;
   anim->position[1] = scm_to_double(y) + DY;
-  if (scm_is_real(z))
-    anim->position[2] = scm_to_double(z);
-  else
-    anim->has_moved();
+  if (scm_is_real(z)) anim->position[2] = scm_to_double(z);
+  anim->drawChanged = true;
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -683,7 +731,10 @@ SCM_DEFINE(set_modtime, "set-modtime", 3, 0, 0, (SCM s_obj, SCM s_mod, SCM s_tim
   double time = scm_to_double(s_time);
   Animated *anim = (Animated *)SCM_CDR(s_obj);
   Ball *ball = dynamic_cast<Ball *>(anim);
-  if (ball) ball->modTimeLeft[mod] = time;
+  if (ball) {
+    ball->modTimeLeft[mod] = time;
+    ball->drawChanged = true;
+  }
   return s_obj;
 }
 #undef FUNC_NAME
@@ -733,10 +784,9 @@ SCM_DEFINE(set_primary_color, "set-primary-color", 4, 1, 0,
   SCM_ASSERT(scm_is_real(g), g, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_real(b), b, SCM_ARG4, FUNC_NAME);
   Animated *anim = (Animated *)SCM_CDR(obj);
-  anim->primaryColor[0] = scm_to_double(r);
-  anim->primaryColor[1] = scm_to_double(g);
-  anim->primaryColor[2] = scm_to_double(b);
-  if (scm_is_real(a)) anim->primaryColor[3] = scm_to_double(a);
+  anim->primaryColor = Color(scm_to_double(r), scm_to_double(g), scm_to_double(b),
+                             scm_is_real(a) ? scm_to_double(a) : 1.0);
+  anim->drawChanged = true;
   return obj;
 }
 #undef FUNC_NAME
@@ -752,10 +802,9 @@ SCM_DEFINE(set_secondary_color, "set-secondary-color", 4, 1, 0,
   SCM_ASSERT(scm_is_real(g), g, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_real(b), b, SCM_ARG4, FUNC_NAME);
   Animated *anim = (Animated *)SCM_CDR(obj);
-  anim->secondaryColor[0] = scm_to_double(r);
-  anim->secondaryColor[1] = scm_to_double(g);
-  anim->secondaryColor[2] = scm_to_double(b);
-  if (scm_is_real(a)) anim->secondaryColor[3] = scm_to_double(a);
+  anim->secondaryColor = Color(scm_to_double(r), scm_to_double(g), scm_to_double(b),
+                               scm_is_real(a) ? scm_to_double(a) : 1.0);
+  anim->drawChanged = true;
   return obj;
 }
 #undef FUNC_NAME
@@ -771,11 +820,10 @@ SCM_DEFINE(set_specular_color, "set-specular-color", 4, 1, 0,
   SCM_ASSERT(scm_is_real(g), g, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_real(b), b, SCM_ARG4, FUNC_NAME);
   Animated *anim = (Animated *)SCM_CDR(obj);
-  anim->specularColor[0] = scm_to_double(r);
-  anim->specularColor[1] = scm_to_double(g);
-  anim->specularColor[2] = scm_to_double(b);
-  if (scm_is_real(a)) anim->specularColor[3] = scm_to_double(a);
-  return SCM_UNSPECIFIED;
+  anim->specularColor = Color(scm_to_double(r), scm_to_double(g), scm_to_double(b),
+                              scm_is_real(a) ? scm_to_double(a) : 1.0);
+  anim->drawChanged = true;
+  return obj;
 }
 #undef FUNC_NAME
 
@@ -793,6 +841,7 @@ SCM_DEFINE(set_flag, "set-flag", 3, 0, 0, (SCM anim, SCM flag, SCM state),
     a->flags = a->flags & (~iflag);
   else
     a->flags = a->flags | iflag;
+  a->drawChanged = true;
   return anim;
 }
 #undef FUNC_NAME
@@ -844,6 +893,7 @@ SCM_DEFINE(set_texture, "set-texture", 2, 0, 0, (SCM obj, SCM tname),
     if (strcmp(name, textureNames[i]) == 0) {
       Animated *anim = (Animated *)SCM_CDR(obj);
       anim->texture = i;
+      anim->drawChanged = true;
       return obj;
     }
   return SCM_BOOL(false);
@@ -929,10 +979,14 @@ SCM_DEFINE(add_cyclic_platform, "add-cyclic-platform", 8, 0, 0,
   SCM_ASSERT(scm_is_real(high), high, SCM_ARG6, s_add_cyclic_platform);
   SCM_ASSERT(scm_is_real(offset), offset, SCM_ARG7, s_add_cyclic_platform);
   SCM_ASSERT(scm_is_real(speed), speed, SCM_ARG7, s_add_cyclic_platform);
-  CyclicPlatform *platform = new CyclicPlatform(
-      scm_to_int(x1), scm_to_int(y1), scm_to_int(x2), scm_to_int(y2), scm_to_double(low),
-      scm_to_double(high), scm_to_double(offset), scm_to_double(speed));
-  return smobGameHook_make(platform);
+  if (Game::current) {
+    CyclicPlatform *platform = new CyclicPlatform(
+        scm_to_int(x1), scm_to_int(y1), scm_to_int(x2), scm_to_int(y2), scm_to_double(low),
+        scm_to_double(high), scm_to_double(offset), scm_to_double(speed));
+    Game::current->add(platform);
+    return smobGameHook_make(platform);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -950,10 +1004,14 @@ SCM_DEFINE(
   SCM_ASSERT(scm_is_real(v1), v1, SCM_ARG5, FUNC_NAME);
   SCM_ASSERT(scm_is_integer(repeat), repeat, SCM_ARG6, FUNC_NAME);
   SCM_ASSERT(scm_is_bool(fun) | scm_is_true(scm_procedure_p(fun)), fun, SCM_ARG7, FUNC_NAME);
-  Animator *a = new Animator(scm_to_double(length), scm_to_double(position),
-                             scm_to_double(direction), scm_to_double(v0), scm_to_double(v1),
-                             scm_to_int(repeat), scm_is_bool(fun) ? NULL : fun);
-  return smobGameHook_make(a);
+  if (Game::current) {
+    Animator *a = new Animator(scm_to_double(length), scm_to_double(position),
+                               scm_to_double(direction), scm_to_double(v0), scm_to_double(v1),
+                               scm_to_int(repeat), scm_is_bool(fun) ? NULL : fun);
+    Game::current->add(a);
+    return smobGameHook_make(a);
+  }
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -970,6 +1028,8 @@ SCM_DEFINE(set_onoff, "set-onoff", 2, 0, 0, (SCM obj, SCM state),
   SCM_ASSERT(scm_is_bool(state), state, SCM_ARG2, FUNC_NAME);
   GameHook *h = (GameHook *)SCM_CDR(obj);
   h->is_on = scm_is_true(state);
+  Animated *a = dynamic_cast<Animated *>(h);
+  if (a) { a->drawChanged = true; }
   return obj;
 }
 #undef FUNC_NAME
@@ -1298,15 +1358,17 @@ SCM_DEFINE(set_cell_flag, "set-cell-flag", 6, 0, 0,
 
   int ix0 = scm_to_int(x0), iy0 = scm_to_int(y0), ix1 = scm_to_int(x1), iy1 = scm_to_int(y1),
       iflag = scm_to_int(flag);
-  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++)
+  bool istate = SCM_FALSEP(state);
+  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++) {
     for (int y = std::min(iy0, iy1); y <= std::max(iy0, iy1); y++) {
       Cell &c = Game::current->map->cell(x, y);
-      if (SCM_FALSEP(state))
+      if (istate)
         c.flags = c.flags & (~iflag);
       else
         c.flags = c.flags | iflag;
     }
-  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, 0);
+  }
+  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, false);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1341,13 +1403,15 @@ SCM_DEFINE(set_cell_velocity, "set-cell-velocity", 6, 0, 0,
   if (Game::current && Game::current->edit_mode) return SCM_UNSPECIFIED;
 
   int ix0 = scm_to_int(x0), iy0 = scm_to_int(y0), ix1 = scm_to_int(x1), iy1 = scm_to_int(y1);
-  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++)
+  float ivx = scm_to_double(vx), ivy = scm_to_double(vy);
+  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++) {
     for (int y = std::min(iy0, iy1); y <= std::max(iy0, iy1); y++) {
       Cell &c = Game::current->map->cell(x, y);
-      c.velocity[0] = scm_to_double(vx);
-      c.velocity[1] = scm_to_double(vy);
+      c.velocity[0] = ivx;
+      c.velocity[1] = ivy;
     }
-  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, 0);
+  }
+  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, false);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1369,19 +1433,25 @@ SCM_DEFINE(set_cell_heights, "set-cell-heights", 8, 1, 0,
   if (Game::current && Game::current->edit_mode) return SCM_UNSPECIFIED;
 
   int ix0 = scm_to_int(x0), iy0 = scm_to_int(y0), ix1 = scm_to_int(x1), iy1 = scm_to_int(y1);
-  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++)
+  double ih0 = scm_to_double(h0), ih1 = scm_to_double(h1), ih2 = scm_to_double(h2),
+         ih3 = scm_to_double(h3);
+  double ih4;
+  if (scm_is_real(h4)) {
+    ih4 = scm_to_double(h4);
+  } else {
+    ih4 = (ih0 + ih1 + ih2 + ih3) / 4.;
+  }
+  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++) {
     for (int y = std::min(iy0, iy1); y <= std::max(iy0, iy1); y++) {
       Cell &c = Game::current->map->cell(x, y);
-      c.heights[0] = scm_to_double(h0);
-      c.heights[1] = scm_to_double(h1);
-      c.heights[2] = scm_to_double(h2);
-      c.heights[3] = scm_to_double(h3);
-      if (scm_is_real(h4))
-        c.heights[4] = scm_to_double(h4);
-      else
-        c.heights[4] = (c.heights[0] + c.heights[1] + c.heights[2] + c.heights[3]) / 4.;
+      c.heights[0] = ih0;
+      c.heights[1] = ih1;
+      c.heights[2] = ih2;
+      c.heights[3] = ih3;
+      c.heights[4] = ih4;
     }
-  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, 1);
+  }
+  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, true);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1403,21 +1473,25 @@ SCM_DEFINE(set_cell_water_heights, "set-cell-water-heights", 8, 1, 0,
   if (Game::current && Game::current->edit_mode) return SCM_UNSPECIFIED;
 
   int ix0 = scm_to_int(x0), iy0 = scm_to_int(y0), ix1 = scm_to_int(x1), iy1 = scm_to_int(y1);
-  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++)
+  double ih0 = scm_to_double(h0), ih1 = scm_to_double(h1), ih2 = scm_to_double(h2),
+         ih3 = scm_to_double(h3);
+  double ih4;
+  if (scm_is_real(h4)) {
+    ih4 = scm_to_double(h4);
+  } else {
+    ih4 = (ih0 + ih1 + ih2 + ih3) / 4.;
+  }
+  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++) {
     for (int y = std::min(iy0, iy1); y <= std::max(iy0, iy1); y++) {
       Cell &c = Game::current->map->cell(x, y);
-      c.waterHeights[0] = scm_to_double(h0);
-      c.waterHeights[1] = scm_to_double(h1);
-      c.waterHeights[2] = scm_to_double(h2);
-      c.waterHeights[3] = scm_to_double(h3);
-      if (scm_is_real(h4))
-        c.waterHeights[4] = scm_to_double(h4);
-      else
-        c.waterHeights[4] =
-            (c.waterHeights[0] + c.waterHeights[1] + c.waterHeights[2] + c.waterHeights[3]) /
-            4.;
+      c.waterHeights[0] = ih0;
+      c.waterHeights[1] = ih1;
+      c.waterHeights[2] = ih2;
+      c.waterHeights[3] = ih3;
+      c.waterHeights[4] = ih4;
     }
-  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, 0);
+  }
+  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, false);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1441,18 +1515,16 @@ SCM_DEFINE(set_cell_colors, "set-cell-colors", 8, 1, 0,
   if (Game::current && Game::current->edit_mode) return SCM_UNSPECIFIED;
 
   int ix0 = scm_to_int(x0), iy0 = scm_to_int(y0), ix1 = scm_to_int(x1), iy1 = scm_to_int(y1);
-  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++)
+  GLfloat ir = scm_to_double(r), ig = scm_to_double(g), ib = scm_to_double(r);
+  GLfloat ia = scm_is_real(a) ? scm_to_double(a) : 1.0;
+  Color color(ir, ig, ib, ia);
+  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++) {
     for (int y = std::min(iy0, iy1); y <= std::max(iy0, iy1); y++) {
       Cell &c = Game::current->map->cell(x, y);
-      c.colors[i][0] = (GLfloat)scm_to_double(r);
-      c.colors[i][1] = (GLfloat)scm_to_double(g);
-      c.colors[i][2] = (GLfloat)scm_to_double(b);
-      if (scm_is_real(a))
-        c.colors[i][3] = (GLfloat)scm_to_double(a);
-      else
-        c.colors[i][3] = 1.0;
+      c.colors[i] = color;
     }
-  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, 0);
+  }
+  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, false);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1476,18 +1548,16 @@ SCM_DEFINE(set_cell_wall_colors, "set-cell-wall-colors", 8, 1, 0,
   if (Game::current && Game::current->edit_mode) return SCM_UNSPECIFIED;
 
   int ix0 = scm_to_int(x0), iy0 = scm_to_int(y0), ix1 = scm_to_int(x1), iy1 = scm_to_int(y1);
-  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++)
+  GLfloat ir = scm_to_double(r), ig = scm_to_double(g), ib = scm_to_double(r);
+  GLfloat ia = scm_is_real(a) ? scm_to_double(a) : 1.0;
+  Color color(ir, ig, ib, ia);
+  for (int x = std::min(ix0, ix1); x <= std::max(ix0, ix1); x++) {
     for (int y = std::min(iy0, iy1); y <= std::max(iy0, iy1); y++) {
       Cell &c = Game::current->map->cell(x, y);
-      c.wallColors[i][0] = (GLfloat)scm_to_double(r);
-      c.wallColors[i][1] = (GLfloat)scm_to_double(g);
-      c.wallColors[i][2] = (GLfloat)scm_to_double(b);
-      if (scm_is_real(a))
-        c.wallColors[i][3] = (GLfloat)scm_to_double(a);
-      else
-        c.wallColors[i][3] = 1.0;
+      c.wallColors[i] = color;
     }
-  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, 1);
+  }
+  Game::current->map->markCellsUpdated(ix0, iy0, ix1, iy1, true);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1539,9 +1609,9 @@ SCM_DEFINE(copy_cells, "copy-cells", 9, 0, 0,
     }
   }
   if (fxy) {
-    map->markCellsUpdated(tx, ty, tx + ys * h, ty + xs * w, 1);
+    map->markCellsUpdated(tx, ty, tx + ys * h, ty + xs * w, true);
   } else {
-    map->markCellsUpdated(tx, ty, tx + xs * w, ty + ys * h, 1);
+    map->markCellsUpdated(tx, ty, tx + xs * w, ty + ys * h, true);
   }
   delete[] buf;
   return SCM_UNSPECIFIED;
@@ -1645,7 +1715,12 @@ SCM_DEFINE(trigger, "trigger", 4, 0, 0, (SCM x, SCM y, SCM r, SCM expr),
   SCM_ASSERT(scm_is_real(y), y, SCM_ARG2, FUNC_NAME);
   SCM_ASSERT(scm_is_real(r), r, SCM_ARG3, FUNC_NAME);
   SCM_ASSERT(scm_is_true(scm_procedure_p(expr)), expr, SCM_ARG4, FUNC_NAME);
-  new Trigger(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(r), expr);
+  if (Game::current) {
+    Trigger *trigger =
+        new Trigger(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(r), expr);
+    Game::current->add(trigger);
+    return smobGameHook_make(trigger);
+  }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1667,8 +1742,12 @@ SCM_DEFINE(smart_trigger, "smart-trigger", 5, 0, 0,
     leaving = NULL;
   else
     SCM_ASSERT(scm_is_true(scm_procedure_p(leaving)), leaving, SCM_ARG5, FUNC_NAME);
-  new SmartTrigger(scm_to_double(x) + DX, scm_to_double(y) + DY, scm_to_double(r), entering,
-                   leaving);
+  if (Game::current) {
+    SmartTrigger *st = new SmartTrigger(scm_to_double(x) + DX, scm_to_double(y) + DY,
+                                        scm_to_double(r), entering, leaving);
+    Game::current->add(st);
+    return smobGameHook_make(st);
+  }
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
