@@ -25,7 +25,8 @@
 #include "map.h"
 
 HeightModifier::HeightModifier(int corner, int x, int y, Real min, Real max, Real freq,
-                               Real phase, int not1, int not2, int not3) {
+                               Real phase, int not1, int not2, int not3)
+    : GameHook(Role_GameHook) {
   this->x = x;
   this->y = y;
   this->min = min;
@@ -46,23 +47,11 @@ HeightModifier::HeightModifier(int corner, int x, int y, Real min, Real max, Rea
   this->not1 = not1;
   this->not2 = not2;
   this->not3 = not3;
-  // not used
-  position[0] = 0.;
-  position[1] = 0.;
-  position[2] = 0.;
-  primaryColor[0] = 1.;
-  primaryColor[1] = 1.;
-  primaryColor[2] = 1.;
-  secondaryColor[0] = 1.;
-  secondaryColor[1] = 1.;
-  secondaryColor[2] = 1.;
 }
 
-int HeightModifier::generateBuffers(GLuint*& /*idxbufs*/, GLuint*& /*databufs*/) { return 0; }
-void HeightModifier::drawBuffers1(GLuint* /*idxbufs*/, GLuint* /*databufs*/) {}
-void HeightModifier::drawBuffers2(GLuint* /*idxbufs*/, GLuint* /*databufs*/) {}
+void HeightModifier::tick(Real t) {
+  GameHook::tick(t);
 
-void HeightModifier::tick(Real /*t*/) {
   double tt = Game::current->gameTime;
   Cell& c = Game::current->map->cell(x, y);
   int x1 = 0, y1 = 0, x2 = 0, y2 = 0, x3 = 0, y3 = 0;
@@ -141,8 +130,8 @@ void HeightModifier::tick(Real /*t*/) {
       c3.heights[4] = (c3.heights[0] + c3.heights[1] + c3.heights[2] + c3.heights[3]) / 4.;
   }
 
-  Game::current->map->markCellsUpdated(x, y, x, y, 1);
-  Game::current->map->markCellsUpdated(x1, y1, x1, y1, 1);
-  Game::current->map->markCellsUpdated(x2, y2, x2, y2, 1);
-  Game::current->map->markCellsUpdated(x3, y3, x3, y3, 1);
+  Game::current->map->markCellsUpdated(x, y, x, y, true);
+  Game::current->map->markCellsUpdated(x1, y1, x1, y1, true);
+  Game::current->map->markCellsUpdated(x2, y2, x2, y2, true);
+  Game::current->map->markCellsUpdated(x3, y3, x3, y3, true);
 }
