@@ -23,6 +23,7 @@
 #include "font.h"
 #include "game.h"
 #include "gamer.h"
+#include "mainMode.h"
 #include "map.h"
 #include "menuMode.h"
 #include "menusystem.h"
@@ -33,26 +34,31 @@
 #include <SDL2/SDL_mouse.h>
 #include <SDL2/SDL_surface.h>
 
-HelpMode *HelpMode::helpMode;
-SDL_Surface *HelpMode::background;
+static HelpMode *helpMode = NULL;
 
 #define CODE_BACK 1
 #define CODE_MOREHELP 2
 
-void HelpMode::init() {
-  if (low_memory)
-    background = NULL;
-  else {
-    background = loadImage("helpBackground.jpg");
-  }
-
-  helpMode = new HelpMode();
+HelpMode *HelpMode::init() {
+  if (!helpMode) helpMode = new HelpMode();
+  return helpMode;
+}
+void HelpMode::cleanup() {
+  if (helpMode) delete helpMode;
 }
 HelpMode::HelpMode() {
   timeLeft = 0.;
   isExiting = 0;
   page = 0;
   helpGame = NULL;
+  if (low_memory)
+    background = NULL;
+  else {
+    background = loadImage("helpBackground.jpg");
+  }
+}
+HelpMode::~HelpMode() {
+  if (background) SDL_FreeSurface(background);
 }
 void HelpMode::activated() {
   if (!background) { background = loadImage("helpBackground.jpg"); }
@@ -123,16 +129,7 @@ void HelpMode::display() {
     activeView.fog_end = 26.0 - 4.0 * helpGame->fogThickness;
   } else
     activeView.fog_enabled = 0;
-  GLfloat sunLight[3] = {0.8, 0.8, 0.8};
-  GLfloat ambient[3] = {0.2, 0.2, 0.2};
-  GLfloat black[3] = {0.2, 0.2, 0.2};
-  GLfloat lightPosition[3] = {-100., -100., 150.};
-  assign(sunLight, activeView.light_diffuse);
-  assign(sunLight, activeView.light_specular);
-  assign(lightPosition, activeView.light_position);
-  assign(black, activeView.global_ambient);
-  assign(ambient, activeView.light_ambient);
-  activeView.quadratic_attenuation = 0.;
+  MainMode::setupLighting(false);
 
   /* Render two views of the level map */
   perspectiveMatrix(25, (GLdouble)screenWidth / (GLdouble)std::max(screenHeight, 1), 0.1, 200,
@@ -146,8 +143,7 @@ void HelpMode::display() {
   Coord3d cameraTo2(5. + step * page, 5. + step, -8.);
   lookAtMatrix(cameraFrom1[0], cameraFrom1[1], cameraFrom1[2], cameraTo1[0], cameraTo1[1],
                cameraTo1[2], 0.0, 0.0, 1.0, activeView.modelview);
-  activeView.day_mode = 1;
-  updateUniforms();
+  markViewChanged();
   if (Settings::settings->doShadows) {
     renderShadowCascade(cameraTo1, helpGame->map, helpGame);
   } else {
@@ -164,7 +160,7 @@ void HelpMode::display() {
   lookAtMatrix(cameraFrom2[0], cameraFrom2[1], cameraFrom2[2], cameraTo2[0], cameraTo2[1],
                cameraTo2[2], 0.0, 0.0, 1.0, activeView.modelview);
 
-  updateUniforms();
+  markViewChanged();
   if (Settings::settings->doShadows) {
     renderShadowCascade(cameraTo1, helpGame->map, helpGame);
   } else {
@@ -296,7 +292,7 @@ void HelpMode::key(int key) {
   if (key == SDLK_TAB) { moveKeyboardFocus(shift); }
   if (key == SDLK_RETURN || key == SDLK_KP_ENTER || key == SDLK_SPACE)
     mouseDown(shift ? 3 : 1, -1, -1);
-  if (key == SDLK_ESCAPE) GameMode::activate(MenuMode::menuMode);
+  if (key == SDLK_ESCAPE) GameMode::activate(MenuMode::init());
 }
 void HelpMode::idle(Real td) {
   helpGame->tick(td);
@@ -306,7 +302,7 @@ void HelpMode::idle(Real td) {
     timeLeft -= td;
   else
     timeLeft = fmin(1.0, timeLeft + td);
-  if (timeLeft < 0.0) MenuMode::activate(MenuMode::menuMode);
+  if (timeLeft < 0.0) GameMode::activate(MenuMode::init());
 }
 void HelpMode::mouseDown(int button, int /*x*/, int /*y*/) {
   int selected = getSelectedArea();
