@@ -25,15 +25,15 @@ class HighScore {
  public:
   HighScore();
 
-  static void init();
-  static HighScore *highScore;
+  static HighScore* init();
+  static void cleanup();
 
   int points[256][10];
   char names[256][10][25];
   int dummy_player[256][10];
 
   int isHighScore(int points); /* returns true if this is a highscore */
-  void addHighScore(int points, char *name);
+  void addHighScore(int points, char* name);
 };
 
 #endif
