@@ -40,21 +40,17 @@
 
 #define ENVIRONMENT_TEXTURE_SIZE 512
 
-Coord3d sunPosition(-20, -40, 40);
-Coord3d moonPosition(0, 0, 2);
-
-const int MainMode::statusBeforeGame = 0, MainMode::statusGameOver = 1,
-          MainMode::statusInGame = 2;
-const int MainMode::statusRestartPlayer = 3, MainMode::statusNextLevel = 4,
-          MainMode::statusVictory = 5;
-const int MainMode::statusLevelComplete = 6, MainMode::statusBonusLevelComplete = 7;
-const int MainMode::statusPaused = 8;
-
 static const int debug_shadowmap = 0;
 
 MainMode *MainMode::mainMode = NULL;
 
-void MainMode::init() { mainMode = new MainMode(); }
+MainMode *MainMode::init() {
+  if (!mainMode) mainMode = new MainMode();
+  return mainMode;
+}
+void MainMode::cleanup() {
+  if (mainMode) delete mainMode;
+}
 MainMode::MainMode() {
   flash = 0;
   zAngle = 0.;
@@ -63,7 +59,7 @@ MainMode::MainMode() {
   wantedXYAngle = 0.;
   camFocus = Coord3d();
   camDelta = Coord3d();
-  gameStatus = 0;
+  gameStatus = statusBeforeGame;
   statusCount = 0.;
   time = 0.;
   this->go_to_pause = 0;
@@ -75,8 +71,10 @@ MainMode::MainMode() {
 MainMode::~MainMode() {}
 
 void MainMode::display() {
-  Map *map = Game::current ? Game::current->map : NULL;
-  Player *player1 = Game::current ? Game::current->player1 : NULL;
+  if (!Game::current) { return; }
+
+  Map *map = Game::current->map;
+  Player *player1 = Game::current->player1;
 
   if (Game::current->fogThickness && Settings::settings->gfx_details != GFX_DETAILS_NONE) {
     activeView.fog_enabled = 1;
@@ -106,13 +104,12 @@ void MainMode::display() {
                  camFocus[0], camFocus[1], camFocus[2], up[0], up[1], up[2],
                  activeView.modelview);
   }
-  updateUniforms();
+  markViewChanged();
 
   /* lighting must be set before we render the shadow map */
   /* Shadow map rendering returns active modelview/projection to orig state */
-  setupLighting();
-  if (Game::current && Game::current->isNight) {
-    activeView.day_mode = 0;
+  setupLighting(Game::current->isNight);
+  if (Game::current->isNight) {
     if (Settings::settings->doShadows) {
       renderShadowMap(camFocus, map, Game::current);
       renderDummyShadowCascade();
@@ -121,7 +118,7 @@ void MainMode::display() {
       renderDummyShadowMap();
     }
   } else {
-    activeView.day_mode = 1;
+    activeView.day_mode = true;
     if (Settings::settings->doShadows) {
       renderShadowCascade(camFocus, map, Game::current);
       renderDummyShadowMap();
@@ -396,30 +393,19 @@ void MainMode::key(int key) {
     }
     break;
   case statusGameOver: {
-    /*	  int buyCost;
-    buyCost=(Game::current->player1->score/200)*100+500;
-    if(Game::current->player1->score >= buyCost && key == 'y') {
-          restartPlayer();
-          player1->lives++;
-          player1->timeLeft += 30;
-          gameStatus=statusInGame;
-          new ScoreSign(-buyCost,player1->position);
-          } else  */
-    if (key == ' ' || key == 'n') GameMode::activate(EnterHighScoreMode::enterHighScoreMode);
+    if (key == ' ' || key == 'n') GameMode::activate(EnterHighScoreMode::init());
   } break;
   case statusVictory:
     if (key == ' ') {
-      GameMode::activate(EnterHighScoreMode::enterHighScoreMode);
+      GameMode::activate(EnterHighScoreMode::init());
       Game::current->gamer->playerLose();
     }
     break;
   }
 }
-void MainMode::special(int /*key*/, int /*x*/, int /*y*/) {}
+
 void MainMode::idle(Real td) {
   Player *player1 = Game::current ? Game::current->player1 : NULL;
-  //   Map *map = Game::current ? Game::current->map : NULL;
-  double t;
 
   time += td;
   flash -= td;
@@ -477,7 +463,7 @@ void MainMode::idle(Real td) {
       xyAngle -= std::min(0.4 * td, xyAngle - wantedXYAngle);
 
     Game::current->tick(td);
-    for (t = td; t >= 0.0; t -= 0.01)
+    for (double t = td; t >= 0.0; t -= 0.01)
       for (int i = 0; i < 3; i++) {
         camDelta[i] = camDelta[i] * 0.9 + 0.002 * (player1->position[i] - camFocus[i]);
         camFocus[i] += camDelta[i];
@@ -543,7 +529,7 @@ void MainMode::startGame() {
   Game::current->gamer->levelStarted();
 
   pos[2] += 2.0;
-  new Sign(_("Good luck!"), 6, 1.0, 60.0, pos);
+  Game::current->add(new Sign(_("Good luck!"), 6, 1.0, 60.0, pos));
   Game::current->player1->position[2] += 1.0;
   Game::current->player1->triggerHook(GameHookEvent_Spawn, NULL);
 }
@@ -558,7 +544,7 @@ void MainMode::restartPlayer() {
 
   Coord3d pos = Game::current->map->startPosition;
   pos[2] += 2.0;
-  new Sign(_("Good luck!"), 7, 1.0, 60.0, pos);
+  Game::current->add(new Sign(_("Good luck!"), 7, 1.0, 60.0, pos));
   player1->triggerHook(GameHookEvent_Spawn, NULL);
 }
 
@@ -633,7 +619,6 @@ void MainMode::showInfo() {
                   0.8);
 
   Leave2DMode();
-  return;
 }
 void MainMode::levelComplete() {
   Game::current->player1->hasWon = 1;
@@ -673,7 +658,7 @@ void MainMode::bonusLevelComplete() {
           sizeof(Game::current->nextLevel));
 }
 
-void MainMode::renderEnvironmentTexture(GLuint texture, Coord3d focus) const {
+void MainMode::renderEnvironmentTexture(GLuint texture, const Coord3d &focus) const {
   // Save gfx_details and change before drawing environment */
   int gfx_details = Settings::settings->gfx_details;
   Settings::settings->gfx_details = GFX_DETAILS_MINIMALISTIC;
@@ -701,7 +686,7 @@ void MainMode::renderEnvironmentTexture(GLuint texture, Coord3d focus) const {
                    : focus[2] + 2.0,
                up[0], up[1], up[2], activeView.modelview);
 
-  setupLighting();
+  setupLighting(Game::current->isNight);
 
   /* Setup how we handle textures based on gfx_details */
   if (Settings::settings->gfx_details == 5) {
@@ -739,7 +724,7 @@ void MainMode::renderEnvironmentTexture(GLuint texture, Coord3d focus) const {
   glBindFramebuffer(GL_DRAW_FRAMEBUFFER, reflFBO);
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-  updateUniforms();
+  markViewChanged();
   Map *map = Game::current ? Game::current->map : NULL;
 
   map->draw(0, (int)focus[0] + 10, (int)focus[1] + 10);
@@ -752,9 +737,10 @@ void MainMode::renderEnvironmentTexture(GLuint texture, Coord3d focus) const {
   Settings::settings->gfx_details = gfx_details;
 }
 
-void MainMode::setupLighting() const {
+void MainMode::setupLighting(bool isNight) {
+  const Coord3d moonPosition(0., 0., 2.);
   GLfloat black[] = {0.0, 0.0, 0.0, 1.0};
-  if (Game::current && Game::current->isNight) {
+  if (isNight && Game::current && Game::current->player1) {
     GLfloat lightDiffuse2[3] = {0.9, 0.9, 0.9};
     GLfloat lightPosition2[3] = {
         (GLfloat)(Game::current->player1->position[0] + moonPosition[0]),
@@ -763,21 +749,14 @@ void MainMode::setupLighting() const {
     assign(lightDiffuse2, activeView.light_diffuse);
     assign(lightDiffuse2, activeView.light_specular);
     assign(lightPosition2, activeView.light_position);
-    assign(black, activeView.global_ambient);
     assign(black, activeView.light_ambient);
-    activeView.quadratic_attenuation = 0.25;
+    activeView.day_mode = false;
   } else {
     GLfloat sunLight[3] = {0.8, 0.8, 0.8};
     GLfloat ambient[3] = {0.2, 0.2, 0.2};
-    GLfloat lightPosition[3] = {
-        (GLfloat)(Game::current->player1->position[0] + sunPosition[0]),
-        (GLfloat)(Game::current->player1->position[1] + sunPosition[1]),
-        (GLfloat)(Game::current->player1->position[2] + sunPosition[2])};
     assign(sunLight, activeView.light_diffuse);
     assign(sunLight, activeView.light_specular);
-    assign(lightPosition, activeView.light_position);
-    assign(black, activeView.global_ambient);
     assign(ambient, activeView.light_ambient);
-    activeView.quadratic_attenuation = 0.;
+    activeView.day_mode = true;
   }
 }
