@@ -47,21 +47,21 @@ class Cell {
    * editMode::pasteRegion */
 
   float velocity[2], heights[5];
-  GLfloat colors[5][4];
-  GLfloat wallColors[4][4];
+  Color colors[5];
+  Color wallColors[4];
   float sunken;           // used for trampoline effect
   float waterHeights[5];  // absolute height of water in cell, values <= -100.0 is treated as
                           // no water.
 
   int flags, texture;
-  int displayListDirty;
+  bool displayListDirty;
 
   void dump(gzFile gp) const;
   void load(class Map *map, gzFile gp, int version);
 
   Cell();
-  void getNormal(Coord3d *, int vertex) const;
-  void getWaterNormal(Coord3d *, int vertex) const;
+  void getNormals(Coord3d normals[5]) const;
+  void getWaterNormals(Coord3d normals[5]) const;
   Real getHeight(Real x, Real y) const;
   Real getWaterHeight(Real x, Real y) const;
   inline int isWaterVisible() const {
@@ -69,7 +69,15 @@ class Cell {
            heights[2] < waterHeights[2] || heights[3] < waterHeights[3] ||
            heights[4] < waterHeights[4];
   }
+  /*
+   * (0,1) NW---NE (1,1) = (x,y)
+   *       | \ / |
+   *       |  C  | C=(0.5,0.5)
+   *       | / \ |
+   * (0,0) SW---SE (1,0)
+   */
   static const int NORTH = 1, SOUTH = 0, EAST = 2, WEST = 0, CENTER = 4;
+  static const int SW = SOUTH + WEST, NW = NORTH + WEST, SE = SOUTH + EAST, NE = NORTH + EAST;
 };
 
 class Chunk {
@@ -81,8 +89,10 @@ class Chunk {
   GLuint tile_vbo[2];
   GLuint flui_vbo[2];
   GLuint line_vbo[2];
+  GLuint wall_vao, tile_vao, flui_vao, line_vao;
   GLfloat maxHeight, minHeight;
-  int is_active, is_visible, is_updated, last_shown;
+  bool is_active, is_visible, is_updated;
+  int last_shown;
 };
 
 class Map {
@@ -96,7 +106,7 @@ class Map {
     return cells[lx + width * ly];
   };
   inline Chunk *chunk(int x, int y) const;
-  void markCellsUpdated(int x1, int y1, int x2, int y2, int changed_walls);
+  void markCellsUpdated(int x1, int y1, int x2, int y2, bool changed_walls);
   int save(char *name, int x, int y);
   void draw(int stage, int x, int y);
   void fillChunkVBO(Chunk *c) const;
