@@ -81,77 +81,6 @@ void changeScreenResolution() {
   int full = Settings::settings->is_windowed == 0;
   int fixed = Settings::settings->resolution >= 0;
 
-  if (window == NULL) {
-    char buffer[1024];
-    snprintf(buffer, sizeof(buffer), "/%s/ V%s", PACKAGE, VERSION);
-
-    if (Settings::settings->colorDepth == 16) {
-      SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
-      SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
-      SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
-    } else {
-      SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
-      SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
-      SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
-    }
-    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
-    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-
-    /* Version 3.3 is required for UNSIGNED_INT_2_10_10_10_REV packing method
-     * for the VertexAttribPointer function; the 3.2 spec does not permit it */
-    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
-    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
-    SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);
-
-    /* Uncomment to apply basic antialiasing.
-     * SDL can't recover from failure to obtain this if asked for */
-    //    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
-    //    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);
-
-    // Start at default size; will be adjusted
-    int windowHeight, windowWidth;
-    Uint32 flags = SDL_WINDOW_OPENGL;
-    if (fixed) {
-      windowWidth = screenResolutions[Settings::settings->resolution][0];
-      windowHeight = screenResolutions[Settings::settings->resolution][1];
-      if (full) { flags |= SDL_WINDOW_FULLSCREEN; }
-    } else {
-      if (full) {
-        flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
-      } else {
-        flags |= SDL_WINDOW_RESIZABLE;
-      }
-
-      SDL_Rect disprect;
-      if (!SDL_GetDisplayBounds(0, &disprect)) {
-        windowWidth = disprect.w / 2;
-        windowHeight = disprect.h / 2;
-      } else {
-        windowWidth = 800;
-        windowHeight = 600;
-      }
-    }
-    not_yet_windowed = full;
-
-    window = SDL_CreateWindow(buffer, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
-                              windowWidth, windowHeight, flags);
-
-    if (window == NULL) {
-      warning("Could not create window: %s", SDL_GetError());
-      return;
-    }
-
-    mainContext = SDL_GL_CreateContext(window);
-
-    char str[256];
-    snprintf(str, sizeof(str), "%s/icons/trackballs-128x128.png", effectiveShareDir);
-    SDL_Surface *wmIcon = IMG_Load(str);
-    if (wmIcon) {
-      SDL_SetWindowIcon(window, wmIcon);
-      SDL_FreeSurface(wmIcon);
-    }
-  }
-
   if (fixed) {
     SDL_SetWindowSize(window, screenResolutions[Settings::settings->resolution][0],
                       screenResolutions[Settings::settings->resolution][1]);
@@ -205,11 +134,85 @@ void changeScreenResolution() {
 
   /* Adjust for size change in editmode */
   if (EditMode::editMode) { EditMode::editMode->resizeWindows(); }
-
-  resetTextures();
 }
 
-void print_usage(FILE *stream) {
+static void createWindow() {
+  int full = Settings::settings->is_windowed == 0;
+  int fixed = Settings::settings->resolution >= 0;
+
+  char buffer[1024];
+  snprintf(buffer, sizeof(buffer), "/%s/ V%s", PACKAGE, VERSION);
+
+  if (Settings::settings->colorDepth == 16) {
+    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
+    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
+    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
+  } else {
+    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
+    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
+    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
+  }
+  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
+  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
+
+  /* Version 3.3 is required for UNSIGNED_INT_2_10_10_10_REV packing method
+   * for the VertexAttribPointer function; the 3.2 spec does not permit it */
+  SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
+  SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
+  SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);
+
+  /* Uncomment to apply basic antialiasing.
+   * SDL can't recover from failure to obtain this if asked for */
+  //    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
+  //    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);
+
+  // Start at default size; will be adjusted
+  int windowHeight, windowWidth;
+  Uint32 flags = SDL_WINDOW_OPENGL;
+  if (fixed) {
+    windowWidth = screenResolutions[Settings::settings->resolution][0];
+    windowHeight = screenResolutions[Settings::settings->resolution][1];
+    if (full) { flags |= SDL_WINDOW_FULLSCREEN; }
+  } else {
+    if (full) {
+      flags |= SDL_WINDOW_FULLSCREEN_DESKTOP;
+    } else {
+      flags |= SDL_WINDOW_RESIZABLE;
+    }
+
+    SDL_Rect disprect;
+    if (!SDL_GetDisplayBounds(0, &disprect)) {
+      windowWidth = disprect.w / 2;
+      windowHeight = disprect.h / 2;
+    } else {
+      windowWidth = 800;
+      windowHeight = 600;
+    }
+  }
+  not_yet_windowed = full;
+
+  window = SDL_CreateWindow(buffer, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
+                            windowWidth, windowHeight, flags);
+
+  if (window == NULL) {
+    warning("Could not create window: %s", SDL_GetError());
+    return;
+  }
+
+  mainContext = SDL_GL_CreateContext(window);
+
+  char str[256];
+  snprintf(str, sizeof(str), "%s/icons/trackballs-128x128.png", effectiveShareDir);
+  SDL_Surface *wmIcon = IMG_Load(str);
+  if (wmIcon) {
+    SDL_SetWindowIcon(window, wmIcon);
+    SDL_FreeSurface(wmIcon);
+  }
+
+  changeScreenResolution();
+}
+
+static void print_usage(FILE *stream) {
   fprintf(stream, "%s %s %s\n", _("Usage:"), program_name,
           _("[-w, -m] [-e, -l -t <level>] [-r <width>] [-s <sensitivity>]"));
   const char *options[12][2] = {
@@ -269,7 +272,6 @@ int testDir() {
 }
 
 void innerMain(void * /*closure*/, int argc, char **argv) {
-  int is_running = 1;
   int editMode = 0, touchMode = 0;
   int audio = SDL_INIT_AUDIO;
   SDL_Event event;
@@ -368,7 +370,8 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
       repair_joystick = 1;
       break;
     default:
-      abort();
+      print_usage(stderr);
+      exit(1);
     }
   } while (next_option != -1);
 
@@ -384,7 +387,7 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
   }
   atexit(SDL_Quit);
 
-  changeScreenResolution();
+  createWindow();
   if (!screen) {
     error("Could not initialize screen resolution (message: '%s')", SDL_GetError());
   }
@@ -435,36 +438,16 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
     map->save(mapname, (int)map->startPosition[0], (int)map->startPosition[1]);
     exit(0);
   } else if (editMode) {
-    EditMode::init();
     Font::init();
-    GameMode::activate(EditMode::editMode);
+    GameMode::activate(EditMode::init());
   } else {
     Font::init();
-    MenuMode::init();
-
-    /* Initialize modes */
-    MainMode::init();
-    HighScore::init();
-    EditMode::init();
-    EnterHighScoreMode::init();
-    HallOfFameMode::init();
-    SettingsMode::init();
-    SetupMode::init();
-    CalibrateJoystickMode::init();
-    HelpMode::init();
-
-    /* Initialize game structures */
-    GameHook::init();
-    Ball::init();
-    ForceField::init();
-    Pipe::init();
-    PipeConnector::init();
 
     /* Activate initial mode */
     if (Settings::settings->doSpecialLevel) {
-      GameMode::activate(SetupMode::setupMode);
+      GameMode::activate(SetupMode::init());
     } else {
-      GameMode::activate(MenuMode::menuMode);
+      GameMode::activate(MenuMode::init());
     }
     volumeChanged();
   }
@@ -488,17 +471,16 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
   /* Main event loop */
   /*                 */
 
-  double td = 1. / 60.;
-
   /* Initialize random number generator */
   int seed = getMonotonicTime().tv_nsec;
   srand(seed);
   int keyUpReceived = 1;
 
+  bool is_running = true;
   while (is_running) {
     theFrameNumber++;
 
-    td = getTimeDifference(lastDisplayStartTime, displayStartTime);
+    double td = getTimeDifference(lastDisplayStartTime, displayStartTime);
     // If timing doesn't work assume 60fps
     if (td <= 0.0) { td = 1. / 60.; }
     // No slower than 5 fps
@@ -515,9 +497,9 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
 
     /* Draw world */
     glViewport(0, 0, screenWidth, screenHeight);
-    if (GameMode::current)
+    if (GameMode::current) {
       GameMode::current->display();
-    else {
+    } else {
       glClear(GL_COLOR_BUFFER_BIT);
     }
     warnForGLerrors("uncaught from display routine");
@@ -533,19 +515,11 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
     /*                */
     /* Process events */
     /*                */
-
-    /* Joystick - this was a bug since joystick data are also queried using SDL_JoystickUpdate
-     */
-    /*
-    if(Settings::settings->hasJoystick()) SDL_JoystickEventState(SDL_ENABLE);
-    else SDL_JoystickEventState(SDL_DISABLE);
-    */
-
     SDL_MouseButtonEvent *e = (SDL_MouseButtonEvent *)&event;
     while (SDL_PollEvent(&event)) {
       switch (event.type) {
       case SDL_QUIT:
-        is_running = 0;
+        is_running = false;
         break;
       case SDL_MOUSEBUTTONDOWN:
         if (GameMode::current) GameMode::current->mouseDown(e->button, e->x, e->y);
@@ -591,9 +565,9 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
           if (editMode) {
             ((EditMode *)GameMode::current)->askQuit();
           } else if ((GameMode::current && GameMode::current == MenuMode::menuMode))
-            is_running = 0;
+            is_running = false;
           else {
-            GameMode::activate(MenuMode::menuMode);
+            GameMode::activate(MenuMode::init());
             /* Flush all events that occured while switching screen
              resolution */
             while (SDL_PollEvent(&event)) {}
@@ -624,15 +598,25 @@ void innerMain(void * /*closure*/, int argc, char **argv) {
 
   Settings::settings->closeJoystick();
   Settings::settings->save();
-  if (GameMode::current) delete (GameMode::current);
 
-  /* TODO. Delete all gamemodes, including the editor */
+  /* Delete all game modes */
+  CalibrateJoystickMode::cleanup();
+  EditMode::cleanup();
+  EnterHighScoreMode::cleanup();
+  HallOfFameMode::cleanup();
+  HelpMode::cleanup();
+  MainMode::cleanup();
+  MenuMode::cleanup();
+  SettingsMode::cleanup();
+  SetupMode::cleanup();
+
+  HighScore::cleanup();
   glHelpCleanup();
+
   SDL_Quit();
 }
 
 int main(int argc, char **argv) {
-  char guileLoadPath[256 + 16]; /*longest effective share directory plus"GUILE_LOAD_PATH="*/
   program_name = argv[0];
 
   /*** Autmatic detection of SHARE_DIR ***/
@@ -688,6 +672,7 @@ int main(int argc, char **argv) {
   }
 
   if (NULL == getenv("GUILE_LOAD_PATH")) {
+    char guileLoadPath[256 + 16]; /*longest effective share directory plus"GUILE_LOAD_PATH="*/
     snprintf(guileLoadPath, sizeof(guileLoadPath), "GUILE_LOAD_PATH=%s", effectiveShareDir);
     putenv(guileLoadPath);
   }
