@@ -31,15 +31,14 @@
 int isGoodPill[NUM_MODS] = {1, 1, 1, 0, 0, 0, 1, 1};
 
 ModPill::ModPill(Real x, Real y, int kind, int time, int resurrecting)
-    : Ball(), kind(kind), resurrecting(resurrecting), time(time) {
-  no_physics = 1;
+    : Ball(Role_OtherAnimated), kind(kind), resurrecting(resurrecting), time(time) {
+  no_physics = true;
   realRadius = 0.2;
   radius = realRadius;
+  is_on = true;
 
   /* Change our color to red */
-  primaryColor[0] = 1.0;
-  primaryColor[1] = 0.2;
-  primaryColor[2] = 0.2;
+  primaryColor = Color(1., 0.2, 0.2, 1.0);
 
   /* set bogus velocity for the rendering of speed mods */
   if (kind == MOD_SPEED) {
@@ -55,29 +54,26 @@ ModPill::ModPill(Real x, Real y, int kind, int time, int resurrecting)
   if (kind == MOD_EXTRA_LIFE) {
     realRadius = 0.3;
     radius = realRadius;
-    primaryColor[0] = 1.0;
-    primaryColor[1] = 0.9;
-    primaryColor[2] = 0.2;
-    specularColor[0] = 1.0;
-    specularColor[1] = 0.9;
-    specularColor[2] = 0.2;
+    primaryColor = Color(1., 0.9, 0.2, 1.);
+    specularColor = Color(1., 0.9, 0.2, 1.);
   }
 
   position[0] = x;
   position[1] = y;
   position[2] = Game::current->map->getHeight(position[0], position[1]) + radius;
 
-  alive = 1;
   timeLeft = 0.;
 }
 
 ModPill::~ModPill() { this->Animated::~Animated(); }
 
 void ModPill::tick(Real t) {
+  Animated::tick(t);
+
   Player *player = Game::current->player1;
-  if (!alive) {
+  if (!is_on) {
     timeLeft -= t;
-    if (resurrecting > 0.0 && timeLeft < 0) alive = 1;
+    if (resurrecting > 0.0 && timeLeft < 0) is_on = true;
   }
 
   clock += t;
@@ -89,9 +85,9 @@ void ModPill::tick(Real t) {
     radius = realRadius;
   }
 
-  if (alive && kind == MOD_NITRO) Ball::generateNitroDebris(t);
+  if (is_on && kind == MOD_NITRO) Ball::generateNitroDebris(t);
 
-  if (alive) {
+  if (is_on) {
     position[2] = Game::current->map->getHeight(position[0], position[1]) + radius;
     Coord3d v = player->position - position;
     double dist = length(v);
@@ -105,7 +101,7 @@ void ModPill::tick(Real t) {
           _("Speed ball"), _("Extra jump"), _("Spikes"),   _("Glass ball"),
           _("Dizzy!"),     _("Freeze!"),    _("Floating"), _("Extra life"),
           _("Small ball"), _("Large ball"), _("Nitro")};
-      new Sign(modExplanations[kind], 6.0, 1.0, 60.0, signPos);
+      Game::current->add(new Sign(modExplanations[kind], 6.0, 1.0, 60.0, signPos));
 
       if (kind == MOD_EXTRA_LIFE) {
         player->lives = std::min(4, player->lives + 1);
@@ -125,14 +121,14 @@ void ModPill::tick(Real t) {
       else
         playEffect(SFX_GOT_BADPILL);
 
-      alive = 0;
+      is_on = false;
       if (!resurrecting) remove();
       timeLeft = (Real)resurrecting;
     }
   }
 }
-void ModPill::playerRestarted() {}
+
 void ModPill::die(int /*how*/) {
-  alive = 0;
-  if (!resurrecting) delete this;
+  is_on = false;
+  if (!resurrecting) remove();
 }
