@@ -20,35 +20,31 @@
 
 #include "pipe.h"
 
-std::set<Pipe *> *Pipe::pipes;
+#include "game.h"
 
-void Pipe::init() { pipes = new std::set<Pipe *>(); }
-void Pipe::reset() {
-  delete pipes;
-  pipes = new std::set<Pipe *>();
-}
-Pipe::Pipe(Coord3d f, Coord3d t, Real r) : Animated() {
+Pipe::Pipe(const Coord3d &f, const Coord3d &t, Real r)
+    : Animated(Role_Pipe, 1), from(f), to(t), radius(r) {
   /* Note that the position attribute of Pipes are not used, use rather the to/from values */
-  from = f;
-  to = t;
-  radius = r;
-  pipes->insert(this);
-  primaryColor[0] = primaryColor[1] = primaryColor[2] = 0.6;
+  position = 0.5 * (from + to);
+  primaryColor = Color(0.6, 0.6, 0.6, 1.0);
 
-  windForward = windBackward = 0.0;
+  windBackward = 0.0;
   windForward = 0.0;
-  computeBoundingBox();
+
+  for (int i = 0; i < 3; i++) {
+    boundingBox[0][i] = fmin(from[i] - radius, to[i] - radius) - position[i];
+    boundingBox[1][i] = fmax(from[i] + radius, to[i] + radius) - position[i];
+  }
 }
 
-int Pipe::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  allocateBuffers(1, idxbufs, databufs);
-  up[0] = up[1] = 0.0;
-  up[2] = 1.0;
+void Pipe::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                           const GLuint *vaolist, bool mustUpdate) const {
+  if (!mustUpdate) return;
+
+  Coord3d up(0., 0., 0.);
   Coord3d dir = to - from;
   dir = dir / length(dir);
-  right[0] = dir[1];
-  right[1] = -dir[0];
-  right[2] = 0.0;
+  Coord3d right(dir[1], -dir[0], 0.0);
   right = right / length(right);
   up = crossProduct(dir, right);
   if (up[2] < 0.0) up = -up;
@@ -85,55 +81,36 @@ int Pipe::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
     idxs[2 * i + 1][2] = 2 * j + 1;
   }
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, sizeof(data), data, GL_STATIC_DRAW);
-
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idxs), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void Pipe::drawBuffers1(GLuint *idxbufs, GLuint *databufs) {
-  if (primaryColor[3] >= 1.0) drawTrunk(idxbufs, databufs);
+void Pipe::drawBuffers1(const GLuint *vaolist) const {
+  if (primaryColor.v[3] >= 65535) drawTrunk(vaolist);
 }
-void Pipe::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
-  if (activeView.calculating_shadows && primaryColor[3] < 0.7) return;
-  if (primaryColor[3] < 1.0) drawTrunk(idxbufs, databufs);
+void Pipe::drawBuffers2(const GLuint *vaolist) const {
+  if (activeView.calculating_shadows && primaryColor.v[3] < 45000) return;
+  if (primaryColor.v[3] < 65535) drawTrunk(vaolist);
 }
-void Pipe::drawTrunk(GLuint *idxbufs, GLuint *databufs) {
+void Pipe::drawTrunk(const GLuint *vaolist) const {
   // Keep off unconditionally since pipe ends show both sides
   glDisable(GL_CULL_FACE);
-  if (primaryColor[3] < 1.f) {
+  if (primaryColor.v[3] < 65535) {
     glEnable(GL_BLEND);
   } else {
     glDisable(GL_BLEND);
   }
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0] * 0.1,
-              specularColor[1] * 0.1, specularColor[2] * 0.1, 1.);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 128.f / 128.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 1., Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
   int nfacets = 24;
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 3 * 2 * nfacets, GL_UNSIGNED_SHORT, (void *)0);
 }
-void Pipe::tick(Real /*t*/) {
-  /* TODO. Ugly fix, a better fix is to make from/to etc. *private* variables and add this fn.
-         inside the accessor methods */
-  computeBoundingBox();
-}
-void Pipe::computeBoundingBox() {
-  for (int i = 0; i < 3; i++) {
-    boundingBox[0][i] = fmin(from[i] - radius, to[i] - radius) - position[i];
-    boundingBox[1][i] = fmax(from[i] + radius, to[i] + radius) - position[i];
-  }
-}
-void Pipe::onRemove() {
-  Animated::onRemove();
-  pipes->erase(this);
-}
+void Pipe::tick(Real t) { Animated::tick(t); }
