@@ -24,34 +24,25 @@
 #define PIPE_SOFT_ENTER 1
 #define PIPE_SOFT_EXIT 2
 
-#include <set>
 #include "animated.h"
 
 class Pipe : public Animated {
  public:
-  Pipe(Coord3d from, Coord3d to, Real radius);
+  Pipe(const Coord3d& from, const Coord3d& to, Real radius);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
-  void tick(Real t);
-  /** Recomputes the bounding box of the pipe. Needed after changes in from/to position */
-  void computeBoundingBox();
-  void onRemove();
+  virtual void tick(Real t);
 
-  static void init();
-  static void reset();
+  const Coord3d from, to;
+  const Real radius;
 
-  Coord3d from, to;
-  Coord3d up, right;
-  Real radius;
-
-  static std::set<Pipe*>* pipes;
   Real windForward, windBackward;
 
  private:
-  void drawTrunk(GLuint* idxbufs, GLuint* databufs);
+  void drawTrunk(const GLuint* vaolist) const;
 };
 
 #endif
