@@ -20,21 +20,24 @@
 
 #include "pipeConnector.h"
 
-std::set<PipeConnector *> *PipeConnector::connectors;
-void PipeConnector::init() { connectors = new std::set<PipeConnector *>(); }
-void PipeConnector::reset() {
-  delete connectors;
-  connectors = new std::set<PipeConnector *>();
-}
-PipeConnector::PipeConnector(Coord3d pos, Real r) : Animated() {
+#include "game.h"
+
+PipeConnector::PipeConnector(const Coord3d &pos, Real r)
+    : Animated(Role_PipeConnector, 1), radius(r) {
   position = pos;
-  radius = r;
-  primaryColor[0] = primaryColor[1] = primaryColor[2] = 0.6;
-  connectors->insert(this);
+  primaryColor = Color(0.6, 0.6, 0.6, 1.0);
+
+  boundingBox[0][0] = -radius;
+  boundingBox[0][1] = -radius;
+  boundingBox[0][2] = -radius;
+  boundingBox[1][0] = radius;
+  boundingBox[1][1] = radius;
+  boundingBox[1][2] = radius;
 }
 
-int PipeConnector::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  allocateBuffers(1, idxbufs, databufs);
+void PipeConnector::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                                    const GLuint *vaolist, bool mustUpdate) const {
+  if (!mustUpdate) return;
 
   int ntries = 0;
   int nverts = 0;
@@ -47,24 +50,24 @@ int PipeConnector::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
 
   placeObjectSphere(data, idxs, 0, pos, identity, radius, detail, primaryColor);
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, nverts * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, ntries * 3 * sizeof(ushort), idxs, GL_STATIC_DRAW);
+  configureObjectAttributes();
   delete[] data;
   delete[] idxs;
-
-  return 1;
 }
-void PipeConnector::drawBuffers1(GLuint *idxbufs, GLuint *databufs) {
-  if (primaryColor[3] >= 1.0) drawMe(idxbufs, databufs);
+void PipeConnector::drawBuffers1(const GLuint *vaolist) const {
+  if (primaryColor.v[3] >= 65535) drawMe(vaolist);
 }
-void PipeConnector::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
-  if (activeView.calculating_shadows && primaryColor[3] < 0.7) return;
-  if (primaryColor[3] < 1.0) drawMe(idxbufs, databufs);
+void PipeConnector::drawBuffers2(const GLuint *vaolist) const {
+  if (activeView.calculating_shadows && primaryColor.v[3] < 45000) return;
+  if (primaryColor.v[3] < 65535) drawMe(vaolist);
 }
-void PipeConnector::drawMe(GLuint *idxbufs, GLuint *databufs) {
-  if (primaryColor[3] < 1.0f) {
+void PipeConnector::drawMe(const GLuint *vaolist) const {
+  if (primaryColor.v[3] < 65535) {
     glEnable(GL_BLEND);
     glDisable(GL_CULL_FACE);
   } else {
@@ -77,26 +80,11 @@ void PipeConnector::drawMe(GLuint *idxbufs, GLuint *databufs) {
   int detail = 6;
   countObjectSpherePoints(&ntries, &nverts, detail);
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0] * 0.1,
-              specularColor[1] * 0.1, specularColor[2] * 0.1, 1.);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 128.f / 128.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 1., Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 3 * ntries, GL_UNSIGNED_SHORT, (void *)0);
 }
-void PipeConnector::tick(Real t) {
-  boundingBox[0][0] = -radius;
-  boundingBox[0][1] = -radius;
-  boundingBox[0][2] = -radius;
-  boundingBox[1][0] = radius;
-  boundingBox[1][1] = radius;
-  boundingBox[1][2] = radius;
-}
-void PipeConnector::onRemove() {
-  Animated::onRemove();
-  connectors->erase(this);
-}
+void PipeConnector::tick(Real t) { Animated::tick(t); }
