@@ -23,27 +23,20 @@
 
 #include "animated.h"
 
-#include <set>
-
 class PipeConnector : public Animated {
  public:
-  PipeConnector(Coord3d pos, Real radius);
+  PipeConnector(const Coord3d& pos, Real radius);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(Real t);
-  void onRemove();
 
-  Real radius;
-
-  static void init();
-  static void reset();
-  static class std::set<PipeConnector*>* connectors;
+  const Real radius;
 
  private:
-  void drawMe(GLuint* idxbufs, GLuint* databufs);
+  void drawMe(const GLuint* vaolist) const;
 };
 
 #endif
