@@ -32,7 +32,7 @@
 #include <SDL2/SDL_keyboard.h>
 #include <SDL2/SDL_mouse.h>
 
-Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
+static Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
   /* Finite impulse response filter to smooth mouse input */
   const int N = 1000;
   const double T = 0.2;
@@ -83,9 +83,9 @@ Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
   return mouseState;
 }
 
-Player::Player() : Ball() {
-  inTheAir = 0;
-  inPipe = 0;
+Player::Player() : Ball(Role_Player) {
+  inTheAir = false;
+  inPipe = false;
   lives = 3;
   realRadius = 0.3;
   radius = realRadius;
@@ -93,13 +93,8 @@ Player::Player() : Ball() {
   ballResolution = BALL_HIRES;
 
   /* Change our color to red */
-  primaryColor[0] = 1.0;
-  primaryColor[1] = 0.2;
-  primaryColor[2] = 0.2;
-
-  specularColor[0] = 1.0;
-  specularColor[1] = 1.0;
-  specularColor[2] = 1.0;
+  primaryColor = Color(1., 0.2, 0.2, 1.0);
+  specularColor = Color(0.5, 0.5, 0.5, 1.);
 
   timeLeft = 5;
   bounceFactor = .7;
@@ -119,10 +114,6 @@ Player::Player() : Ball() {
 }
 
 Player::~Player() {}
-void Player::draw() {
-  if (!playing) return;
-  Ball::draw();
-}
 void Player::tick(Real t) {
   static Real timeFraction = 0.;
 
@@ -317,25 +308,27 @@ void Player::die(int how) {
   playing = false;
   health = 0.0;
 
-  Coord3d pos, vel;
   if (how == DIE_ACID) {
-    GLfloat acidColor[4] = {0.1, 0.8, 0.1, 0.5};
-    Coord3d vel;
+    Color acidColor(0.1, 0.8, 0.1, 0.5);
+    Coord3d vel(0., 0., 1.);
     Coord3d center(position[0], position[1], map->getHeight(position[0], position[1]));
-    new Splash(center, vel, acidColor, 32.0, radius);
-  } else
-    for (int i = 0; i < 4; i++)
+    Game::current->add(new Splash(center, vel, acidColor, 32.0, radius));
+  } else {
+    for (int i = 0; i < 4; i++) {
       for (int j = 0; j < 4; j++) {
         Real a = i / 4.0 * M_PI2;
         Real b = (j + 0.5) / 4.0 * M_PI;
+        Coord3d pos, vel;
         pos[0] = position[0] + cos(a) * 0.25 * sin(b) * 2.0;
         pos[1] = position[1] + sin(a) * 0.25 * sin(b) * 2.0;
         pos[2] = position[2] + 0.25 * cos(b) + 0.5;
         vel[0] = velocity[0] + (sink ? 0.1 : 0.5) * 1 / 2048.0 * ((rand() % 2048) - 1024);
         vel[1] = velocity[1] + (sink ? 0.1 : 0.5) * 1 / 2048.0 * ((rand() % 2048) - 1024);
         vel[2] = velocity[2] + (sink ? 0.01 : 0.5) * 1 / 2048.0 * ((rand() % 2048) - 1024);
-        new Debris(this, pos, vel, 2.0 + 8.0 * frandom());
+        Game::current->add(new Debris(this, pos, vel, 2.0 + 8.0 * frandom()));
       }
+    }
+  }
 
   if (how == DIE_CRASH)
     playEffect(SFX_PLAYER_DIES);
@@ -348,7 +341,7 @@ void Player::die(int how) {
     ((MainMode *)GameMode::current)->playerLose();
   else
     ((MainMode *)GameMode::current)->playerDie();
-  alive = 0;
+  is_on = false;
 }
 
 void Player::setStartVariables() {
@@ -356,16 +349,16 @@ void Player::setStartVariables() {
   rotation[0] = rotation[1] = 0.;
   velocity = Coord3d();
   playing = true;
-  alive = 1;
   hasWon = 0;
   health = 1.0;
   oxygen = 1.0;
-  inTheAir = 0;
-  inPipe = 0;
+  inTheAir = false;
+  inPipe = false;
   moveBurst = 0.0;
+  is_on = true;
 }
 
-void Player::restart(Coord3d pos) {
+void Player::restart(const Coord3d &pos) {
   setStartVariables();
   position = pos;
   position[2] = Game::current->map->getHeight(position[0], position[1]) + radius;
@@ -376,7 +369,7 @@ void Player::restart(Coord3d pos) {
     modTimeLeft[i] = 0.0;*/
 }
 void Player::mouse(int /*state*/, int /*x*/, int /*y*/) {}
-void Player::newLevel() { Game::current->balls->insert(this); }
+void Player::newLevel() {}
 void Player::setHealth(Real d) {
   if (d < health) health = d;
   if (health < 0.0) health = 0.0;
