@@ -28,13 +28,12 @@ class Player : public Ball {
   Player();
   virtual ~Player();
 
-  void draw();
   void tick(Real time);
   void key(int k);
   void die(int);
   void newLevel();
-  void setStartVariables(); /**< Setup all default variables when starting a level */
-  void restart(Coord3d);    /**< Restart the ball at given position */
+  void setStartVariables();     /**< Setup all default variables when starting a level */
+  void restart(const Coord3d&); /**< Restart the ball at given position */
   void mouse(int state, int x, int y);
   void setHealth(Real);
   virtual bool crash(Real speed);
