@@ -28,11 +28,11 @@
 
 class ScoreSign : public Sign {
  public:
-  ScoreSign(int points, Coord3d position);
-  ScoreSign(int points, Coord3d position, int type);
+  ScoreSign(int points, const Coord3d& position);
+  ScoreSign(int points, const Coord3d& position, int type);
   void init(int points, int type);
 
-  void tick(Real);
+  virtual void tick(Real);
 
  private:
   /** If the bonus given is of another type than score */
