@@ -48,27 +48,33 @@
 #define CODE_START 5
 #define CODE_COLOR 6
 
-SetupMode *SetupMode::setupMode = NULL;
-SDL_Surface *SetupMode::background;
+Color colors[5] = {Color(1.0, 0.2, 0.2, 1.0), Color(0.2, 1.0, 0.2, 1.0),
+                   Color(0.5, 0.5, 1.0, 1.0), Color(1.0, 1.0, 0.2, 1.0),
+                   Color(1.0, 1.0, 1.0, 1.0)};
 
-GLfloat colors[5][3] = {
-    {1.0, 0.2, 0.2}, {0.2, 1.0, 0.2}, {0.5, 0.5, 1.0}, {1.0, 1.0, 0.2}, {1.0, 1.0, 1.0}};
-
-void SetupMode::init() {
-  if (low_memory)
-    background = NULL;
-  else {
-    background = loadImage("setupBackground.jpg");
-  }
-
-  setupMode = new SetupMode();
+static SetupMode *setupMode = NULL;
+SetupMode *SetupMode::init() {
+  if (!setupMode) setupMode = new SetupMode();
+  return setupMode;
 }
+void SetupMode::cleanup() {
+  if (setupMode) delete setupMode;
+}
+
 SetupMode::SetupMode() : GameMode() {
   gamer = new Gamer();
   levelSet = level = 0;
   screenshot = 0;
   t = 0.;
   name = 0;
+  if (low_memory)
+    background = NULL;
+  else {
+    background = loadImage("setupBackground.jpg");
+  }
+}
+SetupMode::~SetupMode() {
+  if (background) SDL_FreeSurface(background);
 }
 void SetupMode::display() {
   Settings *settings = Settings::settings;
@@ -107,6 +113,8 @@ void SetupMode::display() {
   /* Draw the player ball */
   /*                      */
 
+  warnForGLerrors("start of setup ball");
+
   Matrix4d persp_trans = {{1.f, 0.f, 0.f, 0.f},
                           {0.f, 1.f, 0.f, 0.f},
                           {0.f, 0.f, 1.f, 0.f},
@@ -121,18 +129,9 @@ void SetupMode::display() {
   glEnable(GL_CULL_FACE);
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);
-  GLfloat lightDiffuse[3] = {0.9, 0.9, 0.9};
-  GLfloat ambient[3] = {0.2, 0.2, 0.2};
-  GLfloat black[3] = {0., 0., 0.};
-  GLfloat lightPosition[3] = {-100.0, -50.0, 150.0};
-  assign(lightPosition, activeView.light_position);
-  assign(lightDiffuse, activeView.light_diffuse);
-  assign(lightDiffuse, activeView.light_specular);
-  assign(black, activeView.global_ambient);
-  assign(ambient, activeView.light_ambient);
-  activeView.quadratic_attenuation = 0.0;
+  MainMode::setupLighting(false);
   activeView.fog_enabled = 0;
-  updateUniforms();
+  markViewChanged();
 
   if (settings->gfx_details == 5) {
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -150,8 +149,7 @@ void SetupMode::display() {
   GLfloat *data = new GLfloat[nverts * 8];
   ushort *idxs = new ushort[ntries * 3];
   GLfloat pos[3] = {0.f, 0.f, 0.f};
-  GLfloat color[4] = {colors[gamer->color][0], colors[gamer->color][1],
-                      colors[gamer->color][2], 1.f};
+  Color color = colors[gamer->color];
   Matrix4d frommtx;
   identityMatrix(frommtx);
   rotateY(t * -0.3, frommtx);
@@ -162,16 +160,18 @@ void SetupMode::display() {
   placeObjectSphere(data, idxs, 0, pos, rotation, 1.0, detail, color);
 
   // Transfer
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), 0.3f, 0.3f, 0.3f, 0.3f);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 20.f);
-  glUniform1f(glGetUniformLocation(shaderObject, "ignore_shadow"), 1.);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(Color(0.3, 0.3, 0.3, 0.3), 20., Lighting_NoShadows);
   glBindTexture(GL_TEXTURE_2D, textures[gamer->textureNum]);
 
-  GLuint databuf, idxbuf;
+  warnForGLerrors("trans");
+
+  GLuint databuf, idxbuf, vao;
   glGenBuffers(1, &databuf);
   glGenBuffers(1, &idxbuf);
+  glGenVertexArrays(1, &vao);
 
+  glBindVertexArray(vao);
   glBindBuffer(GL_ARRAY_BUFFER, databuf);
   glBufferData(GL_ARRAY_BUFFER, nverts * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbuf);
@@ -182,9 +182,12 @@ void SetupMode::display() {
   configureObjectAttributes();
   glDrawElements(GL_TRIANGLES, 3 * ntries, GL_UNSIGNED_SHORT, (void *)0);
 
+  glBindVertexArray(0);
   glDeleteBuffers(1, &databuf);
   glDeleteBuffers(1, &idxbuf);
+  glDeleteVertexArrays(1, &vao);
 
+  warnForGLerrors("end of setup ball");
   Enter2DMode();
 
   char str[256];
@@ -283,7 +286,7 @@ void SetupMode::key(int key) {
   if (key == SDLK_RETURN || key == SDLK_KP_ENTER ||
       (key == SDLK_SPACE && selected != CODE_NAME))
     mouseDown(shift ? 3 : 1, -1, -1);
-  if (key == SDLK_ESCAPE) GameMode::activate(MenuMode::menuMode);
+  if (key == SDLK_ESCAPE) GameMode::activate(MenuMode::init());
 
   if (selected == CODE_NAME) {
     if (key == SDLK_BACKSPACE) {
@@ -350,9 +353,9 @@ void SetupMode::start() {
   } else {
     Game::current = new Game(gamer->levels[levelSet][level].fileName, gamer);
     Game::current->currentLevelSet = levelSet;
-    HallOfFameMode::hallOfFameMode->levelSet = levelSet;
+    if (HallOfFameMode::hallOfFameMode) HallOfFameMode::hallOfFameMode->levelSet = levelSet;
   }
-  GameMode::activate(MainMode::mainMode);
+  GameMode::activate(MainMode::init());
 }
 
 void SetupMode::mouseDown(int button, int x, int y) {
