@@ -29,7 +29,8 @@
 #include "sound.h"
 #include "spike.h"
 
-SideSpike::SideSpike(Coord3d position, Real speed, Real phase, int side) {
+SideSpike::SideSpike(const Coord3d &position, Real speed, Real phase, int side)
+    : Animated(Role_OtherAnimated, 1) {
   this->position = position;
   this->position[2] = Game::current->map->getHeight(position[0], position[1]) + 0.0;
   this->speed = speed;
@@ -44,90 +45,77 @@ SideSpike::SideSpike(Coord3d position, Real speed, Real phase, int side) {
   this->soundDone = 0;
   this->offset = 0.;
 
-  primaryColor[0] = 0.9;
-  primaryColor[1] = 0.5;
-  primaryColor[2] = 0.4;
-
-  secondaryColor[0] = 0.9;
-  secondaryColor[1] = 0.8;
-  secondaryColor[2] = 0.5;
-
-  specularColor[0] = 0.1;
-  specularColor[1] = 0.1;
-  specularColor[2] = 0.1;
+  primaryColor = Color(0.9, 0.5, 0.4, 1.0);
+  secondaryColor = Color(0.9, 0.8, 0.5, 1.0);
+  specularColor = Color(0.1, 0.1, 0.1, 1.0);
 }
 
-int SideSpike::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  allocateBuffers(1, idxbufs, databufs);
-
+void SideSpike::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                                const GLuint *vaolist, bool /*mustUpdate*/) const {
   const int nfacets = 6;
   GLfloat data[(4 * nfacets) * 8];
   ushort idxs[3 * nfacets][3];
 
-  Matrix4d frommtx;
-  identityMatrix(frommtx);
+  Matrix3d rotmtx1 = {{0, 0, 1}, {0, 1, 0}, {-1, 0, 0}};
+  Matrix3d rotmtx2 = {{0, 0, -1}, {0, 1, 0}, {1, 0, 0}};
+  Matrix3d rotmtx3 = {{1, 0, 0}, {0, 0, 1}, {0, -1, 0}};
+  Matrix3d rotmtx4 = {{1, 0, 0}, {0, 0, -1}, {0, 1, 0}};
+
+  Matrix3d *rot;
 
   Coord3d pos(position[0], position[1], position[2] + 0.25);
   switch (side) {
   case 1:
-    rotateY(M_PI / 2, frommtx);
+    rot = &rotmtx1;
     pos[0] += -0.5 + offset;
     break;
   case 2:
-    rotateY(-M_PI / 2, frommtx);
+    rot = &rotmtx2;
     pos[0] += 0.5 - offset;
     break;
   case 3:
-    rotateX(M_PI / 2, frommtx);
+    rot = &rotmtx3;
     pos[1] += -0.5 + offset;
     break;
   case 4:
-    rotateX(-M_PI / 2, frommtx);
+    rot = &rotmtx4;
     pos[1] += 0.5 - offset;
     break;
   }
 
-  Matrix3d rotmtx;
-  for (int i = 0; i < 3; i++)
-    for (int j = 0; j < 3; j++) rotmtx[i][j] = frommtx[i][j];
-  generateSpikeVBO(data, idxs, nfacets, rotmtx, pos, primaryColor, secondaryColor, 0.7);
+  generateSpikeVBO(data, idxs, *rot, pos, primaryColor, secondaryColor, 0.7);
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, sizeof(data), data, GL_STATIC_DRAW);
-
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idxs), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void SideSpike::drawBuffers1(GLuint *idxbufs, GLuint *databufs) {
+void SideSpike::drawBuffers1(const GLuint *vaolist) const {
   glEnable(GL_CULL_FACE);
   glDisable(GL_BLEND);
 
   const int nfacets = 6;
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0],
-              specularColor[1], specularColor[2], specularColor[3]);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 128.f / 128.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 1., Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, (3 * 3 * nfacets), GL_UNSIGNED_SHORT, (void *)0);
 }
-void SideSpike::drawBuffers2(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void SideSpike::drawBuffers2(const GLuint * /*vaolist*/) const {}
 
 void SideSpike::tick(Real t) {
-  double dist, h;
+  Animated::tick(t);
 
   if (is_on) phase += t * this->speed;
 
   double z = position[2];
   while (phase > 1.0) phase -= 1.0;
-  h = -0.3;
+  double h = -0.3;
   if (phase < 0.4) {
     /* spike is low */
     z = h;  //
@@ -146,8 +134,9 @@ void SideSpike::tick(Real t) {
 
   // play a 'rising' sound if the ball is in the round
   Player *player = Game::current->player1;
-  dist = sqrt((position[0] - player->position[0]) * (position[0] - player->position[0]) +
-              (position[1] - player->position[1]) * (position[1] - player->position[1]));
+  double dist =
+      sqrt((position[0] - player->position[0]) * (position[0] - player->position[0]) +
+           (position[1] - player->position[1]) * (position[1] - player->position[1]));
   if ((dist < 9.) && (!soundDone) && (phase >= 0.4) && (phase < 0.5)) {
     if (dist < 6.)
       playEffect(SFX_SPIKE, 0.66);
@@ -162,13 +151,10 @@ void SideSpike::tick(Real t) {
   double od = 0., aod;  // lateral distance to the spike
   int ol = 0;           // true if possible bounce
 
-  const std::set<Animated *> &balls = Game::current->balls->bboxOverlapsWith(this);
-  std::set<Animated *>::iterator iter = balls.begin();
-  std::set<Animated *>::iterator end = balls.end();
-  for (; iter != end; iter++) {
-    Ball *ball = (Ball *)*iter;
-
-    if (!ball->alive) continue;
+  Animated **balls;
+  int nballs = Game::current->balls->bboxOverlapsWith(this, &balls);
+  for (int i = 0; i < nballs; i++) {
+    Ball *ball = (Ball *)balls[i];
 
     switch (side) {
     case 1:
@@ -220,8 +206,8 @@ void SideSpike::tick(Real t) {
         ol = 1;
       break;
     }
-    dist = sqrt((sx - ball->position[0]) * (sx - ball->position[0]) +
-                (sy - ball->position[1]) * (sy - ball->position[1]));
+    double dist = sqrt((sx - ball->position[0]) * (sx - ball->position[0]) +
+                       (sy - ball->position[1]) * (sy - ball->position[1]));
 
     // too far
     if (dist > ball->radius + 1. + 0.3) continue;
