@@ -26,11 +26,11 @@
 
 class SideSpike : public Animated {
  public:
-  SideSpike(Coord3d position, Real speed, Real phase, int side);
+  SideSpike(const Coord3d& position, Real speed, Real phase, int side);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(Real t);
 
