@@ -1,5 +1,5 @@
-/* template.cc
-   Use to create new .cc project files
+/* sign.cc
+   Show rotating text at a point
 
    Copyright (C) 2000  Mathias Broxvall
 
@@ -27,7 +27,8 @@
 
 #define SIGN_SCALE 0.007
 
-Sign::Sign(const char *string, Real l, Real s, Real r, Coord3d pos) {
+Sign::Sign(const char *string, Real l, Real s, Real r, const Coord3d &pos)
+    : Animated(Role_OtherAnimated, 1) {
   position = pos;
   if (l <= 0.0) l = 1e10;
   life = l;
@@ -35,17 +36,11 @@ Sign::Sign(const char *string, Real l, Real s, Real r, Coord3d pos) {
   rotation = r;
   tot_rot = 0.0;
   if (Game::current->isNight) {
-    primaryColor[0] = 1.0;
-    primaryColor[1] = 1.0;
-    primaryColor[2] = 1.0;
+    primaryColor = Color(1., 1., 1., 1.);
   } else if (Game::current->fogThickness) {
-    primaryColor[0] = 0.0;
-    primaryColor[1] = 0.0;
-    primaryColor[2] = 0.0;
+    primaryColor = Color(0., 0., 0., 1.);
   } else {
-    primaryColor[0] = 1.0;
-    primaryColor[1] = 1.0;
-    primaryColor[2] = 1.0;
+    primaryColor = Color(1., 1., 1., 1.);
   }
 
   textimg = 0;
@@ -57,11 +52,10 @@ Sign::~Sign() {
 }
 
 void Sign::mkTexture(const char *string) {
-  SDL_Surface *text;
   SDL_Color fgColor = {255, 255, 255, 255};
 
   if (textimg) glDeleteTextures(1, &textimg);
-  text = TTF_RenderUTF8_Blended(ingameFont, string, fgColor);
+  SDL_Surface *text = TTF_RenderUTF8_Blended(ingameFont, string, fgColor);
   textimg = LoadTexture(text, texcoord);
   width = text->w;
   height = text->h;
@@ -76,9 +70,8 @@ void Sign::mkTexture(const char *string) {
   boundingBox[1][2] = SIGN_SCALE * scale * height;
 }
 
-int Sign::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  allocateBuffers(1, idxbufs, databufs);
-
+void Sign::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                           const GLuint *vaolist, bool /*mustUpdate*/) const {
   GLfloat flat[3] = {0.f, 0.f, 0.f};
 
   GLfloat data[8 * 8];
@@ -87,8 +80,8 @@ int Sign::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
   GLfloat dy = std::sin(M_PI * (tot_rot - 45.) / 180.) * SIGN_SCALE * scale * width;
   GLfloat dz = SIGN_SCALE * scale * height;
 
-  GLfloat color[4] = {primaryColor[0], primaryColor[1], primaryColor[2],
-                      std::min((GLfloat)life, 1.0f)};
+  Color color = primaryColor.toOpaque();
+  color.v[3] = std::min(1.0, life) * 65535;
 
   pos += packObjectVertex(pos, position[0] + dx, position[1] + dy, position[2] + dz,
                           texcoord[0], texcoord[1], color, flat);
@@ -108,19 +101,18 @@ int Sign::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
   pos += packObjectVertex(pos, position[0] + dx, position[1] + dy, position[2] - dz,
                           texcoord[0] + texcoord[2], texcoord[1] + texcoord[3], color, flat);
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 8 * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
-
   ushort idxs[4][3] = {{0, 1, 2}, {1, 3, 2}, {4, 5, 6}, {5, 7, 6}};
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, 12 * sizeof(ushort), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void Sign::drawBuffers1(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void Sign::drawBuffers1(const GLuint * /*vaolist*/) const {}
 
-void Sign::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
+void Sign::drawBuffers2(const GLuint *vaolist) const {
   if (activeView.calculating_shadows) return;
 
   // Keep the depth function on but trivial so as to record depth values
@@ -129,21 +121,19 @@ void Sign::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
   glDepthFunc(GL_ALWAYS);
 
   // Transfer data
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), 0., 0., 0., 1.);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 0.);
-  glUniform1f(glGetUniformLocation(shaderObject, "use_lighting"), -1.);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(Color(0., 0., 0., 1.), 0., Lighting_None);
   glBindTexture(GL_TEXTURE_2D, textimg);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 12, GL_UNSIGNED_SHORT, (void *)0);
 
   glDepthFunc(GL_LEQUAL);
 }
 
 void Sign::tick(Real t) {
+  Animated::tick(t);
+
   life -= t;
   if (life <= 0) {
     /* time to die... */
