@@ -25,13 +25,13 @@
 
 class Sign : public Animated {
  public:
-  Sign(const char* text, Real life, Real scale, Real rotation, Coord3d pos);
+  Sign(const char* text, Real life, Real scale, Real rotation, const Coord3d& pos);
   virtual ~Sign();
   void mkTexture(const char*);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(Real);
 
