@@ -28,7 +28,8 @@
 #include "player.h"
 #include "sound.h"
 
-Spike::Spike(Coord3d position, Real speed, Real phase) {
+Spike::Spike(const Coord3d &position, Real speed, Real phase)
+    : Animated(Role_OtherAnimated, 1) {
   this->position = position;
   this->position[2] = Game::current->map->getHeight(position[0], position[1]) + 0.0;
   this->speed = speed;
@@ -36,69 +37,52 @@ Spike::Spike(Coord3d position, Real speed, Real phase) {
   this->phase = phase;
   this->soundDone = 0;
 
-  primaryColor[0] = 0.9;
-  primaryColor[1] = 0.5;
-  primaryColor[2] = 0.4;
-
-  secondaryColor[0] = 0.9;
-  secondaryColor[1] = 0.8;
-  secondaryColor[2] = 0.5;
-
-  specularColor[0] = 0.1;
-  specularColor[1] = 0.1;
-  specularColor[2] = 0.1;
+  primaryColor = Color(0.9, 0.5, 0.4, 1.0);
+  secondaryColor = Color(0.9, 0.8, 0.5, 1.0);
+  specularColor = Color(0.1, 0.1, 0.1, 1.0);
 }
 
-int Spike::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  allocateBuffers(1, idxbufs, databufs);
-
+void Spike::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                            const GLuint *vaolist, bool /*mustUpdate*/) const {
   const int nfacets = 6;
   GLfloat data[(4 * nfacets) * 8];
   ushort idxs[3 * nfacets][3];
 
-  Matrix3d rotmtx;
-  Matrix4d frommtx;
-  identityMatrix(frommtx);
-  for (int i = 0; i < 3; i++)
-    for (int j = 0; j < 3; j++) rotmtx[i][j] = frommtx[i][j];
-  generateSpikeVBO(data, idxs, nfacets, rotmtx, position, primaryColor, secondaryColor, 2.0);
+  Matrix3d rotmtx = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
+  generateSpikeVBO(data, idxs, rotmtx, position, primaryColor, secondaryColor, 2.0);
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, sizeof(data), data, GL_STATIC_DRAW);
-
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idxs), idxs, GL_STATIC_DRAW);
-
-  return 1;
+  configureObjectAttributes();
 }
 
-void Spike::drawBuffers1(GLuint *idxbufs, GLuint *databufs) {
+void Spike::drawBuffers1(const GLuint *vaolist) const {
   glEnable(GL_CULL_FACE);
   glDisable(GL_BLEND);
 
   const int nfacets = 6;
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0],
-              specularColor[1], specularColor[2], specularColor[3]);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 128.f / 128.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 1., Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, (3 * 3 * nfacets), GL_UNSIGNED_SHORT, (void *)0);
 }
-void Spike::drawBuffers2(GLuint * /*idxbufs*/, GLuint * /*databufs*/) {}
+void Spike::drawBuffers2(const GLuint * /*vaolist*/) const {}
 
 void Spike::tick(Real t) {
-  double dist, dx, dy, speed, h;
+  Animated::tick(t);
+
   int is_sinking = 0;
 
   double x = position[0], y = position[1], z = position[2];
   if (is_on) phase += t * this->speed;
   while (phase > 1.0) phase -= 1.0;
-  h = Game::current->map->getHeight(x, y) - 0.3;
+  double h = Game::current->map->getHeight(x, y) - 0.3;
   if (phase < 0.4) {
     /* spike is low */
     z = h;  //
@@ -116,16 +100,16 @@ void Spike::tick(Real t) {
   }
   position[2] = z;
 
-  const std::set<Animated *> &balls = Game::current->balls->bboxOverlapsWith(this);
-  std::set<Animated *>::iterator iter = balls.begin();
-  std::set<Animated *>::iterator end = balls.end();
-  for (; iter != end; iter++) {
-    Ball *ball = (Ball *)*iter;
-    if (ball->alive && ball->position[0] > x - 1 && ball->position[0] < x + 1 &&
-        ball->position[1] > y - 1 && ball->position[1] < y + 1) {
-      dx = ball->position[0] - x;
-      dy = ball->position[1] - y;
-      dist = sqrt(dx * dx + dy * dy);
+  Animated **balls;
+  int nballs = Game::current->balls->bboxOverlapsWith(this, &balls);
+  for (int i = 0; i < nballs; i++) {
+    Ball *ball = (Ball *)balls[i];
+
+    if (ball->position[0] > x - 1 && ball->position[0] < x + 1 && ball->position[1] > y - 1 &&
+        ball->position[1] < y + 1) {
+      double dx = ball->position[0] - x;
+      double dy = ball->position[1] - y;
+      double dist = sqrt(dx * dx + dy * dy);
 
       if (dist < 0.1 + ball->radius) {
         h = ball->position[2] + dist;
@@ -133,10 +117,10 @@ void Spike::tick(Real t) {
           /* Bounce against it */
           dx /= dist;
           dy /= dist;
-          speed = dx * ball->velocity[0] + dy * ball->velocity[1];
-          if (speed < 0) {
-            ball->velocity[0] -= 2 * speed * dx;
-            ball->velocity[1] -= 2 * speed * dy;
+          double bounce_speed = dx * ball->velocity[0] + dy * ball->velocity[1];
+          if (bounce_speed < 0) {
+            ball->velocity[0] -= 2 * bounce_speed * dx;
+            ball->velocity[1] -= 2 * bounce_speed * dy;
           }
         } else if (h < 0.4 + z)
           /* Die */
@@ -147,8 +131,9 @@ void Spike::tick(Real t) {
 
   // play a 'rising' sound if the ball is in the round
   Player *player = Game::current->player1;
-  dist = sqrt((position[0] - player->position[0]) * (position[0] - player->position[0]) +
-              (position[1] - player->position[1]) * (position[1] - player->position[1]));
+  double dist =
+      sqrt((position[0] - player->position[0]) * (position[0] - player->position[0]) +
+           (position[1] - player->position[1]) * (position[1] - player->position[1]));
   if ((dist < 9.) && (!soundDone) && (phase >= 0.4) && (phase < 0.5)) {
     if (dist < 6.)
       playEffect(SFX_SPIKE, 0.66);
@@ -158,45 +143,47 @@ void Spike::tick(Real t) {
   }
 }
 
-void generateSpikeVBO(GLfloat *data, ushort idxs[][3], int nfacets, Matrix3d rotmtx,
-                      Coord3d position, GLfloat sidec[4], GLfloat tipc[4], GLfloat length) {
+void generateSpikeVBO(GLfloat *data, ushort idxs[][3], Matrix3d rotmtx,
+                      const Coord3d &position, Color sidec, Color tipc, GLfloat length) {
+  const int nfacets = 6;
   char *pos = (char *)data;
 
   double d1 = 1 / sqrt(10.0), d2 = 3 / sqrt(10.0);
   for (int i = 0; i < 4 * nfacets; i++) {
     Coord3d local;
     Coord3d normal;
+    bool is_tip;
     GLfloat *color = NULL;
 
     int step = i / nfacets;
-    double angle = (i % nfacets) * 2. * M_PI / nfacets;
-    local[0] = 0.1 * std::sin(angle);
-    local[1] = 0.1 * std::cos(angle);
+    int j = i % nfacets;
+    local[0] = 0.1 * sin6[j];
+    local[1] = 0.1 * cos6[j];
     if (step == 0) {
-      color = tipc;
+      is_tip = true;
       local[2] = 0.;
-      normal[0] = d2 * std::sin(angle);
-      normal[1] = d2 * std::cos(angle);
+      normal[0] = d2 * sin6[j];
+      normal[1] = d2 * cos6[j];
       normal[2] = d1;
     } else if (step == 1) {
-      color = sidec;
+      is_tip = false;
       local[2] = 0;
-      normal[0] = std::sin(angle);
-      normal[1] = std::cos(angle);
+      normal[0] = sin6[j];
+      normal[1] = cos6[j];
       normal[2] = 0.;
     } else if (step == 2) {
-      color = sidec;
+      is_tip = false;
       local[2] = -length;
-      normal[0] = std::sin(angle);
-      normal[1] = std::cos(angle);
+      normal[0] = sin6[j];
+      normal[1] = cos6[j];
       normal[2] = 0.;
     } else {
-      color = tipc;
+      is_tip = true;
       local[0] = 0.;
       local[1] = 0.;
       local[2] = 0.3;
-      normal[0] = d2 * std::sin(angle + M_PI / nfacets);
-      normal[1] = d2 * std::cos(angle + M_PI / nfacets);
+      normal[0] = d2 * sin12[2 * j + 1];
+      normal[1] = d2 * cos12[2 * j + 1];
       normal[2] = d1;
     }
     Coord3d tlocal = useMatrix(rotmtx, local);
@@ -204,7 +191,7 @@ void generateSpikeVBO(GLfloat *data, ushort idxs[][3], int nfacets, Matrix3d rot
     GLfloat flocal[3] = {(GLfloat)tlocal[0], (GLfloat)tlocal[1], (GLfloat)tlocal[2]};
     GLfloat fnormal[3] = {(GLfloat)tnormal[0], (GLfloat)tnormal[1], (GLfloat)tnormal[2]};
     pos += packObjectVertex(pos, position[0] + flocal[0], position[1] + flocal[1],
-                            position[2] + flocal[2], 0., 0., color, fnormal);
+                            position[2] + flocal[2], 0., 0., is_tip ? tipc : sidec, fnormal);
   }
 
   for (int i = 0; i < nfacets; i++) {
