@@ -23,15 +23,14 @@
 #define SPIKE_H
 
 #include "animated.h"
-#include "glHelp.h"
 
 class Spike : public Animated {
  public:
-  Spike(Coord3d position, Real speed, Real phase);
+  Spike(const Coord3d& position, Real speed, Real phase);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   void tick(Real t);
 
@@ -41,7 +40,7 @@ class Spike : public Animated {
   int soundDone;
 };
 
-void generateSpikeVBO(GLfloat* data, ushort idxs[][3], int nfacets, Matrix3d rotmtx,
-                      Coord3d position, GLfloat sidec[4], GLfloat tipc[4], GLfloat length);
+void generateSpikeVBO(GLfloat* data, ushort idxs[][3], Matrix3d rotmtx,
+                      const Coord3d& position, Color sidec, Color tipc, GLfloat length);
 
 #endif
