@@ -20,14 +20,15 @@
 
 #include "splash.h"
 
+#include "game.h"
 #include "settings.h"
 
-Splash::Splash(Coord3d center, Coord3d velocity, GLfloat color[4], double strength,
-               double radius) {
+Splash::Splash(Coord3d center, Coord3d velocity, Color color, double strength, double radius)
+    : Animated(Role_OtherAnimated, 1) {
   timeLeft = 3.0;
   nDroplets = (int)strength;
   if (nDroplets > 32) nDroplets = 32;
-  for (int i = 0; i < 4; i++) primaryColor[i] = color[i];
+  primaryColor = color;
   for (int i = 0; i < nDroplets; i++) {
     positions[i][0] = center[0] + (frandom() - 0.5) * 2.0 * radius;
     positions[i][1] = center[1] + (frandom() - 0.5) * 2.0 * radius;
@@ -38,10 +39,9 @@ Splash::Splash(Coord3d center, Coord3d velocity, GLfloat color[4], double streng
   }
 }
 
-int Splash::generateBuffers(GLuint*& idxbufs, GLuint*& databufs) {
-  if (Settings::settings->gfx_details <= GFX_DETAILS_SIMPLE) return 0;
-
-  allocateBuffers(1, idxbufs, databufs);
+void Splash::generateBuffers(const GLuint* idxbufs, const GLuint* databufs,
+                             const GLuint* vaolist, bool /*mustUpdate*/) const {
+  if (Settings::settings->gfx_details <= GFX_DETAILS_SIMPLE) return;
 
   GLfloat* data = new GLfloat[3 * nDroplets];
   ushort* idxs = new ushort[nDroplets];
@@ -52,35 +52,36 @@ int Splash::generateBuffers(GLuint*& idxbufs, GLuint*& databufs) {
     idxs[i] = i;
   }
 
+  glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
   glBufferData(GL_ARRAY_BUFFER, 3 * nDroplets * sizeof(GLfloat), data, GL_STATIC_DRAW);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
   glBufferData(GL_ELEMENT_ARRAY_BUFFER, nDroplets * sizeof(ushort), idxs, GL_STATIC_DRAW);
+  glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, (void*)0);
+  glEnableVertexAttribArray(0);
   delete[] data;
   delete[] idxs;
-
-  return 1;
 }
 
-void Splash::drawBuffers1(GLuint* /*idxbufs*/, GLuint* /*databufs*/) {}
+void Splash::drawBuffers1(const GLuint* /*vaolist*/) const {}
 
-void Splash::drawBuffers2(GLuint* idxbufs, GLuint* databufs) {
+void Splash::drawBuffers2(const GLuint* vaolist) const {
   if (Settings::settings->gfx_details <= GFX_DETAILS_SIMPLE) return;
+  if (activeView.calculating_shadows) return;
 
   glEnable(GL_BLEND);
 
   glPointSize(1.5 * screenWidth / 600.);
-  setActiveProgramAndUniforms(shaderLine);
-  glUniform4f(glGetUniformLocation(shaderLine, "line_color"), primaryColor[0], primaryColor[1],
-              primaryColor[2], primaryColor[3]);
+  setActiveProgramAndUniforms(Shader_Line);
+  glUniformC(uniformLocations.line_color, primaryColor);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, (void*)0);
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_POINTS, nDroplets, GL_UNSIGNED_SHORT, (void*)0);
 }
 
 void Splash::tick(Real t) {
+  Animated::tick(t);
+
   timeLeft -= t;
   if (t < 0.0) remove();
   for (int i = 0; i < nDroplets; i++) {
