@@ -25,12 +25,12 @@
 
 class Splash : public Animated {
  public:
-  Splash(Coord3d center, Coord3d velocity, GLfloat color[4], double strength, double radius);
+  Splash(Coord3d center, Coord3d velocity, Color color, double strength, double radius);
   void tick(Real t);
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
  private:
   int nDroplets;
