@@ -27,15 +27,16 @@ class CSwitch : public Animated {
  public:
   CSwitch(Real x, Real y, SCM on, SCM off);
   virtual ~CSwitch();
+  virtual void releaseCallbacks();
 
-  virtual int generateBuffers(GLuint*& idxbufs, GLuint*& databufs);
-  virtual void drawBuffers1(GLuint* idxbufs, GLuint* databufs);
-  virtual void drawBuffers2(GLuint* idxbufs, GLuint* databufs);
+  virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
+  virtual void drawBuffers1(const GLuint*) const;
+  virtual void drawBuffers2(const GLuint*) const;
 
   virtual void tick(Real);
 
  private:
-  int is_touched;
+  bool is_touched;
   SCM on, off;
 };
 
