@@ -26,8 +26,11 @@
 #include "player.h"
 
 #define NFACETS 14
+#define sinN sin14
+#define cosN cos14
 
-Teleport::Teleport(Real x, Real y, Real dx, Real dy, Real radius) {
+Teleport::Teleport(Real x, Real y, Real dx, Real dy, Real radius)
+    : Animated(Role_OtherAnimated, 2) {
   this->x = x;
   this->y = y;
   this->dx = dx;
@@ -36,15 +39,9 @@ Teleport::Teleport(Real x, Real y, Real dx, Real dy, Real radius) {
   position[0] = x;
   position[1] = y;
   position[2] = Game::current->map->getHeight(position[0], position[1]);
-  primaryColor[0] = .5;
-  primaryColor[1] = .7;
-  primaryColor[2] = .6;
-  primaryColor[3] = 1.0;
-  secondaryColor[0] = 1.0;
-  secondaryColor[1] = 0.9;
-  secondaryColor[2] = 0.4;
-  secondaryColor[3] = 1.0;
-  is_on = 1;
+  primaryColor = Color(0.5, 0.7, 0.6, 1.0);
+  secondaryColor = Color(1.0, 0.9, 0.4, 1.0);
+  is_on = true;
   boundingBox[0][0] = -2 * radius;
   boundingBox[1][0] = 2 * radius;
   boundingBox[0][1] = -2 * radius;
@@ -53,11 +50,10 @@ Teleport::Teleport(Real x, Real y, Real dx, Real dy, Real radius) {
   boundingBox[1][2] = 2 * radius + 0.5;
 }
 
-int Teleport::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
-  allocateBuffers(2, idxbufs, databufs);
-
+void Teleport::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
+                               const GLuint *vaolist, bool mustUpdate) const {
   GLfloat cent_height = 0.5f;
-  {
+  if (mustUpdate) {
     GLfloat data[(9 * NFACETS + 1) * 8];
     ushort idxs[9 * NFACETS][3];
 
@@ -68,44 +64,40 @@ int Teleport::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
     // Draw torus
     char *pos = (char *)data;
     for (int i = 0; i < NFACETS; i++) {
-      GLfloat angle = 2 * i * M_PI / NFACETS;
       GLfloat nnormal[3] = {0.f, 1.f, 0.f};
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * irad, position[1] + width,
-                              position[2] + std::cos(angle) * irad + cent_height, 0., 0.,
-                              primaryColor, nnormal);
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * orad, position[1] + width,
-                              position[2] + std::cos(angle) * orad + cent_height, 0., 0.,
-                              primaryColor, nnormal);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * irad, position[1] + width,
+                              position[2] + cosN[i] * irad + cent_height, 0., 0., primaryColor,
+                              nnormal);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * orad, position[1] + width,
+                              position[2] + cosN[i] * orad + cent_height, 0., 0., primaryColor,
+                              nnormal);
     }
     for (int i = 0; i < NFACETS; i++) {
-      GLfloat angle = 2 * i * M_PI / NFACETS;
       GLfloat snormal[3] = {0.f, -1.f, 0.f};
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * irad, position[1] - width,
-                              position[2] + std::cos(angle) * irad + cent_height, 0., 0.,
-                              primaryColor, snormal);
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * orad, position[1] - width,
-                              position[2] + std::cos(angle) * orad + cent_height, 0., 0.,
-                              primaryColor, snormal);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * irad, position[1] - width,
+                              position[2] + cosN[i] * irad + cent_height, 0., 0., primaryColor,
+                              snormal);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * orad, position[1] - width,
+                              position[2] + cosN[i] * orad + cent_height, 0., 0., primaryColor,
+                              snormal);
     }
     for (int i = 0; i < NFACETS; i++) {
-      GLfloat angle = 2 * i * M_PI / NFACETS;
-      GLfloat inormal[3] = {-std::sin(angle), -std::cos(angle), 0.f};
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * irad, position[1] - width,
-                              position[2] + std::cos(angle) * irad + cent_height, 0., 0.,
-                              primaryColor, inormal);
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * irad, position[1] + width,
-                              position[2] + std::cos(angle) * irad + cent_height, 0., 0.,
-                              primaryColor, inormal);
+      GLfloat inormal[3] = {(GLfloat)-sinN[i], (GLfloat)-cosN[i], 0.f};
+      pos += packObjectVertex(pos, position[0] + sinN[i] * irad, position[1] - width,
+                              position[2] + cosN[i] * irad + cent_height, 0., 0., primaryColor,
+                              inormal);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * irad, position[1] + width,
+                              position[2] + cosN[i] * irad + cent_height, 0., 0., primaryColor,
+                              inormal);
     }
     for (int i = 0; i < NFACETS; i++) {
-      GLfloat angle = 2 * i * M_PI / NFACETS;
-      GLfloat onormal[3] = {std::sin(angle), std::cos(angle), 0.f};
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * orad, position[1] - width,
-                              position[2] + std::cos(angle) * orad + cent_height, 0., 0.,
-                              primaryColor, onormal);
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * orad, position[1] + width,
-                              position[2] + std::cos(angle) * orad + cent_height, 0., 0.,
-                              primaryColor, onormal);
+      GLfloat onormal[3] = {(GLfloat)sinN[i], (GLfloat)cosN[i], 0.f};
+      pos += packObjectVertex(pos, position[0] + sinN[i] * orad, position[1] - width,
+                              position[2] + cosN[i] * orad + cent_height, 0., 0., primaryColor,
+                              onormal);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * orad, position[1] + width,
+                              position[2] + cosN[i] * orad + cent_height, 0., 0., primaryColor,
+                              onormal);
     }
 
     for (int k = 0; k < 4; k++) {
@@ -126,13 +118,12 @@ int Teleport::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
     }
 
     // Draw base
-    GLfloat white[4] = {1.f, 1.f, 1.f, 1.f};
+    Color white(1., 1., 1., 1.);
     for (int i = 0; i < NFACETS; i++) {
-      GLfloat angle = 2 * i * M_PI / NFACETS;
-      GLfloat inormal[3] = {std::cos(angle), std::sin(angle), 0.4f};
-      pos += packObjectVertex(pos, position[0] + radius * std::cos(angle),
-                              position[1] + radius * std::sin(angle), position[2] + 0.1f, 0.,
-                              0., white, inormal);
+      GLfloat inormal[3] = {(GLfloat)cosN[i], (GLfloat)sinN[i], 0.4f};
+      pos +=
+          packObjectVertex(pos, position[0] + radius * cosN[i], position[1] + radius * sinN[i],
+                           position[2] + 0.1f, 0., 0., white, inormal);
     }
     GLfloat vnormal[3] = {0.f, 0.f, 1.f};
     pos += packObjectVertex(pos, position[0], position[1], position[2] + 0.3, 0., 0., white,
@@ -144,15 +135,16 @@ int Teleport::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
       idxs[8 * NFACETS + i][2] = 8 * NFACETS + (i + 1) % NFACETS;
     }
 
+    glBindVertexArray(vaolist[0]);
     glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
     glBufferData(GL_ARRAY_BUFFER, sizeof(data), data, GL_STATIC_DRAW);
-
     glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
     glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idxs), idxs, GL_STATIC_DRAW);
+    configureObjectAttributes();
   }
   {
-    GLfloat color[4] = {secondaryColor[0], secondaryColor[1], secondaryColor[2],
-                        secondaryColor[3] * (0.4f + 0.2f * (GLfloat)frandom())};
+    Color color = secondaryColor;
+    color.v[3] *= (0.4f + 0.2f * (GLfloat)frandom());
     GLfloat flat[3] = {0., 0., 0.};
     GLfloat rad = (4.0 / 3.0) * radius;
 
@@ -162,61 +154,52 @@ int Teleport::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
     pos += packObjectVertex(pos, position[0], position[1], position[2] + cent_height, 0., 0.,
                             color, flat);
     for (int i = 0; i < NFACETS; i++) {
-      GLfloat angle = 2 * i * M_PI / NFACETS;
-
-      pos += packObjectVertex(pos, position[0] + std::sin(angle) * rad, position[1],
-                              position[2] + std::cos(angle) * rad + cent_height, 0., 0., color,
-                              flat);
+      pos += packObjectVertex(pos, position[0] + sinN[i] * rad, position[1],
+                              position[2] + cosN[i] * rad + cent_height, 0., 0., color, flat);
       idxs[i][0] = 0;
       idxs[i][1] = i + 1;
       idxs[i][2] = (i + 1) % NFACETS + 1;
     }
 
+    glBindVertexArray(vaolist[1]);
     glBindBuffer(GL_ARRAY_BUFFER, databufs[1]);
     glBufferData(GL_ARRAY_BUFFER, (NFACETS + 1) * 8 * sizeof(GLfloat), data, GL_STATIC_DRAW);
     glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[1]);
     glBufferData(GL_ELEMENT_ARRAY_BUFFER, NFACETS * 3 * sizeof(ushort), idxs, GL_STATIC_DRAW);
+    configureObjectAttributes();
   }
-
-  return 2;
 }
 
-void Teleport::drawBuffers1(GLuint *idxbufs, GLuint *databufs) {
+void Teleport::drawBuffers1(const GLuint *vaolist) const {
   glEnable(GL_CULL_FACE);
   glDisable(GL_BLEND);
 
   // Transfer
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), specularColor[0],
-              specularColor[1], specularColor[2], specularColor[3]);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 15.f / 128.f);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(specularColor, 0.12, Lighting_Regular);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[0]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 3 * 9 * NFACETS, GL_UNSIGNED_SHORT, (void *)0);
 }
 
-void Teleport::drawBuffers2(GLuint *idxbufs, GLuint *databufs) {
+void Teleport::drawBuffers2(const GLuint *vaolist) const {
   if (activeView.calculating_shadows) return;
 
   glDisable(GL_CULL_FACE);
   glEnable(GL_BLEND);
 
-  setActiveProgramAndUniforms(shaderObject);
-  glUniform4f(glGetUniformLocation(shaderObject, "specular"), 0., 0., 0., 1.);
-  glUniform1f(glGetUniformLocation(shaderObject, "shininess"), 0.);
-  glUniform1f(glGetUniformLocation(shaderObject, "use_lighting"), -1.);
-  glBindTexture(GL_TEXTURE_2D, textures[loadTexture("blank.png")]);
+  setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(Color(0., 0., 0., 1.), 0., Lighting_None);
+  glBindTexture(GL_TEXTURE_2D, textureBlank);
 
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxbufs[1]);
-  glBindBuffer(GL_ARRAY_BUFFER, databufs[1]);
-  configureObjectAttributes();
+  glBindVertexArray(vaolist[1]);
   glDrawElements(GL_TRIANGLES, NFACETS * 3, GL_UNSIGNED_SHORT, (void *)0);
 }
 
-void Teleport::tick(Real /*t*/) {
+void Teleport::tick(Real t) {
+  Animated::tick(t);
+
   position[2] = Game::current->map->getHeight(position[0], position[1]);
   Player *p = Game::current->player1;
   Coord3d diff = position - p->position;
