@@ -23,8 +23,10 @@
 #include "game.h"
 #include "map.h"
 
-Trampoline::Trampoline(int x, int y) : GameHook(), cx(x), cy(y) {}
+Trampoline::Trampoline(int x, int y) : GameHook(Role_GameHook), cx(x), cy(y) {}
 void Trampoline::tick(Real t) {
+  GameHook::tick(t);
+
   Map* map = Game::current->map;
   Cell& c = map->cell(cx, cy);
   Real dh = std::min(0.5 * t, c.sunken - 1e-5);
@@ -36,5 +38,5 @@ void Trampoline::tick(Real t) {
     for (int i = 0; i < 5; i++) c.heights[i] += c.sunken;
     c.sunken = 0.;
   }
-  map->markCellsUpdated(cx, cy, cx, cy, 1);
+  map->markCellsUpdated(cx, cy, cx, cy, true);
 }
