@@ -7,12 +7,12 @@
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -21,12 +21,15 @@
 #ifndef TRAMPOLINE_H
 #define TRAMPOLINE_H
 
+#include "gameHook.h"
+
 class Trampoline : public GameHook {
  public:
-  Trampoline(Cell&);
+  Trampoline(int x, int y);
   virtual void tick(Real);
+
  private:
-  Cell& cell;
+  int cx, cy;
 };
 
 #endif
