@@ -25,16 +25,20 @@
 #include "player.h"
 
 Trigger::Trigger(Real x, Real y, Real radius, SCM expr)
-    : GameHook(), x(x), y(y), radius(radius), expr(expr) {
-  scm_gc_protect_object(expr);
+    : GameHook(Role_GameHook), x(x), y(y), radius(radius), expr(expr) {
+  if (expr) scm_gc_protect_object(expr);
 }
-Trigger::~Trigger() {
-  scm_gc_unprotect_object(expr);
-  this->GameHook::~GameHook();
+Trigger::~Trigger() { Trigger::releaseCallbacks(); }
+void Trigger::releaseCallbacks() {
+  if (expr) scm_gc_unprotect_object(expr);
+  expr = NULL;
 }
-void Trigger::tick(Real /*t*/) {
+void Trigger::tick(Real t) {
+  GameHook::tick(t);
+
+  if (!is_on) return;
   Player *ply = Game::current->player1;
   double dx = ply->position[0] - x;
   double dy = ply->position[1] - y;
-  if (dx * dx + dy * dy < radius * radius) scm_catch_apply_0(expr);
+  if (dx * dx + dy * dy < radius * radius) Game::current->queueCall(expr);
 }
